# stories_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

# +EpicsController+ allows users to work with +Epic+ stories.
class EpicsController < ApplicationController
  before_filter :authenticated,      :except => [:index, :show]
  before_filter :load_project,       :only => [:new, :create]
  before_filter :load_epic,          :except => [:index, :new, :create]
  before_filter :verify_can_create,  :only => [:new, :create]
  before_filter :verify_can_comment, :only => [:comment]
  before_filter :verify_can_edit,    :only => [:edit, :update]
  before_filter :verify_can_delete,  :only => [:destroy]
  before_filter :verify_can_close,   :only => [:close]
  before_filter :verify_can_reopen,  :only => [:reopen]
  before_filter :load_comment,       :only => [:comment]
  before_filter :path_to_list,       :only => [:index, :new, :create]
  before_filter :path_to_one,        :only => [:show, :edit, :update]

  # GET /epics
  def index
    @title = "Epic Stories"
    @title = "Epic Stories For #{@project.name}" if @project
    @project = Project.find_by_id(params[:project])
    @epics = Epic.for_project(@project).paginate(:page     => params[:page],
                                                         :per_page => 10)
    respond_to do |format|
      format.html
    end
  end

  # GET /epics/1
  def show
    @title = "Epic:#{@epic.title}"
    @comment = Message.new if @epic.can_comment?(@user)
  end

  # GET /epics/new
  def new
    add_breadcrumb "New"
    @title = "Create new epic for #{@project.name}"
    @epic = Epic.new(:project => @project)
  end

  # GET /epics/1/edit
  def edit
    add_breadcrumb "Edit"
    @title = "Edit epic for #{@project.name}"
  end

  # POST /epics
  def create
    add_breadcrumb "New"
    respond_to do |format|
      Epic.transaction do
        @epic = Epic.new(params[:epic])
        @epic.project = @project
        if @epic.save
          flash[:message] = "Epic successfully created."
          format.html { redirect_to epics_path(:project => @project) }
        else
          @title = "Create new epic for #{@project.name}"
          @epic.valid?
          format.html { render :action => :edit }
        end
      end
    end
  end

  # PUT /epics/1
  def update
    add_breadcrumb "Edit"
    respond_to do |format|
      Epic.transaction do
        @epic.update_attributes(params[:epic])
        if @epic.save
          flash[:message] = "Epic was updated."
          format.html { redirect_to epic_path(@epic) }
        else
          @title = "Edit epic for #{@project.name}"
          @epic.valid?
          format.html { render :action => :edit }
        end
      end
    end
  end

  # DELETE /epics/1
  def destroy
    respond_to do |format|
      Epic.transaction do
        if @epic.destroy
          flash[:message] = "Epic was deleted."
          format.html { redirect_to epics_path(:project => @project) }
        else
          flash[:error] = "Unable to delete this epic."
          format.html {  redirect_to.epic_path(@epic) }
        end
      end
    end
  end

  # PUT /epics/1/close
  def close
    Epic.transaction do
      @epic.closed = true
      if @epic.save
        respond_to do |format|
          flash[:message] = "Epic is now marked as closed."
          format.html { redirect_to epics_path(:project => @project) }
        end
      else
        report_error "Unable to close this epic at this time."
      end
    end
  end

  # PUT /epics/1/reopen
  def reopen
    @epic.closed = false
    if @epic.save
      respond_to do |format|
        flash[:message] = "Epic now reopened."
        format.html { redirect_to epic_path(@epic) }
      end
    else
      report_error "You may not reopen this epic."
    end
  end

  # POST /epics/1/comment
  def comment
    Epic.transaction do
      respond_to do |format|
        @comment.author = @user
        @comment.posted = Time.current
        if @comment.valid?
          @epic.comments << @comment
          @epic.save!
          flash[:message] = "Comment saved."
          format.html {redirect_to epic_path(@epic)}
        else
          format.html {render :action => :show}
        end
      end
    end
  end

  private

  def load_project
    @project = Project.find_by_id(params[:project])
    unless @project
      report_error "Missing or invalid project id."
    end
  end

  def load_epic
    @epic = Epic.find_by_id(params[:id])
    @project = @epic.project if @epic
    report_error "Invalid or missing epic id." unless @epic
  end

  def verify_can_create
    report_error "You are not allowed to created epics for this project." unless @project.can_create_epics?(@user)
  end

  def verify_can_comment
    report_error("You are not allowed to comment on this epic.", epic_path(@epic)) unless @epic.can_comment?(@user)
  end

  def verify_can_edit
    report_error "You are not allowed to edit this epic." unless @epic.can_edit?(@user)
  end

  def verify_can_delete
    report_error "You are not allowed to delete this epic." unless @epic.can_delete?(@user)
  end

  def verify_can_close
    report_error "You are not allowed to close this epic." unless @epic.can_close?(@user)
  end

  def verify_can_reopen
    report_error "You are not allowed to reopen this epic." unless @epic.can_reopen?(@user)
  end

  def load_comment
    @comment = Message.new(params[:message])
  end

  def path_to_list
    if @project || params[:project]
      @project = Project.find_by_id(params[:project]) unless @project
      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{@project.id}", project_path(@project)
      add_breadcrumb "Epics", epics_path(:project => @project)
    else
      add_breadcrumb "Epics", epics_path
    end
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@epic.id}", epic_path(@epic)
  end
end
