# home_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +HomeController+ handles requests relating to authentication and showing a
# user their account overview.
#
# The +dashboard+ represents a high level view of the user's account.
#
# The login and logout accounts allow a user to create and destroy their
# authenticated user session.
#
class HomeController < ApplicationController
  before_filter :authenticated,
    :except => [:index, :error, :login, :register, :generate_new_password]

  def index
    @title = 'Home'
  end

  def error
    @title = 'Error'
  end

  # Attempts to authenticate the user. On success, the user is either redirected
  # to the page they were originally trying to access or else forwarded to their
  # dashboard.
  #
  # On failure, the user is reshown the login page.
  #
  def login
    session[:user_id] = nil

    # if the user was doing a login, then process the values submitted
    if request.post?
      user = User.authenticate(params[:email], params[:password])

      if user
        session[:user_id] = user.id
        uri = session[:original_uri]
        session[:original_uri] = nil

        redirect_to(uri || dashboard_url)
      else
        flash[:message] = 'Invalid username or password.'
      end
    end
  end

  # Destroys the users session and redirects them to the login page.
  #
  def logout
    session[:user_id] = nil

    redirect_to :action => :login
  end

  def dashboard
    @backlog = BacklogItem.find_all_by_state_and_owner_id(BacklogItem::STATE_ASSIGNED, session[:user_id])
  end

  # Allows the user to email him a new password.
  #
  def generate_new_password
    if request.post?
      user = User.find_by_email(params[:email])

      if user

        if UserMailer.deliver_new_generated_password(user, user.generate_new_password!)
          flash[:notice] = "Your new password has been sended to #{params[:email]}."
        else
          flash[:error] = "Error when sended password. Please contact the administrator."
        end
        redirect_to :action => 'login'

      else
        flash[:error] = "This user doesn't exist."
      end

    end
  end

end
