# items_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +ItemsController+ handles CRUD operations on instances of +BacklogItem+.
class ItemsController < ApplicationController
  before_filter :unsupported,             :only => [:edit, :update]
  before_filter :authenticated,           :except => [:index, :show]
  before_filter :load_sprint,             :only => [:new, :create]
  before_filter :load_backlog_item,       :except => [:index, :new, :create]
  before_filter :verify_can_delete,       :only => [:destroy]
  before_filter :verify_can_defer,        :only => [:defer]
  before_filter :verify_can_create,       :only => [:new, :create]
  before_filter :verify_sprint_is_active, :only => [:accept, :estimate, :drop, :complete, :reopen, :blocked, :update_blocked]
  before_filter :verify_is_member,        :only => [:accept, :drop, :complete, :reopen]
  before_filter :verify_is_owner,         :only => [:drop, :complete, :estimate, :blocked, :update_blocked]
  before_filter :verify_can_mark_blocked, :only => [:blocked, :update_blocked]
  before_filter :load_message,            :only => [:blocked, :update_blocked]
  before_filter :load_hours,              :only => [:estimate]
  before_filter :path_to_list,            :only => [:index, :new]
  before_filter :path_to_one,             :only => [:show]

  # GET /items
  def index
    @title = "Backlog Items"
    @backlog_items = BacklogItem.for_sprint(params[:sprint]).paginate(:page     => params[:page],
                                                                      :per_page => 10)

    respond_to do |format|
      format.html
    end
  end

  # GET /items/1
  def show
    @title = "Backlog Item (#{@backlog_item.user_story.title})"
    respond_to do |format|
      format.html
    end
  end

  # GET /items/new?sprint=1
  def new
    add_breadcrumb("New")
    @backlog_item = BacklogItem.new(:sprint => @sprint)
    @backlog_item.discovered = true
  end

  # POST /items/new
  def create
    BacklogItem.transaction do
      @backlog_item = BacklogItem.new(params[:backlog_item])
      @backlog_item.sprint = @sprint
      @backlog_item.discovered = true

      respond_to do |format|
        if @backlog_item.save
          flash[:message] = "Backlog item created."
          format.html {redirect_to item_path(@backlog_item)}
        else
          @backlog_item.valid?
          format.html {render :action => :new}
        end
      end
    end
  end

  # DELETE /items/1
  def destroy
    UserStory.transaction do
      if @backlog_item.destroy
        respond_to do |format|
          flash[:message] = "Backlog item deleted successfully."
          format.html { redirect_to items_path(:sprint => @sprint) }
        end
      else
        report_error "Unable to delete that backlog item."
      end
    end
  end

  # GET /items/1/accept
  def accept
    BacklogItem.transaction do
      if @backlog_item.user_can_accept?(@user)
        @backlog_item.assign_to_user(@user)
        add_product_rss_entry("Backlog item ##{@backlog_item.id} accepted.",
                                "#{@user.display_name} accepted \"#{@backlog_item.user_story.title}\".")
        if @backlog_item.save && @backlog_item.sprint.product.save
          respond_to do |format|
            flash[:message] = "This backlog item is now owned by #{@user.display_name}."
            format.html {redirect_to params[:source] ? params[:source] : item_path(@backlog_item)}
          end
        else
          report_error "Unable to change ownership of this backlog item."
        end
      else
        report_error "You cannot accept this backlog item."
      end
    end
  end

  # GET /items/1/drop
  def drop
    BacklogItem.transaction do
      if @backlog_item.user_can_drop?(@user)
        add_product_rss_entry("Backlog item ##{@backlog_item.id} dropped.",
                              "#{@user.display_name} dropped \"#{@backlog_item.user_story.title}\".")
        if @backlog_item.drop && @backlog_item.sprint.product.save
          flash[:message] = "This item has been dropped."
          respond_to do |format|
            format.html {redirect_to items_path(:sprint => @backlog_item.sprint)}
          end
        else
          report_error "Unable to drop this item."
        end
      else
        report_error "You are not allowed to drop this backlog item."
      end
    end
  end

  # GET /items/1/complete
  def complete
    BacklogItem.transaction do
      if @backlog_item.user_can_complete?(@user)
        add_product_rss_entry("Backlog item ##{@backlog_item.id} completed.",
                              "#{@user.display_name} completed \"#{@backlog_item.user_story.title}\".")
        if @backlog_item.complete && @backlog_item.sprint.product.save
          respond_to do |format|
            flash[:message] = "This item is now marked as completed."
            format.html {redirect_to items_path(:sprint => @backlog_item.sprint)}
          end
        else
          report_error "Unable to mark this item as completed."
        end
      else
        report_error "You are now allowed to markt his item as completed."
      end
    end
  end

  # GET /items/1/reopen
  def reopen
    BacklogItem.transaction do
      if @backlog_item.user_can_reopen?(@user)
        add_product_rss_entry("Backlog item ##{@backlog_item.id} reopened.",
                              "#{@user.display_name} reopened \"#{@backlog_item.user_story.title}\".")
        if @backlog_item.reopen && @backlog_item.sprint.product.save
          flash[:message] = "This backlog item has been reopened."
          respond_to do |format|
            format.html {redirect_to params[:source] ? params[:source] : item_path(@backlog_item)}
          end
        else
          report_error "Unable to reopen this backlog item."
        end
      else
        report_error "You are not allowed to reopen this backlog item."
      end
    end
  end

  # PUT /items/1/defer
  def defer
    BacklogItem.transaction do
      @backlog_item.defer
      add_product_rss_entry("Backlog item ##{@backlog_item.id} deferred.",
                            "#{@user.display_name} deferred \"#{@backlog_item.user_story.title}\".")
      if @backlog_item.save
        flash[:message] = "Item has been deferred."
        respond_to do |format|
          format.html {redirect_to items_path(:sprint => @sprint)}
        end
      else
        report_error "Unable to defer item at this time."
      end
    end
  end

  # POST /items/1/estimate
  def estimate
    if @backlog_item.can_estimate?(@user)
      old_hours = @backlog_item.remaining_hours
      @backlog_item.update_remaining_hours(@hours.to_f, @user)
      add_product_rss_entry("Backlog item estimate changed.",
                            "Hours changed from #{old_hours} to #{@hours.to_f} by #{@user.display_name}.")
      BacklogItem.transaction do
        if @backlog_item.save and @backlog_item.sprint.product.save
          respond_to do |format|
            flash[:message] = "Estimated hours updated from #{old_hours} to #{@hours} hours."
            format.html { redirect_to item_path(@backlog_item) }
          end
        else
          report_error "Unable to update the estimated hours at this time."
        end
      end
    else
      report_error "You are not allowed to update the estimated hours for this item."
    end
  end

  # GET /items/1/blocked
  def blocked
    respond_to do |format|
      format.html
    end
  end

  # PUT /items/1/update_blocked
  def update_blocked
    Message.transaction do
      if params[:blocked]
        @backlog_item.blocked = params[:blocked]
        if @message.new_record?
          flash[:message] = "This item is now marked as blocked."
          @message = Message.new(params[:message])
          @backlog_item.messages << @message
        else
          flash[:message] = "This item's blocker message has been updated."
          @message.update_attributes(params[:message])
        end
      else
        if @backlog_item.blocked?
          flash[:message] = "This item is no longer marked as blocked."
          @backlog_item.blocked = false
        else
          flash[:message] = "Item blocked status unchanged."
        end
      end
      add_product_rss_entry("Backlog item ##{@backlog_item.id} is blocked.",
                            "#{@user.display_name} has marked this item as blocked: \"#{@backlog_item.user_story.title}\".")
      if @backlog_item.save && @message.save
        respond_to do |format|
          format.html {redirect_to item_path(@backlog_item)}
        end
      else
        report_error "Unable to update item's blocked state at this time."
      end
    end
  end

  private

  def unsupported
    report_error "That function is not supported for backlog items."
  end

  def load_sprint
    @sprint = Sprint.find_by_id(params[:sprint])
    @team_lead = @sprint.team_lead if @sprint
    @user_stories = UserStory.for_product(@sprint.product).not_in_sprint(@sprint)

    report_error "Missing or invalid sprint." unless @sprint
  end

  def load_backlog_item
    @backlog_item = BacklogItem.find_by_id(params[:id])
    @tasks = @backlog_item.tasks if @backlog_item
    @sprint = @backlog_item.sprint if @backlog_item
    @team_lead = @sprint.team_lead if @backlog_item

    report_error "Missing or invalid backlog item." unless @backlog_item && (@backlog_item.sprint_id == @sprint.id)
  end

  def load_message
    @message = Message.blocker_messages(@backlog_item).last
    @message = Message.new(:author => @user, :posted => Time.current) unless @message
  end

  def load_hours
    @hours = params[:hours]

    report_error "Missing or invalid hours." unless @hours && @hours.to_f > 0.0
  end

  def verify_can_delete
    report_error "You are not allowed to delete this item." unless @backlog_item.can_delete?(@user)
  end

  def verify_can_defer
    report_error "You may not defer this item." unless @backlog_item.user_can_defer?(@user)
  end

  def verify_can_create
    report_error "You are not allowed to create or edit backlog items." unless @sprint.can_add_backlog_items?(@user)
  end

  def verify_sprint_is_active
    report_error "This sprint is not in the active state." unless @sprint.active?
  end

  def verify_is_member
    report_error "You must be a member of the sprint team." unless @sprint.is_team_member?(@user)
  end

  def verify_is_owner
    report_error "You must be owner for this backlog item." unless @backlog_item.owner?(@user)
  end

  def verify_can_mark_blocked
    report_error "You may not mark this item as blocked." unless @backlog_item.can_mark_blocked?(@user)
  end

  def add_product_rss_entry(title, content)
    product = @backlog_item.sprint.product
    product.rss_entries << create_rss_entry(title, item_path(@backlog_item), content, @user)
  end

  def path_to_list
    if @sprint || params[:sprint]
      @sprint = Sprint.find_by_id(params[:sprint]) unless @sprint
      @product = @sprint.product
      @project = @product.project

      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{@project.id}", project_path(@project)
      add_breadcrumb "Products", products_path(:project => @project.id)
      add_breadcrumb "#{@product.id}", product_path(@product)
      add_breadcrumb "Sprints", sprints_path(:product => @product)
      add_breadcrumb "#{@sprint.id}", sprint_path(@sprint)
      add_breadcrumb "Items", items_path(:sprint => @sprint)
    else
      add_breadcrumb "Items", items_path
    end
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@backlog_item.id}", item_path(@backlog_item)
  end
end
