# products_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +ProductController+ handles the management and viewing of instances of
# +Product+.
#
class ProductsController < ApplicationController
  before_filter :authenticated, :except => [:rss, :index, :show]
  before_filter :load_project,  :only => [:new, :create]
  before_filter :is_approved,   :only => [:new, :create]
  before_filter :load_product,  :only => [:rss, :show, :edit, :update, :join]
  before_filter :path_to_one,   :only => [:show, :edit, :update, :join]
  before_filter :path_to_list,  :only => [:index, :new, :create]

  # GET /products/1/rss
  def rss
    @entries = @product.rss_entries

    render :layout => false
  end

  # GET /products
  def index
    project_id = params[:project]

    @products = Product.for_project(project_id).paginate(:page     => params[:page],
                                                          :per_page => 10,
                                                          :order    => 'name ASC')

    if project_id
      @project = Project.find_by_id(project_id)
      @title = "Products For #{@project.name}"
    else
      @title = "All Products"
    end

    respond_to do |format|
      format.html
    end
  end

  # GET /products/1
  def show
    @title = "#{@product.name} (Product)"
    @project = @product.project
    @sprints = Sprint.for_product(@product.id).paginate(:page     => params[:sprint_page],
                                                        :per_page => 10)
    @user_stories = UserStory.for_product(@product.id).paginate(:order    => 'priority ASC',
                                                               :page     => params[:user_story_page],
                                                               :per_page => 10)

    respond_to do |format|
      format.html
    end
  end

  # GET /products/new
  def new
    add_breadcrumb "New"
    if @project.can_create_products?(@user)
      @title = "New Product"
      @product = Product.new(:project_id => @project.id, :owner_id => @user.id)
      @users = User.find(:all, :order => 'display_name ASC')
    else
      flash[:error] = "You are not authorized to create products for #{@project.name}."
      redirect_to error_path
    end
  end

  # GET /products/1/edit
  def edit
    add_breadcrumb "Edit"
    if @product.can_edit?(@user)
      @title = "#{@product.name} (Edit)"
      @users = User.find(:all, :order => 'display_name ASC')
    else
      flash[:error] = "You are not authorized to modify #{@product.name}."
      redirect_to product_path(@product)
    end
  end

  # PUT /products
  def create
    add_breadcrumb "New"
    respond_to do |format|
      if params[:product]
        Product.transaction do
          @product = Product.new(params[:product])
          @product.project = @project

          if @product.project.can_create_products?(@user)
            if @product.save
              create_role_for_owner
              flash[:message] = "Product created successfully."
              format.html { redirect_to product_path(@product) }
              add_rss_entry(create_rss_entry("New product created: #{@product.name}",
                                             product_url(@product),
                                             "A new product was created:\r#{@product.description}}"))
            else
              @title = "New Product"
              @users = User.find(:all, :order => 'display_name ASC')
              @product.valid?
              format.html { render :action => :new }
            end
          else
            flash[:error] = "You are not authorized to create products for #{@product.project.name}."
            format.html {redirect_to error_path}
          end
        end
      else
        flash[:error] = "Missing product details."
        format.html { redirect_to error_url }
      end
    end
  end

  # PUT /products/1
  def update
    add_breadcrumb "Edit"
    respond_to do |format|
      if params[:product]
        if @product.can_edit?(@user)
          begin
            Product.transaction do
              old_owner = @product.owner
              @product.update_attributes(params[:product])

              if @product.save
                # create a new product role if the owner changed
                if (@product.owner_id != old_owner.id)
                  old_role = ProductRole.for_product(@product).for_user(old_owner)
                  ProductRole.delete(old_role)
                end
                create_role_for_owner
                add_rss_entry(create_rss_entry("Details updated: #{@product.name}",
                                               product_url(@product),
                                               "The product's details have been updated..."))
                flash[:message] = "#{@product.name} successfully updated."
                format.html { redirect_to product_path(@product) }
              else
                @title = "#{@product.name} (Edit)"
                @users = User.find(:all, :order => 'display_name ASC')
                @product.valid?
                format.html { render :action => :edit }
              end
            end
          end
        else
          flash[:error] = "You are not authorized to change #{@product.name}."
          format.html { redirect_to product_path(@product) }
        end
      else
        flash[:error] = "Missing product details."

        format.html { redirect_to error_path }
      end
    end
  end

  def delete
    respond_to do |format|
      flash[:error] = "Product deletion is not supported at this time."
      format.html { redirect_to error_url}
    end
  end

  private

  def load_project
    @project = Project.find_by_id(params[:project])

    report_error "Invalid project specified." unless @project
    report_error "That project is not approved." if @project && !@project.approved
  end

  def is_approved
    unless @project.approved
      flash[:error] = "A project must be approved before it can have products."
      respond_to do |format|
        format.html {redirect_to project_path(@project)}
      end
    end
  end

  def load_product
    @product = Product.find_by_id(params[:id])
    @project = @product.project if @product

    unless @product
      respond_to do |format|
        flash[:message] = 'Invalid product specified.'
        format.html { redirect_to error_path }
      end
    end
  end

  def add_rss_entry rss_entry
    Project.transaction do
      @product.rss_entries << rss_entry
      @product.save
    end
  end

  def create_role_for_owner
    owner = User.find_by_id(@product.owner_id)
    unless @product.is_member?(owner)
      ProductRole.create(:product  => @product,
                         :user_id  => @product.owner_id,
                         :role     => Role.find_by_name("Developer"),
                         :pending  => false,
                         :approved => true)
    end
  end

  def path_to_list
    @project = Project.find_by_id(params[:project]) if params[:project]
    if @project
      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{@project.id}", project_path(@project)
      add_breadcrumb "Products", products_path(:project => @project.id)
    else
      add_breadcrumb "Products", products_path
    end
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@product.id}", product_path(@product)
  end
end
