# projects_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +ProjectsController+ handles requests relating to the management and viewing
# of +Product+s.
#
class ProjectsController < ApplicationController
  before_filter :authenticated,   :except => [:index, :show]
  before_filter :load_project,    :except => [:index, :new, :create, :destroy]
  before_filter :filter_approved, :only => [:approve, :reject]
  before_filter :can_approve,     :only => [:approve, :reject]
  before_filter :setup_for_edit,  :only => [:new, :create, :edit, :update]
  before_filter :path_to_list,    :only => [:index, :new, :create]
  before_filter :path_to_one,     :only => [:show, :edit, :update]

  # GET /projects
  def index
    @title = "All Projects"
    @order = "id"
    @show_all_projects = params[:all] ? params[:all] : false
    @projects = Project.approved(@show_all_projects).paginate(:page => params[:page],
                                                              :per_page => 10,
                                                              :order => @order)
    respond_to do |format|
      format.html
    end
  end

  # GET /projects/1
  def show
    @title = "#{@project.name} (Project)"
    @products = Product.paginate_by_project_id @project.id,  :page => params[:page], :per_page => 10

    respond_to do |format|
      format.html
    end
  end

  # GET /projects/new
  def new
    add_breadcrumb("New")
    if @user
      @title = "New Project"
      @project = Project.new(:owner_id => @user.id)
      @source = params[:source] if params[:source]
    else
      flash[:error] = "You are not authorized to create projects."

      redirect_to error_url
    end
  end

  # GET /projects/1/edit
  def edit
    add_breadcrumb("Edit")
    @title = "#{@project.name} (Edit)"
    @source = params[:source] if params[:source]
    unless @project.can_edit?(@user)
      flash[:message] = "You are not authorized to modify that project."
      redirect_to project_path(@project)
    end
  end

  # POST /projects
  def create
    add_breadcrumb("New")
    respond_to do |format|
      @project = Project.new(params[:project])
      # if the project is being created by an admin, then mark it approved
      @project.approved = @user.can_create_projects?
      Project.transaction do
        if @project.save
          # if the user does not have admin rights, then email the project admins
          unless @user.can_create_projects?
            UserMailer.deliver_project_requested(User.project_admins, @project)
          end

          flash[:message] = "Project '#{@project.name}' created!"
          format.html { redirect_to params[:source] ? params[:source] : project_url(@project) }
        else
          @title = "New Project"
          @project.valid?
          format.html { render :action => "new" }
        end
      end
    end
  end

  # PUT /projects/1
  def update
    add_breadcrumb("Edit")
    respond_to do |format|
      if @project.can_edit?(@user)
        @project.update_attributes(params[:project])
        Project.transaction do
          if @project.save
            flash[:message] = "Project was successfully updated."
            format.html { redirect_to params[:source] ? params[:source] : project_path(@project) }
          else
            @title = "#{@project.name} (Edit)"
            @project.valid?
            format.html { render :action => 'edit'}
          end
        end
      else
        flash[:error] = "You are not authorized to edit this project."
        format.html { redirect_to params[:source] ? params[:source] : project_path(@project) }
      end
    end
  end

  # DELETE /projects/1
  def destroy
    respond_to do |format|
      flash[:message] = "Project deletion is not current supported."
      format.html { redirect_to error_url}
    end
  end

  # PUT /project/1/approve
  def approve
    Project.transaction do
      @project.approved = true
      @project.save!
      UserMailer.deliver_project_request(@project, true)

      flash[:message] = "#{@project.name} has been approved."
      respond_to do |format|
        format.html { redirect_to project_path(@project) }
      end
    end
  end

  # PUT /project/1/reject
  def reject
    Project.transaction do
      @project.destroy
      UserMailer.deliver_project_request(@project, false)

      flash[:message] = "#{@project.name} has been rejected/deleted."
      respond_to do |format|
        format.html {redirect_to projects_path}
      end
    end
  end

  private

  # Loads the requested project.
  def load_project
    @project = Project.find_by_id(params[:id])

    unless @project
      flash[:message] = 'Invalid project specified.'
      respond_to do |format|
        format.html { redirect_to error_path }
      end
    end
  end

  def setup_for_edit
    if @user.can_create_projects?
      @users = User.find(:all, :order => 'display_name ASC')
    else
      @users = [@user]
    end
  end

  # Ensures that the project is unapproved.
  def filter_approved
    if @project.approved
      flash[:message] = "This project has already been approved."
      respond_to do |format|
        format.html { redirect_to project_path(@project)}
      end
    end
  end

  # Ensures that the user can approve projects.
  def can_approve
    unless @project.can_approve?(@user)
      flash[:message] = "You do not have project administration rights."
      respond_to do |format|
        format.html {redirect_to error_path}
      end
    end
  end

  def path_to_list
    add_breadcrumb "Projects", projects_path
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@project.id}", project_path(@project)
  end
end
