# report_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +ReportController+ allows users to view various reports.
#
class ReportController < ApplicationController
  before_filter :load_sprint, :except => [:index, :effort]
  before_filter :load_user, :only => [:effort]
  before_filter :load_size, :only => [:burndown, :burndown_graphic]

  SIZE_SMALL  = "800x600"
  SIZE_MEDIUM = "1024x768"
  SIZE_LARGE  = "1280x1024"

  REPORT_SIZES = {
    SIZE_SMALL  => 0,
    SIZE_MEDIUM => 1,
    SIZE_LARGE  => 2
  }.sort_by { |k,v| v }

  def index
    breadcrumbs
    @title = "Reports"
    @users = User.find(:all, :order => 'display_name ASC')
  end

  def burndown
    @title = "Burndown Chart For Sprint #{@sprint.id}"
    @size = params[:size]
  end

  def effort
    @title = "Effort Report For #{@this_user.display_name}"
    @tasks = Task.find(
      :all,
      :conditions => ['(primary_id = ? or backup_id = ?) and
        backlog_item_id in
    (select bi.id from backlog_items bi where bi.sprint_id in (select id from sprints where state = ?))',
        @this_user.id, @this_user.id, Sprint::STATE_ACTIVE],
      :order => 'backlog_item_id')
  end

  # Generates a burndown report.
  #
  def burndown_graphic
    if @sprint.can_view_burndown?
      @data = @sprint.burndown_data
      @goal = Array.new
      delta = @sprint.estimated_hours / @sprint.duration
      (0..@sprint.duration).each { |day| @goal[day] = @sprint.estimated_hours - (delta * day)}

      g = Gruff::Line.new(@size)
      g.title = "#{@sprint.title}"

      g.font = File.expand_path('Courier.ttf', RAILS_ROOT)
      g.data("Remaining hours", @data)
      g.data("Target", @goal)
      g.labels= {
        0 => @sprint.starts.to_s(:date),
        (@sprint.duration / 2) => (@sprint.starts + (@sprint.duration / 2)).to_s(:date),
        (@sprint.duration - 1) => (@sprint.starts + (@sprint.duration - 1)).to_s(:date)}

      send_data(g.to_blob, :disposition => 'inline', :type => 'image/png', :filename => "site-stats.png")
    else
      flash[:error] = 'Cannot view a burndown chart for this sprint.'
      redirect_to sprint_path(@sprint)
    end
  end

  private

  def load_sprint
    @sprint = Sprint.find_by_id(params[:id])

    unless @sprint
      flash[:error] = 'Missing or invalid sprint id.'
      redirect_to error_url
    end
  end

  def load_user
    @this_user = User.find_by_id(params[:id])

    unless @this_user
      flash[:error] = "Invalid or unspecified user."
      redirect_to error_url
    end
  end

  def load_size
    size = params[:size]
    if size
      size = size.to_i
      if (0...REPORT_SIZES.size).include? params[:size].to_i
        @size = REPORT_SIZES[size.to_i][0]
      end
    end

    @size = REPORT_SIZES[0][0] unless @size
  end

  def breadcrumbs
    @breadcrumbs << Breadcrumb.new("Reports")
  end
end
