# roles_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +RolesController+ allows users to request roles in a product, for product
# owners to approve, deny or remove roles, and for listing current product
# members.
class RolesController < ApplicationController
  before_filter :authenticated,     :except => [:index, :show]
  before_filter :load_product
  before_filter :load_product_role, :only => [:show, :update, :destroy]
  before_filter :path_to_list,      :only => [:index, :new, :create]
  before_filter :path_to_one,       :only => [:show, :edit, :update]

  # GET /products/1/roles
  def index
    @title = "#{@product.name} Members"
    if @product.can_approve_roles?(@user)
      @product_roles = ProductRole.paginate(
        :conditions => ['product_id = ?', @product.id],
        :order      => 'role_id ASC',
        :page       => params[:page],
        :per_page   => 10)
    else
      @product_roles = ProductRole.paginate(
        :conditions => ['product_id = ? and is_approved = true', @product.id],
        :order      => 'role_id ASC',
        :page       => params[:page],
        :per_page   => 10)
    end

    respond_to do |format|
      format.html
    end
  end

  # GET /products/1/roles/1
  def show

  end

  # GET /products/1/roles/new
  def new
    @title = "Request A Role For #{@product.name}"
    @role = ProductRole.find_by_product_id_and_user_id(@product.id, @user.id)

    if @role
      flash[:error] = "You already have a role (#{@role.role.name}) on #{@product.name}."
      redirect_to product_path(@product)
    else
      @role = ProductRole.new(:user_id => @user.id, :product_id => @product.id)
      @roles = Role.find(:all)
    end
  end

  # POST /products/1/roles
  def create
    respond_to do |format|
      if params[:product_role]
        if ProductRole.find_by_product_id_and_user_id(@product, @user)
          flash[:error] = "You already have a role with #{@product.name}. "
          format.html { redirect_to error_url }
        else
          ProductRole.transaction do
            @role = ProductRole.new(params[:product_role])
            @role.product = @product
            @role.user = @user
            @role.pending = true

            if @role.save
              UserMailer.deliver_product_role_request(@role)

              flash[:message] = "Your request has been sent to #{@product.owner.display_name}."
              format.html { redirect_to product_path(@product) }
            else
              @title = "Request A Role For #{@product.name}"
              @roles = Role.find(:all)
              @role.valid?
              format.html { render :action => :new }
            end
          end
        end
      else
        flash[:error] = "Missing role details."
        format.html { redirect_to new_product_role_path(@product) }
      end
    end
  end

  # PUT /products/1/roles/1
  def update
    respond_to do |format|
      if @product.can_approve_roles?(@user)
        begin
          ProductRole.transaction do
            @product_role.update_attributes(params[:product_role])

            success = @product_role.approved ? @product_role.save : @product_role.destroy

            if success
              UserMailer.deliver_product_role_disposition(@product_role)

              flash[:message] = "Product role updated successfully."
              format.html { redirect_to product_roles_path(@product) }
            else
              flash[:error] = "There was an error updating that role."
              format.html { redirect_to product_role_path(@product_role) }
            end
          end
        end
      else
        flash[:error] = "You are not authorized to change product roles for #{@product.name}."
        format.html { redirect_to product_path(@product) }
      end
    end
  end

  # DELETE /products/1/roles/1
  def destroy
    respond_to do |format|
      if @product.can_approve_roles?(@user)
        ProductRole.transaction do
          @product_role.destroy

          flash[:message] = "Deleted the #{@product_role.role.name} role for #{@product_role.user.display_name}."
          format.html { redirect_to product_roles_path(@product) }
        end
      else
        flash[:error] = "You are not authorized to delete roles for #{@product.name}."
        format.html { redirect_to product_roles_path(@product) }
      end
    end
  end

  private

  def load_product
    @product = Product.find_by_id(params[:product_id])
    @project = @product.project if @product

    unless @product
      flash[:error] = "Missing or invalid product id."
      respond_to do |format|
        format.html { redirect_to error_url }
      end
    end
  end

  def load_product_role
    @product_role = ProductRole.find_by_id(params[:id])

    unless @product_role
      flash[:error] = "Missing or invalid product role specified."
      respond_to do |format|
        format.html { redirect_to product_path(@product) }
      end
    end
  end

  def path_to_list
    add_breadcrumb "Project", projects_path
    add_breadcrumb "#{@project.id}", project_path(@project)
    add_breadcrumb "Product", products_path(:project => @project.id)
    add_breadcrumb "#{@product.id}", product_path(@product)
    add_breadcrumb "Roles", product_roles_path(@product)
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@product_role.id}", product_role_path(@product, @product_role)
  end
end
