# sprints_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

# +SprintsController+ handles performing CRUD operations on instances of
# +Sprint+.
class SprintsController < ApplicationController
  before_filter :authenticated,               :except => [:index, :show, :members]
  before_filter :load_product,                :only => [:new, :create]
  before_filter :load_sprint,                 :except => [:index, :new, :create]
  before_filter :verify_planned,              :only => [:plan, :populate, :vote, :cast_votes]
  before_filter :verify_can_edit,             :only => [:edit, :update]
  before_filter :verify_can_change_state,     :only => [:planning, :start, :complete, :cancel]
  before_filter :verify_can_vote,             :only => [:vote, :cast_votes]
  before_filter :verify_sprint_can_start,     :only => [:start]
  before_filter :extract_votes_and_estimates, :only => [:vote]
  before_filter :receive_votes_and_estimates, :only => [:cast_votes]
  before_filter :prepare_for_edit,            :only => [:new, :create, :edit, :update, :plan, :populate]
  before_filter :path_to_list,                :only => [:index, :new, :create]
  before_filter :path_to_one,                 :only => [:show, :edit, :update, :plan, :populate, :vote, :members]

  # GET /sprints
  def index
    @title = "Sprints"
    @sprints = Sprint.for_product(params[:product]).paginate(:page     => params[:page],
                                                             :per_page => 10)

    respond_to do |format|
      format.html
    end
  end

  # GET /sprints/1
  def show
    @title = "Sprint:#{@sprint.title}"
  end

  # GET /sprints/new
  def new
    add_breadcrumb "New"
    if @product.can_create_sprints?(@user)
      @title = "New Sprint For #{@product.name}"
      @sprint = Sprint.new(:product_id => @product.id)
    else
      report_error "Cannot create a sprint at this time."
    end
  end

  # GET /sprints/1/edit
  def edit
    add_breadcrumb "Edit"
    @title = "Sprint:#{@sprint.title}"
  end

  # POST /sprints
  def create
    add_breadcrumb "New"
    Sprint.transaction do
      if @product.can_create_sprints?(@user)
        @sprint = Sprint.new(params[:sprint])
        @sprint.product = @product
        @selected = params[:selected]
        @selected = Array.new unless @selected # if no members were selected, use an empty array
        add_users_to_sprint(@selected)
        @product.sprints << @sprint
        respond_to do |format|
        if @product.save
          @product.rss_entries << create_rss_entry("New sprint created.",
                                                   sprint_url(@sprint),
                                                   "A new sprint was defined. The goals for this sprint are:\r#{@sprint.goals}")
            flash[:message] = "Sprint successfully created!"
            format.html { redirect_to plan_sprint_path(@sprint) }
          else
            @title = "New Sprint For #{@product.name}"
            @sprint.valid?
            format.html { render :action => :edit }
          end
        end
      else
        report_error "You cannot create a sprint for this product."
      end
    end
  end

  # PUT /sprints/1
  def update
    Sprint.transaction do
      add_breadcrumb "Edit"
      @sprint.update_attributes(params[:sprint])
      # empty the member list
      @sprint.sprint_members.clear
      @selected = params[:selected]
      @selected = Array.new unless @selected
      add_users_to_sprint(@selected)
      @product.rss_entries << create_rss_entry("Sprint updated: #{@sprint.title}",
                                               sprint_url(@sprint),
                                               "The details for the sprint have been changed.")

      respond_to do |format|
        if @sprint.save && @product.save
          flash[:message] = "Sprint updated successfully."
          format.html { redirect_to params[:url] ? params[:url] : sprints_path(:product => @product) }
        else
          @title = "Sprint #{@sprint.id} (Edit)"
          @sprint.valid?
          format.html { render :action => :edit }
        end
      end
    end
  end

  # DELETE /sprints/1/destroy
  def destroy
    Product.transaction do
      if @sprint.can_delete?(@user)
        @product = @sprint.product
        @product.sprints.delete(@sprint)
        @product.rss_entries << create_rss_entry("Sprint deleted: #{@sprint.title}",
                                                 sprints_url(:product => @product),
                                                 "This sprint has been deleted.")
        if @product.save
        respond_to do |format|
          flash[:message] = "Sprint deleted successfully."
          format.html { redirect_to sprints_path(:product => @product) }
        end
      else
          report_error "Unable to delete sprint at this time."
      end
    else
      report_error "You are not allowed to delete this sprint."
    end
    end
  end

  # PUT /sprints/1/planning
  def planning
    Sprint.transaction do
      if @sprint.can_plan?
        @sprint.product.rss_entries << create_rss_entry("Sprint in planning: #{@sprint.title}",
                                                        sprint_url(@sprint),
                                                        "This sprint has been moved to the planning state.")
        @sprint.plan
        @sprint.product.save!
        respond_to do |format|
          flash[:message] = "The sprint is now moved to the planning state."
          format.html {redirect_to sprint_path(@sprint)}
        end
      else
        report_error "This sprint cannot be moved to the planning state."
      end
    end
  end

  # GET /sprints/1/vote
  def vote

  end

  # POST /sprints/1/cast_votes
  def cast_votes
    BacklogItemVote.transaction do
      BacklogItemVote.for_sprint(@sprint).for_user(@user).destroy_all
      @user_votes.each do |vote|
        vote.save!
      end
      # send an email to the team lead
      UserMailer.deliver_votes_cast(@sprint, @user, @user_votes)
      respond_to do |format|
        flash[:message] = "Your votes have been cast."
        format.html {redirect_to sprint_path(@sprint)}
      end
    end
  end

  # PUT /sprints/1/start
  def start
    Sprint.transaction do
      if @sprint.can_start?
        @sprint.product.rss_entries << create_rss_entry("Sprint started: #{@sprint.title}",
                                                        sprint_url(@sprint),
                                                        "This sprint has been started.")
        @sprint.start
        @sprint.product.save!
        respond_to do |format|
          flash[:message] = "This sprint is now started."
          format.html {redirect_to sprint_path(@sprint)}
        end
      else
        report_error "This sprint cannot be moved to the active state."
      end
    end
  end

  # PUT /sprints/1/complete
  def complete
    Sprint.transaction do
      if @sprint.can_complete?
        @sprint.product.rss_entries << create_rss_entry("Sprint completed: #{@sprint.title}",
                                                        sprint_url(@sprint),
                                                        "This sprint has been marked as completed.")
        @sprint.complete
        @sprint.product.save!
        respond_to do |format|
          flash[:message] = "This sprint is now completed."
          format.html {redirect_to sprint_path(@sprint)}
        end
      else
        report_error "This sprint cannot be moved to the completed state."
      end
    end
  end

  # PUT /sprints/1/cancel
  def cancel
    Sprint.transaction do
      if @sprint.can_cancel?
        @sprint.product.rss_entries << create_rss_entry("Sprint cancelled: #{@sprint.title}",
                                                        sprint_url(@sprint),
                                                        "This sprint has been cancelled.")
        @sprint.cancel
        @sprint.product.save!
        respond_to do |format|
          flash[:message] = "This sprint has been cancelled."
          format.html {redirect_to sprint_path(@sprint)}
        end
      else
        report_error "This sprint cannot be cancelled."
      end
    end
  end

  # GET /sprints/1/plan
  def plan
    add_breadcrumb "Plan"
    if @sprint.can_populate?(@user)
      @title = "Sprint #{@sprint.id} (Planning)"
      @user_stories = UserStory.find_all_by_product_id_and_closed(@product.id, false)
      @selected = []

      respond_to do |format|
        format.html
      end
    else
      report_error "You are not allowed to plan this sprint.", sprint_path(@sprint)
    end
  end

  # POST /sprints/1/populate
  def populate
    add_breadcrumb "Plan"
    if @sprint.can_populate?(@user)
      @sprint.backlog_items.clear

      @selected = params[:selected]
      if @selected
        @selected.each do |selected|

          @sprint.backlog_items << BacklogItem.new(:sprint_id => @sprint.id,
                                                   :user_story_id => selected)
        end
      end

      respond_to do |format|
        @sprint.product.rss_entries << create_rss_entry("Sprint has been populated: #{@sprint.title}",
                                                        sprint_url(@sprint),
                                                        "Total stories: #{@sprint.backlog_items.size}\nTotal hours: #{@sprint.estimated_hours}")
        if @sprint.save! && @sprint.product.save!
          flash[:message] = "Sprint backlog has been updated with #{@sprint.backlog_items.size} items."
          format.html { redirect_to sprint_path(@sprint) }
        else
          @title = "Sprint #{@sprint.id} (Planning)"
          flash[:error] = "There was an error updating the sprint backlog."
          @sprint.backlog_items.each do |backlog_item|
            if backlog_item.errors['estimated_hours']
              flash[:error] += ' ' + backlog_item.errors['estimated_hours']
              break
            end
          end
          @user_stories = UserStory.find_all_by_product_id(@product.id)
          format.html { render :action => :plan, :selected => params[:selected] }
        end
      end
    else
      report_error "You are not allowed to populate this sprint.", sprint_path(@sprint)
    end
  end

  # GET /sprint/1/members
  def members
    add_breadcrumb "Members", members_sprint_path(@sprint)
  end

  private

  def load_product
    @product = Product.find_by_id(params[:product])
    @members = @product.members if @product
    @project = @product.project if @product

    report_error "Missing or invalid product." unless @product
  end

  def load_sprint
    @sprint = Sprint.find_by_id(params[:id])
    @product = @sprint.product if @sprint
    @members = @sprint.members

    report_error "Missing or invalid sprint" unless @sprint
  end

  def verify_can_edit
    report_error "You are not allowed to edit this sprint." unless @sprint.can_edit?(@user)
  end

  def verify_can_change_state
    report_error "You are not allowed to change this sprint's state." unless @sprint.can_change_state?(@user)
  end

  def verify_can_vote
    report_error("You are not allowed to vote on this sprint.", sprint_path(@sprint)) unless @sprint.is_team_member?(@user)
  end

  def verify_planned
    report_error("You can only vote on items while a sprint is in planning.", sprint_path(@sprint)) unless @sprint.planning?
  end

  def verify_cast_votes
    params[:votes].each_pair do |item_id, vote|
      item = BacklogItem.find_by_id(item_id)
      if !item
        report_error("A vote was cast for a nonexistent backlog item.", sprint_path(@sprint))
      elsif item.sprint_id != @sprint.id then
        report_error("The backlog item \"#{item.user_story.title}\" is not on this sprint.", sprint_path(@sprint))
      end
    end
  end

  def verify_sprint_can_start
    report_error("Only a planned sprint can be started.", sprint_path(@sprint)) unless @sprint.planning?
    report_error("Cannot start a sprint with rejected or unaccepted backlog items.", sprint_path(@sprint)) unless @sprint.accepted?
  end

  def extract_votes_and_estimates
    @votes = {}
    @estimates = {}
    @sprint.backlog_items.each do |item|
      user_vote = BacklogItemVote.for_sprint(@sprint).for_user(@user).find(:first)

      if user_vote
        @votes[item.id] = user_vote.vote
        @estimates[item.id] = user_vote.estimated_hours
      else
        @votes[item.id] = BacklogItemVote::ABSTAIN
        @estimates[item.id] = 0.0
      end
    end
  end

  def receive_votes_and_estimates
    @votes = params[:votes] || {}
    @estimates = params[:estimates] || {}
    @user_votes = []

    # ensures that all votes are for valid backlog items
    @votes.each_pair do |item_id, vote|
      item = BacklogItem.find_by_id(item_id)

      report_error("Invalid backlog item: id=#{item_id}") unless item
      report_error("You cannot vote on an item on a different sprint.") unless (item.sprint_id == @sprint.id)
      report_error("All accepted items must have an estimate.") if (vote == BacklogItemVote::ACK) && (@estimates[item_id] == nil)

      @user_votes << BacklogItemVote.new(:user => @user, :backlog_item => item, :vote => vote, :estimated_hours => @estimates[item_id])
    end
  end

  def prepare_for_edit
    @selected = Array.new
    @members = @product.members
  end

  def add_users_to_sprint(selected)
    selected.each do |id|
      user = User.find_by_id(id)
      raise "Invalid user: #{user.display_name} is not a member of the #{@product.name}." unless @product.is_member?(user)
      @sprint.sprint_members << SprintMember.new(:sprint => @sprint, :member => user)
    end
  end

  def path_to_list
    if @product || params[:product]
      @product = Product.find_by_id(params[:product]) unless @product
      @project = @product.project
      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{@project.id}", project_path(@project)
      add_breadcrumb "Products", products_path(:project => @project.id)
      add_breadcrumb "#{@product.id}", product_path(@product)
      add_breadcrumb "Sprints", sprints_path(:product => @product)
    else
      add_breadcrumb "Sprints", sprints_path
    end
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@sprint.id}", sprint_path(@sprint)
  end
end
