# stories_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

# +StoriesController+ performs CRUD operations for instances of +UserStory+.
class StoriesController < ApplicationController
  before_filter :authenticated,      :except => [:index, :show]
  before_filter :load_product,       :only => [:new, :create]
  before_filter :load_user_story,    :except => [:index, :new, :create]
  before_filter :load_epics,         :only => [:new, :edit, :create, :update]
  before_filter :verify_can_create,  :only => [:new, :create]
  before_filter :verify_can_edit,    :only => [:edit, :update]
  before_filter :verify_can_delete,  :only => [:destroy]
  before_filter :verify_can_comment, :only => [:comment]
  before_filter :load_comment,       :only => [:comment]
  before_filter :path_to_list,       :only => [:index, :new, :create]
  before_filter :path_to_one,        :only => [:show, :edit, :update]

  # GET /stories
  def index
    @title = "User Stories"
    @user_stories = UserStory.for_product(params[:product]).paginate(:page     => params[:page],
                                                                     :per_page => 10)
    respond_to do |format|
      format.html
    end
  end

  # GET /stories/1
  def show
    @title = "User Story - #{@user_story.title}"
    @comment = Message.new(:author => @user) if @user_story.can_comment?(@user)
    respond_to do |format|
      format.html
    end
  end

  # GET /stories/new
  def new
    add_breadcrumb "New"
    @title = "Write User Story"
    @user_story = UserStory.new(:product_id => @product.id)
    @source = params[:source]
  end

  # GET /stories/1/edit
  def edit
    add_breadcrumb "Edit"
    @title = "User Story - #{@user_story.title} (Edit)"
  end

  # POST /stories
  def create
    add_breadcrumb "New"
    respond_to do |format|
      UserStory.transaction do
        @user_story = UserStory.new(params[:user_story])
        @user_story.product = @product
        @source = params[:source]

        if @user_story.save
          flash[:message] = "The user story was successfully created."

          if params[:add_another]
            format.html {
              redirect_to params[:source] ?
              new_story_url(:product => @product, :source => params[:source]) :
              new_story_path(:product => @product)
            }
          else
            format.html { redirect_to params[:source] ? params[:source] : stories_path(:product => @product) }
          end
        else
          @title = "User Story (New)"
          @user_story.valid?
          load_epics
          format.html { render :action => :edit }
        end
      end
    end
  end

  # PUT /stories/1
  def update
    add_breadcrumb "Edit"
    respond_to do |format|
      UserStory.transaction do
        @user_story.update_attributes(params[:user_story])

        if @user_story.save
          flash[:message] = "The user story was successfully updated."
          format.html { redirect_to params[:source] ? params[:source] : stories_path(:product => @product) }
        else
          @title = "User Story - #{@user_story.title} (Edit)"
          @user_story.valid?
          load_epics
          format.html { render :action => :edit }
        end
      end
    end
  end

  # DELETE /stories/1
  def destroy
    respond_to do |format|
      if @user_story.destroy
        flash[:message] = "The user story has been deleted."
        format.html { redirect_to params[:source] ? params[:source] : stories_path(:product => @product) }
      else
        flash[:error] = "Unable to delete this user story."
        format.html { redirect_to params[:source] ? params[:source] : story_path(@user_story) }
      end
    end
  end

  # POST /stories/1/comment
  def comment
    Message.transaction do
      respond_to do |format|
        @comment.author = @user
        @comment.posted = Time.current
        if @comment.valid?
          @user_story.comments << @comment
          @user_story.save!
          flash[:message] = "Comment saved."
          format.html {redirect_to story_path(@user_story)}
        else
          format.html {render :action => :show}
        end
      end
    end
  end

  private

  def load_product
    @product = Product.find_by_id(params[:product])
    @project = @product.project if @product

    report_error "Missing or invalid product." unless @product
  end

  def load_user_story
    @user_story = UserStory.find_by_id(params[:id])
    @product = @user_story.product if @user_story
    @project = @product.project if @product

    report_error "Missing or invalid user story id." unless @user_story
  end

  def verify_can_create
    report_error "You are not allowed to create user stories for this product." unless @product.can_create_user_stories?(@user)
  end

  def verify_can_edit
    report_error "You are not allowed to edit this user story." unless @user_story.can_edit?(@user)
  end

  def verify_can_delete
    report_error "You are not allowed to deslete this user story." unless @user_story.can_delete?(@user)
  end

  def verify_can_comment
    report_error("You are not allowed to comment on this user story.", story_path(@user_story)) unless @user_story.can_comment?(@user)
  end

  def load_comment
    @comment = Message.new(params[:message])
  end

  def load_epics
    @epics = Epic.find_all_by_project_id(@project.id, :order => 'priority', :conditions => "closed = false")
  end

  def path_to_list
    if @product || params[:product]
      @product = Product.find_by_id(params[:product]) unless @product
      project = @product.project if @product

      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{project.id}", project_path(project)
      add_breadcrumb "Products", products_path(:project => project.id)
      add_breadcrumb "#{@product.id}", product_path(@product)
    end
    add_breadcrumb "User Stories", stories_path(:product => @product)
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@user_story.id}", story_path(@user_story)
  end
end
