# tasks_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +TasksController+ allows the user to create and modify tasks for a backlog
# item.
#
class TasksController < ApplicationController
  before_filter :authenticated,       :except => [:index, :show]
  before_filter :load_this_user
  before_filter :load_backlog_item,   :only => [:new, :create]
  before_filter :load_task,           :only => [:show, :edit, :update, :destroy]
  before_filter :verify_can_create,   :only => [:new, :create]
  before_filter :verify_can_edit,     :only => [:edit, :update]
  before_filter :verify_can_delete,   :only => [:destroy]
  before_filter :load_sprint_members, :only => [:new, :create, :edit, :update]
  before_filter :path_to_list,        :only => [:index, :new, :create]
  before_filter :path_to_one,         :only => [:show, :edit, :update]

  # GET /tasks
  def index
    @tasks = Task.for_user(params[:user]).paginate(:page     => params[:page],
                                                   :per_page => 10)
    respond_to do |format|
      format.html
    end
  end

  # GET /tasks/1
  def show
    @title = "Task For Backlog Item #{@task.backlog_item.id}"
    respond_to do |format|
      format.html
    end
  end

  # GET /tasks/new
  def new
    @title = "New Task For \"#{@backlog_item.user_story.title}\""
    @task = Task.new(:primary_id => @user.id, :backlog_item_id => @backlog_item.id)
  end

  # GET /tasks/1/edit
  def edit
    @title = "Task For Backlog Item #{@task.backlog_item.id} (Edit)"
  end

  # POST /tasks
  def create
    Task.transaction do
      @task = Task.new(params[:task])
      @task.primary = @user
      @task.backlog_item_id = @backlog_item.id
      @task.backlog_item.complete if params[:completed]

      if @task.backup_id && !@sprint.is_team_member?(@task.backup)
        report_error "Backup must be a member of the sprint team."
      else
        respond_to do |format|
          add_product_rss_entry("#{@user.display_name} added a new #{@task.hours} hour task.",
                                "#{@task.description}")
          if @task.save && @task.backlog_item.save && @task.backlog_item.sprint.product.save
            flash[:message] = "Recorded #{@task.hours} hours against this backlog item."
            format.html { redirect_to task_path(@task) }
          else
            @title = "New Task For \"#{@backlog_item.user_story.title}\""
            @task.valid?
            format.html { render :action => :new }
          end
        end
      end
    end
  end

  # PUT /tasks/1
  def update
    Task.transaction do
      @task.update_attributes(params[:task])

      respond_to do |format|
        add_product_rss_entry("#{@user.display_name} updated task #{@task.id}.",
                              "#{@task.description}")
        if @task.save && @task.backlog_item.sprint.product.save
          flash[:message] = "Task successfully updated."
          format.html { redirect_to task_path(@task) }
        else
          @title = "Task For Backlog Item #{@task.backlog_item.id} (Edit)"
          @task.valid?
          format.html { render :action => :edit }
        end
      end
    end
  end

  # DELETE /tasks/1
  def destroy
    Task.transaction do
      add_product_rss_entry("#{@user.display_name} deleted task #{@task.id}.",
                            "#{@task.description}")
      if @task.destroy && @task.backlog_item.sprint.product.save
        flash[:message] = "Task has been deleted."
        respond_to do |format|
          format.html { redirect_to tasks_path(:user => @user) }
        end
      else
        report_error "Unable to delete task at this time."
      end
    end
  end

  private

  def load_this_user
    if params[:user]
      @this_user = User.find_by_id(params[:user])

      report_error "Invalid user specified." unless @this_user
    end
  end

  def load_task
    @task = Task.find_by_id(params[:id])
    if @task
      @primary = @task.primary
      @backup = @task.backup
      @backlog_item = @task.backlog_item
      @sprint = @backlog_item.sprint
      @product = @sprint.product
    else
      report_error 'Missing or invalid task.'
    end
  end

  def load_backlog_item
    @backlog_item = BacklogItem.find_by_id(params[:item])
    if @backlog_item
      @sprint = @backlog_item.sprint
      @product = @sprint.product
    else
      report_error "Missing or invalid backlog item."
    end
  end

  def verify_can_create
    report_error "you are not allowed to add tasks to this item." unless @backlog_item.can_add_tasks?(@user)
  end

  def verify_can_edit
    report_error "You are not allowed to edit this task." unless @task.can_edit?(@user)
  end

  def verify_can_delete
    report_error "You are not allowed to delete this task." unless @task.can_delete?(@user)
  end

  def load_sprint_members
    @users = User.for_sprint(@sprint.id)
  end

  def add_product_rss_entry(title, content)
    product = @task.backlog_item.sprint.product
    product.rss_entries << create_rss_entry(title, item_path(@task.backlog_item), content, @user)
  end

  def path_to_list
    if @backlog_item
      @sprint = Sprint.find_by_id(params[:sprint]) unless @sprint
      @product = @sprint.product
      @project = @product.project

      add_breadcrumb "Projects", projects_path
      add_breadcrumb "#{@project.id}", project_path(@project)
      add_breadcrumb "Products", products_path(:project => @project.id)
      add_breadcrumb "#{@product.id}", product_path(@product)
      add_breadcrumb "Sprints", sprints_path(:product => @product)
      add_breadcrumb "#{@sprint.id}", sprint_path(@sprint)
      add_breadcrumb "Items", items_path(:sprint => @sprint)
      add_breadcrumb "#{@backlog_item.id}", item_path(@backlog_item)
    end

    if @this_user
      add_breadcrumb "Users", users_path
      add_breadcrumb "#{@this_user.id}", user_path(@this_user)
    end

    add_breadcrumb "Tasks", tasks_path(:item => @backlog_item)
  end

  def path_to_one
    path_to_list

    add_breadcrumb "#{@task.id}", task_path(@task)
  end
end
