# users_controller.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

# +UsersController+ allows for CRUD operations on instances of +User+.
class UsersController < ApplicationController
  before_filter :authenticated,  :only => [:edit, :update, :verify, :notifications, :change_password]
  before_filter :load_this_user, :except => [:index, :new, :create]
  before_filter :path_to_list,   :only => [:index, :new, :create]
  before_filter :path_to_one,    :only => [:show, :edit, :update, :backlog, :verify, :roles, :password, :change_password ]

  # GET /users
  def index
    @title = "All Users"
    @users = User.paginate(
      :order    => 'display_name ASC',
      :page     => params[:page],
      :per_page => 10)
  end

  # GET /users/1
  def show
    @title = "#{@this_user.display_name}"
    @backlog_items = BacklogItem.paginate(
      :conditions => ['owner_id = ?', @this_user.id],
      :order      => '(select priority from user_stories where id = user_story_id) ASC',
      :page       => params[:item_page],
      :per_page   => 10
    )
  end

  # GET /users/new
  def new
    add_breadcrumb "Register"
    if (@user == nil) || (@user.create_users?)
      @this_user = User.new
      @title = "User Account (New)"
    else
      flash[:error] = "You cannot create a new user account."
      redirect_to users_path
    end
  end

  # GET /users/1/edit
  def edit
    add_breadcrumb "Edit"
    @title = "#{@this_user.display_name} (Edit)"
    unless @this_user.can_edit?(@user)
      flash[:error] = "You are not authorized to edit that account."
      redirect_to users_path
    end
  end

  # POST /users
  def create
    add_breadcrumb "Register"
    respond_to do |format|
      if (@user == nil) || (@user.create_users?)
        @this_user = User.new(params[:user])
        @this_user.privileges = UserPrivilege.new
        @this_user.notifications = Notifications.new
        @this_user.verification = UserVerification.new(
          :sent  => Date.today,
          :token => UserVerification.create_token)

        if @this_user.save
          UserMailer.deliver_email_verification(@this_user,@this_user.verification.token)

          flash[:message] = "An email verification has been sent to #{@this_user.email}."
          format.html { redirect_to user_path(@this_user) }
        else
          @title = "User Account (New)"
          @this_user.valid?
          format.html { render :action => :edit }
        end
      else
        flash[:error] = "You cannot create a new user account."
        redirect_to users_path
      end
    end
  end

  # PUT /users/1
  def update
    add_breadcrumb "Edit"
    respond_to do |format|
      if @this_user.can_edit?(@user)
        old_email = @this_user.email
        @this_user.update_attributes(params[:user])

        if @this_user.save
          UserMailer.deliver_email_change_notification(@this_user,old_email) unless old_email == @this_user.email

          flash[:message] = "User has been updated."
          format.html { redirect_to user_path }
        else
          @title = "#{@this_user.display_name} (Edit)"
          @this_user.valid?
          format.html { render :action => :edit }
        end
      else
        flash[:error] = "You are not allowed to edit that user account."
        format.html { redirect_to users_path }
      end
    end
  end

  # GET /users/1/backlog
  def backlog
    add_breadcrumb "Backlog", backlog_user_path(@this_user)
    @title = "Backlog For #{@this_user.display_name}"
    @backlog_items = BacklogItem.paginate(
      :conditions => ['owner_id = ?', @this_user.id],
      :order      => '(select priority from user_stories where id = user_story_id) ASC',
      :page => params[:page],
      :per_page => 25)
  end

  # GET /verify/1/token
  def verify
    add_breadcrumb "Verify"
    respond_to do |format|
      if @this_user.id == @user.id
        if !@user.verified?
          if params[:resend]
            flash[:message] = "Sent another verification email to #{@this_user.email}."
            UserMailer.deliver_email_verification(@this_user,@this_user.verification.token)
            format.html
          else
            @token = params[:token]

            if @token && @user.verify(@token)
              User.transaction do
                @user.verification.destroy

                flash[:message] = "#{@user.email} is now verified."
                format.html { redirect_to user_path(@user) }
              end
            else
              @title = "Email Verification For #{@user.email}"
              flash[:error] = "Missing or invalid verification token."
              format.html
            end
          end
        else
          flash[:error] = "Your account has already been verified."
          format.html { redirect_to user_path(@user) }
        end
      else
        flash[:error] = "You cannot verify someone else's account."
        format.html { redirect_to user_path(@user.id) }
      end
    end
  end

  # GET /users/1/roles
  def roles
    add_breadcrumb "Roles", roles_user_path(@this_user)
    @product_roles = ProductRole.paginate(
      :conditions => ['user_id = ?', @this_user.id],
      :page       => params[:page],
      :per_page   => 10)
  end

  # POST /users/1/notifications
  def notifications
    respond_to do |format|
      if @user.id == @this_user.id
        Notifications.transaction do
          @this_user.notifications.update_attributes(params[:notifications])

          @this_user.notifications.save!
          flash[:message] = "Notifications updated."
        end
      else
        flash[:error] = "You cannot alter notifications for #{@this_user.display_name}."
      end
      format.html { redirect_to user_path(@this_user) }
    end
  end

  # GET /users/1/password
  def password
    add_breadcrumb "Change Password"
  end

  # POST /users/1/change_password
  def change_password
    add_breadcrumb "Change Password"
    respond_to do |format|
      if @this_user.can_edit?(@user)
        if User.authenticate(@this_user.email, params[:old_password])
          @this_user.update_attributes(params[:user])

          if @this_user.valid?
            if @this_user.save
              flash[:message] = "Password update successfully."
              format.html { redirect_to user_path(@this_user) }
            else
              @this_user.valid?
              format.html { render :action => :password }
            end
          else
            @this_user.valid?
            format.html { render :action => :password }
          end
        else
          flash[:error] = "Old password did not match."
          format.html { redirect_to password_user_path(@this_user) }
        end
      else
        flash[:error] = "You cannot change the password for #{@this_user.display_name}."
        format.html { redirect_to user_path(@this_user) }
      end
    end
  end

  private

  def load_this_user
    @this_user = User.find_by_id(params[:id])

    unless @this_user
      flash[:error] = "Missing or invalid user."

      respond_to do |format|
        format.html { redirect_to users_path }
      end
    end
  end

  def path_to_list
    add_breadcrumb "Users", users_path
  end

  def path_to_one
    path_to_list
    add_breadcrumb "#{@this_user.id}", user_path(@this_user)
  end
end
