# backlog_item.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# A +BacklogItem+ associates a +UserStory+ to a +Sprint+.
#
class BacklogItem < ActiveRecord::Base
  validates_presence_of :sprint_id, :message => 'You need to specify the sprint.'
  validates_presence_of :user_story_id, :message => 'You need to specify a user story.'

  belongs_to :sprint
  belongs_to :user_story
  belongs_to :owner, :class_name => 'User', :foreign_key => 'owner_id'
  belongs_to :backup, :class_name => 'User', :foreign_key => 'backup_id'

  has_many :tasks, :dependent => :destroy
  has_many :remaining_hours_estimates
  has_many :votes, :class_name => 'BacklogItemVote', :order => 'estimated_hours DESC'
  has_many :votes_in_favor, :class_name => 'BacklogItemVote', :conditions => "vote = #{BacklogItemVote::ACK}"

  has_and_belongs_to_many :messages, :join_table => :backlog_items_messages

  STATE_PENDING=0
  STATE_ASSIGNED=1
  STATE_COMPLETED=2
  STATE_DEFERRED=3

  STATE_TEXT={STATE_PENDING => "Pending", STATE_ASSIGNED => "Assigned", STATE_COMPLETED => "Completed", STATE_DEFERRED => "Deferred"}

  state_machine :initial => :pending do
    state :pending,   :value => STATE_PENDING
    state :assigned,  :value => STATE_ASSIGNED
    state :completed, :value => STATE_COMPLETED
    state :deferred,  :value => STATE_DEFERRED

    event :accept do
      transition :pending => :assigned

      def can_fire?(item)
        item.sprint.active? && item.pending?
      end
    end

    before_transition :on => :drop, :do => lambda {|item| item.owner = nil}
    event :drop do
      transition :assigned => :pending

      def can_fire?(item)
        item.assigned? && item.sprint.active?
      end
    end

    event :complete do
      transition :assigned => :completed

      def can_fire?(item)
        item.assigned? && item.sprint.active?
      end
    end

    before_transition :on => :reopen, :do => lambda {|item| item.owner = nil}
    event :reopen do
      transition [:completed, :deferred] => :pending

      def can_fire?(item)
        (item.deferred? || item.completed?) && item.sprint.active?
      end
    end

    before_transition :on => :defer, :do => lambda {|item| item.owner = nil}
    event :defer do
      transition [:pending, :assigned] => :deferred

      def can_fire?(item)
        item.sprint.active? && !item.completed? && !item.deferred?
      end
    end
  end

  named_scope :default, { :order => 'priority ASC' }
  named_scope :for_sprint, lambda { |sprint_id|
    { :conditions => sprint_id ? ['sprint_id = ?', sprint_id] : [] }
  }

  def state_text
    STATE_TEXT[state]
  end

  # Estimated hours is the value for the highest estimate provided by
  # the development team.
  def estimated_hours
    return 0 if votes.empty?

    return votes.first.estimated_hours
  end

  def actual_hours
    result = 0.0

    tasks.each {|task| result += task.hours}

    return result
  end

  def remaining_hours
    return 0.0 if completed? || deferred?

    remaining_hours_estimates.empty? ?
      estimated_hours :
      remaining_hours_estimates.last.hours
  end

  def remaining_hours_for_date(date)
    last_estimate = nil
    remaining_hours_estimates.each do |estimate|
      last_estimate = estimate if estimate.estimated_on <= date
    end

    return last_estimate ? last_estimate.hours.to_f : estimated_hours
  end

  def update_remaining_hours(hours, user)
    remaining_hours_estimates <<
      RemainingHoursEstimate.new(
      :hours => hours,
      :user_id => user.id,
      :estimated_on => Date.today)
  end

  def blocker_message
    messages.last unless messages.empty?
  end

  # Returns if the user can accept the item.
  def user_can_accept?(user)
    can_accept? && sprint.is_team_member?(user)
  end

  # Sets the supplied user as the owner of the item.
  def assign_to_user(user)
    if user_can_accept?(user)
      accept
      self.owner = user
    end
  end

  # Returns whether the item has been accepted or not.
  def accepted?
    return false if votes.empty?

    result = false
    votes.each do |vote|
      return false if vote.vote == BacklogItemVote::NAK
      result |= (vote.vote == BacklogItemVote::ACK) unless vote.vote == BacklogItemVote::ABSTAIN
    end
    return result
  end

  # Returns whether the user can complete the given item.
  def user_can_complete?(user)
    can_complete? && owner?(user)
  end

  # Returns if the user can drop the item.
  def user_can_drop?(user)
    can_drop? && owner?(user)
  end

  # Returns if the user can reopen the item.
  def user_can_reopen?(user)
    can_reopen? && (owner?(user) || sprint.is_team_lead?(user))
  end

  # Returns if the user can defer the item.
  def user_can_defer?(user)
    can_defer? && sprint.is_team_lead?(user)
  end

  # Reports if the task if owned.
  def owned?
    owner_id && assigned?
  end

  # Returns whether the item is current blocked or not.
  def blocked?
    blocked
  end

  # Returns whether the item was discovered.
  def discovered?
    discovered
  end

  # Returns whether the given user can modify this backlog item.
  def can_edit?(user)
    (owner?(user) || product_owner?(user)) && !completed
  end

  # Returns whether the user can assign this item to another.
  def can_assign?(user)
    product_owner?(user) && (!completed? || !deferred?)
  end

  # Returns whether the user can add tasks to this backlog item.
  def can_add_tasks?(user)
    owner?(user) && assigned? && sprint.active?
  end

  # Returns whether the user can provide an estimate for this backlog item.
  def can_estimate?(user)
    owner?(user) && assigned?
  end

  # Returns whether the user can mark the item as blocked.
  def can_mark_blocked?(user)
    owner?(user) && assigned? && sprint.active?
  end

  # Returns whether the user can delete this backlog item.
  def can_delete?(user)
    product_owner?(user) && tasks.empty? && !sprint.active?
  end

  def owner?(user)
    user && owner && (user.id == owner_id)
  end

  private

  def product_owner?(user)
    user && (sprint.product.owner_id == user.id)
  end

  def team_lead?(user)
    user && (sprint.team_lead_id == user.id)
  end
end
