# config_property.rb
# Copyright (C) 2009, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# A +ConfigProperty+ represents a single configuration element.
#
# The +name+ is the unique key for the element.
#
# The +value+ is the value for the element.
#
# The +type+ indicates the element type.
#
# Configuration values should not be retrieved directly. Instead, the developer
# should fetch and store values by name. An instance of +ConfigProperty+ should
# never be used directly.
class ConfigProperty < ActiveRecord::Base
  validates_presence_of :name,
  :message => "A name is required."
  validates_uniqueness_of :name,
  :message => "The name must be unique."

  # Fetches a configuration value by name.
  # If not found, the default value is returned.
  # If no default value is provided, then +nil+ is returned.
  def self.fetch(name, default = nil)
    raise Exception.new("Missing property name") unless name && !name.empty?

    result = ConfigProperty.find_by_name(name)

    return result.value if result
    return default
  end

  # Saves a configuration value.
  def self.store(name, value = nil)
    raise Exception.new("Missing property name") unless name && !name.empty?

    property = ConfigProperty.find_by_name(name)

    if property
      property.value = value
      property.save
    else
      ConfigProperty.create(:name => name, :value => value)
    end
  end

  # Removes a configuration value.
  def self.remove(name)
    raise Exception.new("Missing property name") unless name && !name.empty?

    property = ConfigProperty.find_by_name(name)

    property.destroy if property
  end
end
