# epic.rb
# Copyright (C) 2009, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# An +Epic+ represents a project-level user story, one that is an overarching feature.
# An epic is a feature that is implemented by many individual user stories or
# in more than one product.
class Epic < ActiveRecord::Base
  belongs_to :project
  validates_presence_of :project_id,
  :message => "A project must be specified."

  validates_presence_of :priority,
  :message => "A priority must be provided."
  validates_numericality_of :priority,
  :message => "Priority must be a positive integer value.",
  :only_integer => true,
  :greater_than => 0

  validates_presence_of :title, :message => "A title must be given to all epics."

  belongs_to :project
  has_many :user_stories
  has_and_belongs_to_many :comments, :class_name => "Message", :join_table => "epics_messages"

  named_scope :for_project, lambda { |project_id|
    {
      :conditions => project_id ? ['project_id = ?', project_id] : []
    }
  }

  # Returns a title that's geared for selection lists.
  def selectable_title
    "#{title} (#{priority})"
  end

  # Returns whether the user can post comments on this epic.
  def can_comment?(user)
    project.is_member?(user)
  end

  # Returns whether the user can edit this epic.
  def can_edit?(user)
    is_owner? user
  end

  # Returns whether this epic can be deleted.
  def can_delete?(user)
    is_owner?(user) && user_stories.empty?
  end

  # Returns whether the user can close this epic.
  def can_close?(user)
    is_owner?(user) && !closed && !user_stories.empty? && UserStory.for_epic(self).open.empty?
  end

  # Returns whether the user can reopen this epic.
  def can_reopen?(user)
    is_owner?(user) && closed
  end

  private

  def is_owner?(user)
    (user != nil) && (user.id == project.owner_id)
  end
end
