# product.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# A +Product+ represents one component of a project.
#
class Product < ActiveRecord::Base
  validates_presence_of :project,
    :message => 'A project is required.'

  validates_presence_of :owner,
    :message => 'A product must have an owner.'

  validates_presence_of :name,
    :message => 'A product must have a name.'

  validates_format_of :mailing_list,
    :allow_blank => true,
    :allow_nil   => true,
    :with => /^([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})$/i,
    :message => 'Mailing list must be a valid email address.'

  belongs_to :project
  belongs_to :owner, :class_name => 'User', :foreign_key => 'owner_id'

  has_many :product_roles, :dependent => :destroy, :conditions => {:pending => false, :is_approved => true}
  has_many :members, :through => :product_roles, :source => :user
  has_many :user_stories, :dependent => :destroy
  has_many :sprints, :dependent => :destroy
  has_and_belongs_to_many :rss_entries, :class_name => 'Feed', :join_table => :product_feeds, :order => 'created_at DESC'

  named_scope :default, { :group => 'project_id' }
  named_scope :for_project, lambda { |project_id|
    {
      :conditions => project_id ? ["project_id = ?", project_id] : []
    }
  }
  named_scope :for_owner, lambda { |owner_id| {:conditions => ['owner_id = ?', owner_id]}}

  # Returns the set of roles that are active.
  def active_roles
    result = []

    product_roles.each { |role| result << role if role.active?}

    return result
  end

  # Returns the set of roles that are pending approval.
  def pending_roles
    result = []

    product_roles.each { |role| result << role if !role.active? }

    return result
  end

  # Returns the product's logo url.
  def logo_url
    if logo_name
      "#{ConfigProperty.fetch('image.product_logo_path')}/#{logo_name}"
    else
      project.logo_url
    end
  end

  # Returns whether the user can modify this product.
  def can_edit?(user)
    user && (user.id == owner.id)
  end

  # Returns whether the user can approve or deny role requests.
  def can_approve_roles?(user)
    user && (user.id == owner.id)
  end

  # Returns whether the user can create user stories for the product.
  def can_create_user_stories?(user)
    user && (user.id == owner.id)
  end

  # Returns whether the user can create sprints for this product.
  def can_create_sprints?(user)
    user && (user.id == owner.id) && !user_stories.empty? && Sprint.for_product(self).planned.empty?
  end

  # Returns whether the user specified is the product owner.
  def is_owner?(user)
    user && (owner_id == user.id)
  end

  # Returns whether the specified user is a member of the product team.
  def is_member?(user)
    user && !ProductRole.for_product(self).for_user(user).approved.empty?
  end

  # Returns whether the user has a pending membership.
  def pending_member?(user)
    if (user != nil)
      role = ProductRole.find_by_product_id_and_user_id(self.id, user.id)
      return !role.approved if role
    end
  end

  # Returns whether the product has a mailing list.
  def has_mailing_list?
    mailing_list && !mailing_list.empty?
  end
end
