# project.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require 'avatar'

# +Project+ represents a overall project. Projects are composed of one or more
# +Product+s, each of which produces an artifact.
#
class Project < ActiveRecord::Base
  validates_presence_of :name,
    :message => 'A project has to have a name.'
  validates_length_of :name,
    :minimum => 4,
    :message => 'The project name must be at least four characters long.'
  validates_uniqueness_of :name,
    :message => 'This name is used by an existing project.'

  validates_presence_of :owner_id,
    :message => 'A project must have an owner.'

  belongs_to :owner,  :class_name => 'User', :foreign_key => :owner_id

  has_many :epics
  has_many :products, :dependent => :destroy

  named_scope :approved, lambda{ |show_all|
    {
      :conditions => show_all ? [] : {:approved => true}
    }
  }

  # Returns whether the user is a member of one of the project's product teams.
  def is_member?(user)
    user && !ProductRole.for_project(self).for_user(user).approved.empty?
  end

  # Returns whether the user can modify this project.
  def can_edit?(user)
    user && (user.privileges.admin_projects || (user.id == owner_id))
  end

  # Returns whether the user can create products for this project.
  def can_create_products?(user)
    approved && (user != nil) && (user.id == owner_id)
  end

  # Returns whether the user can create or edit epic stories.
  def can_create_epics?(user)
    approved && (user != nil) && (user.id == owner_id)
  end

  # Returns whether the user can approve the project.
  def can_approve?(user)
    (user != nil) && user.privileges.admin_projects
  end

  # Returns the project logo URL.
  def logo_url
    return ConfigProperty.fetch("image.default_project_logo")
  end
end
