# sprint.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# A +Sprint+ represents a single development interation.
#
# A sprint begins on a +start_date+ and has a +duration+ measured in days.
#
# The +goals+ of a sprint describe what the product owner expects to achieve
# when all tasks for the sprint are completed.
#
# It begins on the +start+ date and ends after +duration+ days.
#
# The +health+ of a sprint is based on the hours of work remaining versus the
# average hours that should be completed per day.
#
# For example, consider a sprint is scheduled to last 20 days and was estimated
# for a total of 200 hours of work. At the 10 day mark an average sprint should
# have around 100 estimated hours of work remaining.
#
# Some adjusting of hours is done since no development team is perfect. But, if
# more than the adjusted amount of hours remain to be done then the sprint is
# considered +unhealthy+. If the estimated amount or fewer hours remain then the
# sprint is considered healthy.
#
class Sprint < ActiveRecord::Base
  validates_presence_of :product_id,
    :message => 'Sprints must be associated with a product.'

  validates_presence_of :title,
    :message => 'Please give the sprint a title.'

  validates_presence_of :starts,
    :message => 'Please set the starting date for the sprint.'

  validates_presence_of :duration,
    :message => 'Please specify the sprint duration.'

  validates_numericality_of :duration,
    :greater_than_or_equal_to => 1,
    :message => 'The duration just be a positive number.'

  validates_presence_of :goals,
    :message => 'Please list the goals for this sprint.'

  validates_presence_of :team_lead_id,
    :message => 'A sprint must have a designated team lead.'

  after_update :close_user_stories

  belongs_to :product
  belongs_to :team_lead, :class_name => 'User', :foreign_key => :team_lead_id

  # sprint membership
  has_many :sprint_members, :dependent => :destroy
  has_many :members, :through => :sprint_members

  has_many :backlog_items,
    :order => '(select priority from user_stories where id = user_story_id) ASC',
    :dependent => :destroy
  has_many :user_stories, :through => :backlog_items, :order => :priority

  STATE_PLANNING='planning'
  STATE_ACTIVE='active'
  STATE_COMPLETED='completed'
  STATE_CANCELLED='cancelled'

  STATES = {
    STATE_PLANNING  => "Planning",
    STATE_ACTIVE    => "Active",
    STATE_COMPLETED => "Completed",
    STATE_CANCELLED => "Cancelled"}

  named_scope :default, { :order => "starts ASC" }
  named_scope :for_product, lambda { |product_id|
    { :conditions => product_id ? ["product_id = ?", product_id]: []}
  }
  named_scope :planned, {:conditions => ['state = ?', STATE_PLANNING]}
  named_scope :active,  {:conditions => ['state = ?', STATE_ACTIVE]}

  state_machine :initial => :planning do
    state :planning,  :value => STATE_PLANNING
    state :active,    :value => STATE_ACTIVE
    state :completed, :value => STATE_COMPLETED
    state :cancelled, :value => STATE_CANCELLED

    event :start do
      transition [:planning, :completed, :cancelled] => :active

      def can_fire?(sprint)
        (sprint.planning? || sprint.completed? || sprint.cancelled?) && Sprint.for_product(sprint.product).active.empty?
      end
    end

    event :plan do
      transition :active => :planning

      def can_fire?(sprint)
        sprint.active? && Sprint.for_product(sprint.product).planned.empty?
      end
    end

    event :complete do
      transition :active => :completed

      def can_fire?(sprint)
        sprint.active?
      end
    end

    event :cancel do
      transition :active => :cancelled

      def can_fire?(sprint)
        sprint.active?
      end
    end
  end

  def state_text
    STATES[state]
  end

  # Returns the end date for the sprint.
  def end_date
    self.starts + (self.duration - 1)
  end

  # Returns the total estimated hours.
  def estimated_hours
    result = 0.0

    backlog_items.each { |story| result += story.estimated_hours }

    return result
  end

  # Returns the actual hours logged on this sprint.
  def actual_hours
    result = 0.0

    backlog_items.each { |story| result += story.actual_hours }

    return result
  end

  # Returns the number of hours remaining. Calculated by subtracting the actual
  # hours worked from the estimated hours for the sprint's backlog.
  def remaining_hours
    backlog_items.inject(0) { |sum, item| sum + item.remaining_hours}
  end

  # Returns the number of hours remaining for the specified day.
  def remaining_hours_for_day(day)
    backlog_items.inject(0) { |sum, item| sum + item.remaining_hours_for_date(starts + day) }
  end

  # Returns true of all backlog items have been accepted and none have been rejected.
  def accepted?
    result = true
    backlog_items.each { |item| result &= item.accepted? }
    return result
  end

  # Returns whether the specified user can modify this sprint.
  def can_edit?(user)
    (active? || planning?) && (is_product_owner(user) || is_team_lead?(user))
  end

  # returns whether the user can change this sprint's state.
  def can_change_state?(user)
    (is_product_owner(user) || is_team_lead?(user))
  end

  # Returns whether the user can delete the current sprint.
  def can_delete?(user)
    is_product_owner(user)
  end

  # Returns whether the specified user is allowed to populate this sprint.
  def can_populate?(user)
    planning? && (is_product_owner(user) || is_team_lead?(user))
  end

  # Returns whether a burndown chart can be viewed for this sprint.
  def can_view_burndown?
    !planning?
  end

  # Rethers whether the sprint can have backlog items added, and whether the
  # specified user can add one.
  def can_add_backlog_items?(user)
    (is_product_owner(user) || is_team_lead?(user)) && active?
  end

  # Returns the data as of the given day into the sprint.
  def burndown_data
    data = []

    (0..duration).each { |day| data[day] = remaining_hours_for_day(day) }

    return data
  end

  # Returns whether the sprint is in a healthy state or not.
  def healthy?
    days_into_sprint = Date.today - starts

    if (0..duration).include? days_into_sprint

      target = ((duration - days_into_sprint) + 1) * (estimated_hours / duration)

      remaining = backlog_items.inject(0) { |sum, item| sum += item.remaining_hours }

      return true if (0..(target * 1.10)).include? remaining
    end

    return false
  end

  # Returns whether the user is a team member.
  def is_team_member?(user)
    if (user != nil) && members
      return members.include?(user)
    end

    false
  end

  def is_team_lead?(user)
    user != nil && user.id == team_lead_id
  end

 private

  def is_product_owner(user)
    user != nil && user.id == product.owner_id
  end

  # When the user close a sprint, every user stories are close if the related
  # backlog item are completed.
  def close_user_stories
    return unless state == 'closed'

    BacklogItem.transaction do
      self.backlog_items.each do |backlog_item|
        if backlog_item.completed?
          backlog_item.user_story.closed = true
          backlog_item.user_story.save
        end
      end
    end
  end
end
