# task.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# A +Task+ represents a single piece of work associated with a +UserStory+.
# Tasks have a primary developer, and an optional backup developer.
class Task < ActiveRecord::Base
  validates_presence_of :backlog_item_id

  belongs_to :primary, :class_name => 'User', :foreign_key => 'primary_id'
  belongs_to :backup,  :class_name => 'User', :foreign_key => 'backup_id'

  validates_presence_of :primary_id,
    :message => 'A primary developer must be defined.'

  validates_presence_of :description,
    :message  => 'You need to describe this task.'
  validates_length_of   :description,
    :maximum  => 1000,
    :too_long => 'Please shorten the description.'

  validates_presence_of :hours,
    :message => 'You must report the hours worked on this task.'
  validates_numericality_of :hours,
    :greater_than => 0,
    :message => 'Hours must be numeric and greater than 0.'

  validates_presence_of :when_entered,
    :message => 'You must specify when the task was performed.'

  belongs_to :backlog_item

  named_scope :default, { :order => 'when_created DESC' }
  named_scope :for_user, lambda { |user_id|
    {
      :conditions => user_id ? ['primary_id = ?', user_id] : []
    }
  }
  # Returns whether the user can edit the task.
  def can_edit?(user)
    user &&
      backlog_item.sprint.active? &&
      (primary_id == user.id)
  end

  # Returns whether the user can delete the task or not.
  def can_delete?(user)
    user &&
      (backlog_item.owner && (backlog_item.owner_id == user.id)) &&
      backlog_item.sprint.active?
  end

  protected

  def validate
    if primary && backup && (primary.id == backup.id)
      errors.add('backup_id',"The backup developer cannot also be #{primary.display_name}.")
    end
    if backlog_item
      unless (backlog_item.sprint.starts...(backlog_item.sprint.starts + backlog_item.sprint.duration)).include? self.when_entered
        errors.add('when_entered', 'Task must be done during the sprint.')
      end
    end
  end
end
