# user.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require 'avatar'

# +User+ represents a single user in the system.
#
class User < ActiveRecord::Base
  validates_presence_of   :email,
    :message => 'A valid email account must be supplied'
  validates_uniqueness_of :email,
    :message => 'That email address is already in use.'
  validates_presence_of :display_name,
    :message => 'No display name was provided.'

  has_one :privileges, :class_name => "UserPrivilege",  :dependent => :destroy
  has_many :product_roles, :dependent => :destroy
  has_one :verification, :class_name => "UserVerification", :dependent => :destroy
  has_many :backlog, :class_name => "BacklogItem", :foreign_key => 'owner_id'
  has_one :notifications, :dependent => :destroy
  has_many :posts, :class_name => 'Message', :foreign_key => 'author_id', :dependent => :destroy

  attr_accessor :password_confirmation
  validates_confirmation_of :password,
    :message => 'Passwords do not match.'

  named_scope :for_sprint, lambda { |sprint_id|
    {
      :conditions => ['id in (select member_id from sprint_members where sprint_id = ?)', sprint_id]
    }
  }
  named_scope :project_admins, {:conditions => 'id in (select user_id from user_privileges where admin_projects = true)'}

  def validate
    errors.add_to_base("Missing password") if hashed_password.blank?
  end

  def password # virtual field
    @password
  end

  def password=(password)
    @password = password

    return if password.blank?

    create_new_salt
    self.hashed_password = User.encrypted_password(self.password, self.salt)
  end

  # Return a generated password which contains 8 caracters
  def self.generate_password
    Base64.encode64(Digest::SHA1.digest("#{rand(1<<64)}/#{Time.now.to_f}"))[0..7]
  end

  # Reset the user's password with a new generated password
  def generate_new_password!
    new_password = self.class.generate_password
    self.password = new_password
    self.save ? new_password : false
  end

  # Performs authentication using the provided email and password. If the user
  # is authenticated, then their user object is returned.
  def self.authenticate(email, password)
    result = User.find_by_email(email)

    if result
      expected_hash = encrypted_password(password, result.salt)

      if result.hashed_password != expected_hash
        result = nil
      end
    end

    result
  end

  # Attempts to verify the user by checking the supplied token against the
  # existing verification token.
  def verify(token)
    raise Exception.new("User has no pending verifications!") if verification == nil

    verification.token == token
  end

  # Returns whether the user has been verified.
  def verified?
    verification == nil
  end

  # Returns whether the user can edit this account.,
  def can_edit?(user)
    user && ((user.id == id) || user.privileges.admin_users)
  end

  # Returns whether the user can create other users.
  def create_users?
    privileges.admin_users
  end

  # Returns whether the user is allowed to create projects.
  def can_create_projects?
    privileges.admin_projects
  end

  # Returns whether the user can modify the specified project.
  def can_modify_project? project
    (id == project.owner.id) || privileges.admin_projects
  end

  # Returns whether the user has an avatar or not.
  def avatar?
    use_gravatar || (!url_for_avatar.nil? && !url_for_avatar.empty?)
  end

  # Returns the user's avatar url, or +nil+ if none was defined.
  def avatar_url
    result = nil

    if !use_gravatar && !url_for_avatar.nil? && !url_for_avatar.empty?
      result = url_for_avatar
    elsif use_gravatar
      result = (!url_for_avatar.nil? && !url_for_avatar.empty?) ? url_for_avatar : email
      result = gravatar_url(result)
    end

    # load the default avatar if all else failed
    unless result
      result = ConfigProperty.fetch("image.default_user_avatar")
    end

    return result
  end

  private

  def self.encrypted_password(password, salt)
    string_to_hash = password + "farkle" + salt
    Digest::SHA1.hexdigest(string_to_hash)
  end

  def create_new_salt
    self.salt = self.object_id.to_s + rand.to_s
  end
end
