# user_privileges.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

class UserMailer < ActionMailer::Base
  # Sends an email verification to the specified user.
  def email_verification(user, token)
    recipients user.email
    from       MAIL_CONFIG[:from]
    subject    "Email verification"
    body       :user => user, :token => token
  end

  # Sends an email to the old address when the user changes his email address.
  def email_change_notification(user, old_email)
    recipients old_email
    from       MAIL_CONFIG[:from]
    subject    "Email change notification"
    body       :user => user, :old_email => old_email
  end

  # Send an e-mail to an user and notify him his new password.
  def new_generated_password(user, new_password)
    recipients  user.email
    from        MAIL_CONFIG[:from]
    subject     "Your password"
    body        :user => user, :new_password => new_password
  end

  # Sends an email to the site admins informing them that a project has been requested.
  def project_requested(admins, project)
    recipients admins.map { |admin| admin.email }
    from       MAIL_CONFIG[:form]
    subject    "Project requested: #{project.name}"
    body       :project => project
  end

  # Sends an email to a project requestor about the disposition of the project.
  def project_request(project, approved)
    recipients project.owner.email
    from       MAIL_CONFIG[:from]
    subject    "Request project: #{project.name}"
    body       :project => project, :approved => approved
  end

  # Sends an email to a product owner when a user requests a product role.
  def product_role_request(requested_role)
    recipients requested_role.product.owner.email
    from       MAIL_CONFIG[:from]
    subject    "Role request for #{requested_role.product.name}..."
    body       :requested_role => requested_role
  end

  # Sends an email to the user letting them know the disposition for their
  # product role request.
  def product_role_disposition(role)
    recipients role.user.email
    from       MAIL_CONFIG[:from]
    subject    "Role request status..."
    body       :role => role
  end

  # Sends an email to the sprint team lead when a developer has cast votes
  # on backlog items for a planned sprint.
  def votes_cast(sprint, developer, votes)
    recipients sprint.team_lead.email
    cc         developer.email
    from       MAIL_CONFIG[:from]
    subject    "Votes cast: #{sprint.title}"
    body       :sprint => sprint, :developer => developer, :votes => votes
  end

  # Sends an email to a user letting him know his daily activities.
  def daily_updates(user, cc_list, open_items, completed_items, task_performed)
    recipients user.email
    cc         cc_list
    from       MAIL_CONFIG[:from]
    subject    "Daily updates for #{user.display_name}"
    body       :user => user, :open_items => open_items,
               :completed_items => completed_items, :tasks_performed => task_performed
  end

  # Send an email to a user to notify him that no activity has been detected in
  # his backlog.
  def no_activity_recorded(user, backlog_items)
    recipients  user.email
    from        MAIL_CONFIG[:from]
    subject     "No activity has been detected"
    body        :user => user, :backlog_items => backlog_items
  end

  # Send an email to a user to notify him that no task has been detected in his
  # backlog.
  def no_task_recorded(user, backlog_items)
    recipients  user.email
    from        MAIL_CONFIG[:from]
    subject     "No task has been detected"
    body        :user => user, :backlog_items => backlog_items
  end

  # Send an email to a user to notify him the status of each sprint of his
  # products.
  def sprints_products_status(user, products)
    recipients  user.email
    from        MAIL_CONFIG[:from]
    subject     "Status of sprints"
    body        :user => user, :products => products
  end

  # Sends an email to a message posted that an reply was posted to their comment.
  def message_reply_notice(reply)
    recipients reply.reference.author.email
    from       MAIL_CONFIG[:from]
    subject    "Someone replied to your comment..."
    body       :reply => reply
  end
end
