# user_story.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

# +UserStory+ represents a single user story.
#
# A user story can be related back to an +Epic+ story, which is a project level
# feature. User stories are the expression of that epic within a single product.
#
# A user story can have one or more +BacklogItem+ objects that refer to it.
#
class UserStory < ActiveRecord::Base
  validates_presence_of(:product_id,
                        :message => 'All user stories must belong to a product.')
  validates_presence_of(:priority,
                        :message => 'A user story must have a priority.')
  validates_numericality_of(:priority,
                            :message => 'The priority must be an integer value',
                            :only_integer => true,
                            :greater_than => 0)
  validates_presence_of(:title,
                        :message => 'You must include a title for the user story.')
  validates_length_of(:title,
                      :message => 'The title must be at least 5 characters long.',
                      :minimum => 1)

  belongs_to :product
  belongs_to :epic
  has_many :backlog_items
  has_and_belongs_to_many :comments, :join_table => :user_stories_messages, :class_name => 'Message'

  named_scope :default, {:order => 'priority ASC'}
  named_scope :for_product, lambda {|product_id| {:conditions => product_id ? ["product_id = ?", product_id] : []}}
  named_scope :for_epic, lambda {|epic_id| {:conditions => ['epic_id = ?', epic_id]}}
  named_scope :not_in_sprint, lambda {|sprint_id| {:conditions => ['id not in (select user_story_id from backlog_items where sprint_id = ?)', sprint_id]}}
  named_scope :open, {:conditions => 'closed = false'}

  # Returns whether the user can edit this user story.
  def can_edit?(user)
    user && (user.id == product.owner_id)
  end

  # Returns whether the user can comment on this user story.
  def can_comment?(user)
    product.is_member?(user)
  end

  # Returns whether the user can delete this user story.
  def can_delete?(user)
    user && (user.id == product.owner_id) && can_be_deleted?
  end

  # Returns whether the user story can be deleted.
  def can_be_deleted?
    backlog_items.empty?
  end

  # Returns whether the user story is closed.
  def closed?
    closed
  end

  def selectable_title
    return "#{title} (#{priority})"
  end
end
