# schedules.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

require 'rufus/sc/scheduler'

# Load the schedules configuration
schedules_file_path = "#{RAILS_ROOT}/config/schedules.yml"

if not File.exist?(schedules_file_path)
    raise "Schedule configuration file not found at #{schedules_file_path}"
end

SCHEDULES_CONFIG = YAML.load(File.open(schedules_file_path))

# Initialize a threads array
threads = {}

threads["daily updates"] = Thread.new do
  scheduler = Rufus::Scheduler::PlainScheduler.start_new(:thread_name => "daily updates")

  hours = SCHEDULES_CONFIG[:daily_updates][:hours]
  minutes = SCHEDULES_CONFIG[:daily_updates][:minutes]

  scheduler.cron("#{minutes} #{hours} * * *") do
    open_items      = {}
    completed_items = {}
    tasks_performed = {}

    BacklogItem.find(:all).each do |item|
      owner = item.owner

      if item.completed? && item.updated_at.to_date == Date.yesterday
        completed_items[owner] ||= []
        completed_items[owner] << item
      elsif item.owner
        open_items[owner] ||= []
        open_items[owner] << item
      end
    end

    Task.find(:all).each do |task|
      if task.when_entered.to_date == Date.yesterday
        tasks_performed[task.primary] ||= []
        tasks_performed[task.primary] << task

        if task.backup
          tasks_performed[task.backup] ||= []
          tasks_performed[task.backup] << task
        end
      end
    end

    User.find(:all).each do |user|
      if user.notifications.daily_updates
        cc_list = []

        if user.notifications.updates_to_product_list
          tasks_performed[user].each do |task|
            product = task.backlog_item.sprint.product
            if product.has_mailing_list? && !cc_list.include?(product.mailing_list)
              cc_list << product.mailing_list
            end
          end
        end

        UserMailer.deliver_daily_updates(user, cc_list, open_items[user],
          completed_items[user], tasks_performed[user])
      end
    end
  end

  scheduler.join
end

threads["daily reminders"] = Thread.new do
  scheduler = Rufus::Scheduler::PlainScheduler.start_new(:thread_name => "daily reminders")

  hours = SCHEDULES_CONFIG[:no_activity_or_no_task_recorded_time][:hours]
  minutes = SCHEDULES_CONFIG[:no_activity_or_no_task_recorded_time][:minutes]

  scheduler.cron("#{minutes} #{hours} * * *") do

    user_no_activity_on_backlog_items = {}
    user_no_tasks_on_backlog_items = {}

    BacklogItem.find_all_by_state(BacklogItem::STATE_ASSIGNED).each do |backlog_item|
      if backlog_item.tasks.empty?

        backlog_item_owner = backlog_item.owner
        user_no_tasks_on_backlog_items[backlog_item_owner] ||= []
        user_no_tasks_on_backlog_items[backlog_item_owner] << backlog_item

      else

        backlog_item.tasks.each do |task|
          if task.created_at.to_date < Date.yesterday
            backlog_item_owner = backlog_item.owner
            user_no_activity_on_backlog_items[backlog_item_owner] ||= []
            user_no_activity_on_backlog_items[backlog_item_owner] << backlog_item
          end
        end

      end
    end

    # Send the e-mail

    user_no_activity_on_backlog_items.each do |owner, backlog_items|
      UserMailer.deliver_no_activity_recorded(owner, backlog_items) if owner.notifications.task_reminders
    end

    user_no_tasks_on_backlog_items.each do |owner, backlog_items|
      UserMailer.deliver_no_task_recorded(owner, backlog_items) if owner.notifications.task_reminders
    end

  end

  scheduler.join
end

threads["product status"] = Thread.new do
  scheduler = Rufus::Scheduler::PlainScheduler.start_new(:thread_name => "product status")

  hours = SCHEDULES_CONFIG[:sprints_products_status_time][:hours]
  minutes = SCHEDULES_CONFIG[:sprints_products_status_time][:minutes]

  scheduler.cron("#{minutes} #{hours} * * *") do
    user_with_products = {}

    Product.find(:all).each do |product|
      product_owner = product.owner
      user_with_products[product_owner] ||= []
      user_with_products[product_owner] << product
    end

    user_with_products.each do |owner, products|
      UserMailer.deliver_sprints_products_status(owner, products)
    end
  end

  scheduler.join
end

threads["user verification expiration"] =  Thread.new do
  scheduler = Rufus::Scheduler::PlainScheduler.start_new(:thread_name => "user verification expiration")

  hours = SCHEDULES_CONFIG[:user_verification_expiration][:hours]
  minutes = SCHEDULES_CONFIG[:user_verification_expiration][:minutes]

  scheduler.cron("#{minutes} #{hours} * * *") do
    UserVerification.find(:all).each do |verification|
      # if the account has been validated too often, then delete it, otherwise
      # generate a new email and send it.
      if verification.expired?
        UserVerification.transaction do
          verification.user.destroy
        end
      else
        begin
          UserVerification.transaction do
            UserMailer.deliver_email_verification(verification.user, verification.token)
            verification.attempts = verification.attempts + 1
            verification.sent = Date.today
            verification.save
          end
        rescue Exception => error
          puts "Unable to send verification email: #{error.message}"
        end
      end
    end
  end
  scheduler.join
end

# Run all threads
threads.each_pair do |key, thread|
  puts "Starting scheduler for \"#{key}\"..."

  begin
    thread.run
  rescue Exception => error
    puts "Error starting thread: #{error.message}"
    puts error.backtrace
  end
end
