# stories_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class EpicsControllerTest < ActionController::TestCase
  fixtures :projects
  fixtures :users

  def setup
    @project = projects(:projxp)
    @owner = @project.owner
    @epic = epics(:login_epic)
    raise "Epic is not in this project!" unless @epic.project_id == @project.id

    @open_epic = @epic
    raise "Open epic cannot be closed!" if @open_epic.closed
    raise "Open epic is not in this project!" unless @open_epic.project_id == @project.id

    @closed_epic = epics(:closed_epic)
    raise "Closed epic must be closed!" unless @closed_epic.closed
    raise "Closed epic is not in this project!" unless @closed_epic.project_id == @project.id

    @epic_with_no_stories = epics(:no_user_story_epic)
    raise "Epic must not have user stories!" unless @epic_with_no_stories.user_stories.empty?

    @other_project = projects(:teatime)
    @other_owner = @other_project.owner

    @unapproved_project = projects(:unapproved_project)
    raise "Project cannot be approved!" if @unapproved_project.approved
    raise "Project owner is wrong!" unless @unapproved_project.owner_id == @owner.id

    @member = users(:mcpierce)
    raise "Member must have a product role!" unless @project.is_member?(@member)
    @nonmember = users(:celliot)
    raise "Nonmember must not have a product role!" if @project.is_member?(@nonmember)

    @nonowner = users(:mcpierce)
    raise "Nonowner and owner cannot be the same user!" if @owner.id == @nonowner.id

    @new_epic = {:title => 'This is a new epic', :priority => 1}
    @new_comment = {:body => "This is a comment"}
  end

  # Ensures that viewing a list of epics requires a valid project.
  def test_index_with_invalid_project
    get :index, {:project => 9999}

    assert_redirected_to error_path
  end

  # Ensures that viewing epics for a specific project limits those displayed.
  def test_index_with_project
    get :index, {:project => @project.id}

    assert_response :success
    assert_template 'epics/index'
    assert assigns['epics'], "Failed to load any epics."
    assigns['epics'].each {|epic| assert_equal @project.id, epic.project_id, "Only those epics for the specified project should have been loaded."}
  end

  # Ensures that viewing a list of epics works as expected.
  def test_index
    get :index

    assert_response :success
    assert assigns['epics'], "Failed to load any epics."
  end

  # Ensures that a valid epic id is required.
  def test_show_with_invalid_epic
    get :show, {}

    assert_redirected_to error_path
  end

  # Ensures that showing an epic works as expected.
  def test_show
    get :show, {:id => @epic.id}

    assert_response :success
    assert assigns['epic'], "Failed to load the epic."
    assert_equal @epic.id, assigns['epic'].id, "Loaded the wrong epic."
  end

  # Ensures that showing an epic for a team member gets a comment.
  def test_show_for_member
    get :show, {:id => @epic.id}, {:user_id => @member.id}

    assert_response :success
    assert assigns['epic'], "Failed to load the epic."
    assert_equal @epic.id, assigns['epic'].id, "Loaded the wrong epic."
    assert assigns['comment'], "A comment should have been created."
  end

  # Ensures that anonymous users cannot create new epics.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_path
  end

  # Ensures that a valid project is required to create an epic.
  def test_new_with_invalid_project
    get :new, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that non-owners can't create new epics.
  def test_new_as_nonowner
    get :new, {:project => @project.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that creating a new epic works as expected.
  def test_new
    get :new, {:project => @project.id}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['epic'], "Failed to create a new epic."
  end

  # Ensures that an anonymous user cannot create a new epic.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_path
  end

  # Ensures that a valid project is required to create a new epic.
  def test_create_with_invalid_project
    post :create, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that an unapproved project cannot have epics created.
  def test_create_for_unapproved_project
    post :create, {:project => @unapproved_project.id, :epic => @new_epic}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that a non-owner cannot create a new epic.
  def test_create_as_nonowner
    post :create, {:project => @project.id, :epic => @new_epic}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that a malformed epic redisplays the edit window.
  def test_create_with_invalid_epic
    post :create, {:project => @project.id, :epic => {:title => "Test"}}, {:user_id => @owner.id}

    assert_response :success
    assert_template "edit"
  end

  # Ensures that a well-formed epic is saved.
  def test_create
    post :create, {:project => @project.id, :epic => @new_epic}, {:user_id => @owner.id}

    result = Epic.find_by_title(@new_epic[:title])
    assert result, "The new epic was not saved."
    assert_redirected_to epics_path(:project => @project)
  end

  # Ensures that anonymous users cannot edit epics.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_path
  end

  # Ensures that a valid epic id is required.
  def test_edit_with_invalid_epic
    get :edit, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that a non-owner cannot edit an epic.
  def test_edit_as_nonowner
    get :edit, {:id => @epic.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that editing loads the epic story correctly.
  def test_edit
    get :edit, {:id => @epic.id}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['epic'], "Failed to load the epic."
    assert_equal @epic.id, assigns['epic'].id, "Failed to load the correct epic."
  end

  # Ensures that anonymous users cannot update epics.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_path
  end

  # Ensures that a valid epic id is required.
  def test_update_with_invalid_epic
    put :update, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the project owner can update an epic.
  def test_update_as_nonowner
    put :update,
    {:project_id => @project.id, :id => @epic.id, :epic => {:title => "test"}},
    {:user_id => @nonowner.id}

    assert_redirected_to error_path
    result = Epic.find_by_id(@epic.id)
    assert_equal @epic.title, result.title, "Epic should not have been updated."
  end

  # Ensures that a malformed epic sends the user back to the edit page.
  def test_update_with_invalid_epic
    put :update,
    {:id => @epic.id, :epic => {:title => ''}},
    {:user_id => @owner.id}

    assert_response :success
    assert_template "edit"
    result = Epic.find_by_id(@epic.id)
    assert_equal @epic.title, result.title, "Epic should not have been updated."
  end

  # Ensures that epics are updated as expected.
  def test_update
    put :update,
    {:id => @epic.id, :epic => {:title => "test"}},
    {:user_id => @owner.id}

    assert_redirected_to epic_path(@epic)
    result = Epic.find_by_id(@epic.id)
    assert_equal "test", result.title, "Epic was not updated as expected."
  end

  # Ensures that anonymous users cannot delete epics.
  def test_destroy_as_anonymous
    delete :destroy

    assert_redirected_to login_path
  end

  # Ensures that a valid epic is required.
  def test_destroy_with_invalid_epic
    delete :destroy, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that epics cannot be deleted by non-owners.
  def test_destroy_as_nonowner
    delete :destroy, {:id => @epic.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
    result = Epic.find_by_id(@epic.id)
    assert result, "Epic should not have been deleted."
  end

  # Ensures that an epic can be deleted.
  def test_destroy
    delete :destroy, {:id => @epic_with_no_stories.id}, {:user_id => @owner.id}

    assert_redirected_to epics_path(:project => @project.id)
    result = Epic.find_by_id(@epic_with_no_stories.id)
    assert_nil result, "Epic should have been deleted."
  end

  # Ensures that anonymous users cannot mark epics completed.
  def test_close_as_anonymous
    put :close

    assert_redirected_to login_path
  end

  # Ensures that a valid epic id is required.
  def test_close_with_invalid_epic
    put :close, {:project_id => @project.id}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can mark a project completed.
  def test_close_as_nonowner
    put :close, {:id => @epic.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that only a closed epic cannot be closed again.
  def test_closed_with_closed_epic
    put :close, {:id => @closed_epic.id}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that closing an epic works as expected.
  def test_close
    put :close, {:id => @epic.id}, {:user_id => @owner.id}

    assert_redirected_to epics_path(:project => @project.id)
    assert Epic.find_by_id(@epic.id).closed, "Epic should have been marked as closed."
  end

  # Ensures that anonymous users cannot reopen a closed epic.
  def test_reopen_as_anonymous
    put :reopen

    assert_redirected_to login_path
  end

  # Ensures that a valid epic id is required.
  def test_reopen_with_invalid_epic
    put :reopen, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can reopen a closed epic.
  def test_reopen_as_nonowner
    put :reopen, {:id => @closed_epic.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that only an open epic cannot be reopened.
  def test_reopen_with_open_epic
    put :reopen, {:id => @epic.id}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that reopening an epic works as expected.
  def test_reopen
    put :reopen, {:id => @closed_epic.id}, {:user_id => @owner.id}

    assert_redirected_to epic_path(@closed_epic)
    assert !Epic.find_by_id(@closed_epic.id).closed, "Epic should have been reopened."
  end

  # Ensures that anonymous user's cannot post comments.
  def test_comment_as_anonymous
    post :comment, {:message => @new_comment}

    assert_redirected_to login_path
  end

  # Ensures that a valid id is required.
  def test_comment_with_invalid_id
    post :comment, {:message => @new_comment}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that a nonmember cannot comment on epics.
  def test_comment_as_nonmember
    count = @epic.comments.size
    post :comment, {:id => @epic.id, :message => @new_comment}, {:user_id => @nonmember.id}

    assert_redirected_to epic_path(@epic)
    result = Epic.find_by_id(@epic.id)
    assert_equal count, result.comments.size, "Comment should not have been posted."
  end

  # Ensures that a bad comment is redisplayed.
  def test_comment_with_invalid_comment
    count = @epic.comments.size
    @new_comment[:body] = ''
    post :comment, {:id => @epic.id, :message => @new_comment},{:user_id => @member.id}

    assert_template 'epics/show'
    result = Epic.find_by_id(@epic.id)
    assert_equal count, result.comments.size, "Comment should not have been posted."
  end

  # Ensures that posting a comment works as expected.
  def test_comment
    count = @epic.comments.size
    post :comment, {:id => @epic.id, :message => @new_comment}, {:user_id => @member.id}

    assert_redirected_to epic_path(@epic)
    result = Epic.find_by_id(@epic.id)
    assert_equal count + 1, result.comments.size, "Comment should have been saved."
  end
end
