# projects_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class ItemsControllerTest < ActionController::TestCase
  fixtures :backlog_items
  fixtures :products
  fixtures :sprints
  fixtures :tasks
  fixtures :user_stories
  fixtures :users

  def setup
    @product = products(:projxp_web)
    @other_product = products(:projxp_web_services)
    raise "Product and other cannot be the same!" if @product.id == @other_product.id

    @user_story = user_stories(:freerange_user_story)
    raise "User story not found!" unless @user_story

    @owner = @product.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same person." if @owner.id == @nonowner.id

    @sprint  = sprints(:active_sprint)
    raise "Product and sprint are mismatched!" unless @product.id == @sprint.product_id
    @team_lead = @sprint.team_lead

    @non_team_lead = users(:jdonuts)
    raise "Non-team lead cannot be the team lead!" if @team_lead.id == @non_team_lead.id

    @pending_sprint = sprints(:planned_sprint)
    raise "Sprint should be pending!" unless @pending_sprint.planning?

    @pending_item = backlog_items(:planned_sprint_backlog_item)
    raise "Pending item should be on a pending sprint." unless @pending_item.sprint_id == @pending_sprint.id

    @other_sprint = sprints(:planned_sprint)
    raise "Product and sprint are mismatched!" unless @product.id == @sprint.product_id

    @item = backlog_items(:owned_backlog_item)
    raise "Item and sprint are mismatched!" unless @item.sprint_id == @sprint.id

    @item_owner = @item.owner
    @item_nonowner = @owner
    raise "Item owner and nonowner cannot be the same person!" if @item_owner.id == @item_nonowner.id

    @completed_item = backlog_items(:closed_backlog_item)
    raise "Item must be marked closed." unless @completed_item.completed?

    @blocked_item = backlog_items(:blocked_backlog_item)
    raise "Item must be marked as blocked." unless @blocked_item.blocked
    @blocker_message = Message.blocker_messages(@blocked_item).last
    raise "The blocker message should have been loaded!" unless @blocker_message

    @new_backlog_item = {:user_story_id => @user_story.id}
  end

  # Ensures that viewing an index works as expected.
  def test_index_with_sprint
    get :index, {:sprint_id => @sprint.id}

    assert_response :success
  end

  # Ensures that a valid product is required.
  def test_index
    get :index

    assert_response :success
  end

  # Ensures that a valid backlog item is required.
  def test_show_with_invalid_item
    get :show, {}

    assert_redirected_to error_path
  end

  # Ensures the item can be shown.
  def test_show
    get :show, {:id => @item.id}

    assert_response :success
    assert assigns['backlog_item'], "Failed to load a backlog item."
    assert_equal @item.id, assigns['backlog_item'].id,
      "Failed to load the right backlog item."
    assert assigns['tasks'], "Failed to load the tasks for this item."
  end

  # Ensures that anonymous users cannot create a new backlog item.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_path
  end

  # Ensures that sprint must be specified.
  def test_new_with_invalid_sprint
    get :new, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the team lead can add a backlog item.
  def test_new_as_not_team_lead
    get :new, {:sprint => @sprint.id}, {:user_id => @non_team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that starting a new item works as expected.
  def test_new
    get :new, {:sprint => @sprint.id}, {:user_id => @team_lead.id}

    assert_response :success
    assert assigns['backlog_item'], "Failed to create a backlog item."
    assert_equal @sprint.id, assigns['backlog_item'].sprint_id, "Did not set the right sprint."
    assert assigns['backlog_item'].discovered?, "Item was not marked as discovered."
    assert assigns['user_stories'], "Failed to load the list of user stories."
  end

  # Ensures anonymous users can't create backlog items.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_path
  end

  # Ensures that a valids print must be defined.
  def test_create_with_invalid_sprint
    post :create, {}, {:user_id => @team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that only the team lead can add a backlog item.
  def test_create_as_non_team_lead
    post :create, {:sprint => @sprint.id},{:user_id => @non_team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that, if the backlog item is malformed, the user is reshown the edit page.
  def test_create_with_invalid_item
    @new_backlog_item[:user_story_id] = nil
    post :create, {:sprint => @sprint.id, :backlog_item => @new_backlog_item}, {:user_id => @sprint.team_lead_id}

    assert_response :success
    assert_template 'items/new'
    assert assigns['backlog_item'], "Failed to maintain the backlog item."
    assert_equal @sprint.id, assigns['backlog_item'].sprint_id, "Did not set the right sprint."
    assert assigns['backlog_item'].discovered?, "Item was not marked as discovered."
    assert assigns['user_stories'], "Failed to load the list of user stories."
  end

  # Ensures that creating an item works as expected.
  def test_create
    post :create, {:sprint => @sprint.id, :backlog_item => @new_backlog_item}, {:user_id => @sprint.team_lead_id}

    result = BacklogItem.find_by_sprint_id_and_user_story_id(@sprint.id, @user_story.id)
    assert result, "Failed to create backlog item."
    assert_redirected_to item_path(result)
    assert result.discovered?, "Item was not marked as discovered."
  end

  # Ensures that anonymous users can't delete.
  def test_destroy_as_anonymous
    delete :destroy

    assert_redirected_to login_path
  end

  # Ensures that the item is required.
  def test_destroy_with_invalid_item
    delete :destroy, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can delete.
  def test_destroy_as_nonowner
    get :destroy, {:id => @item.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
    assert BacklogItem.find_by_id(@item.id), "Item should not have been deleted."
  end

  # Ensures that only an item on a pending sprint can be deleted.
  def test_destroy_for_active_sprint
    get :destroy, {:id => @item.id}, {:user_id => @owner.id}

    assert_redirected_to error_path
    assert BacklogItem.find_by_id(@item.id), "Item should not have been deleted."
  end

  # Ensures that an item on a pending sprint can be deleted.
  def test_destroy
    get :destroy, {:id => @pending_item.id}, {:user_id => @owner.id}

    assert_redirected_to items_path(:sprint => @pending_item.sprint_id)
    assert !BacklogItem.find_by_id(@pending_item.id), "Item should have been deleted."
  end

  # Ensures that anonymous users can't set the blocked status for an item.
  def test_blocked_as_anonymous
    get :blocked

    assert_redirected_to login_path
  end

  # Ensures that an item id is required.
  def test_blocked_with_invalid_item_id
    get :block, {}, {:user_id => @item_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the item owner can set an item as blocked.
  def test_blocked_with_nonowner
    get :blocked, {:id => @item.id}, {:user_id => @item_nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that only an active item can be marked as blocked.
  def test_blocked_with_completed_item
    get :blocked, {:id => @completed_item.id}, {:user_id => @completed_item.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that, if the item's already blocked, the existing blocked message is loaded.
  def test_blocked_for_blocked_message
    get :blocked, {:id => @blocked_item.id}, {:user_id => @blocked_item.owner_id}

    assert_response :success
    assert assigns['message'], "Failed to load the message."
    assert_equal @blocker_message.id, assigns['message'].id, "Failed to load the correct item."
  end

  # Ensures that marking an item as blocked works as expected.
  def test_blocked
    get :blocked, {:id => @item.id},{:user_id => @item.owner_id}

    assert_response :success
    assert assigns['backlog_item'], "Failed to load backlog item."
    assert assigns['message'].new_record?, "A new, empty message should have been loaded."
  end

  # Ensures an anonymous user can't update an items blocked status.
  def test_update_blocked_as_anonymous
    put :update_blocked

    assert_redirected_to login_path
  end

  # Ensures that a valid item id is required.
  def test_update_blocked_with_invalid_id
    put :update_blocked, {}, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the item owner can update an item's blocked status.
  def test_update_blocked_as_nonowner
    put :update_blocked, {:id => @item.id}, {:user_id => @item_nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that only an active item can be updated.
  def test_update_blocked_with_completed_item
    put :update_blocked, {:id => @completed_item.id}, {:user_id => @completed_item.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that a blocked message cannot be reblocked.
  def test_update_blocked_with_blocked_item
    put :update_blocked, {:id => @blocked_item.id, :blocked => true, :message => {:body => 'farkle'}},{:user_id => @blocked_item.owner_id}

    assert_redirected_to item_path(@blocked_item)
    result = BacklogItem.find_by_id(@blocked_item.id)
    assert result.blocked?, "Item should still be marked as blocked."
    assert_equal "farkle", result.blocker_message.body, "Message should have been updated."
    assert_equal @blocked_item.blocker_message.id, result.blocker_message.id, "A new message should not have been created."
  end

  # Ensures that update the item's status works as expected.
  def test_update_blocked
    count = @item.sprint.product.rss_entries.size
    put :update_blocked,
    {:id => @item.id, :blocked => true, :message => {:posted => Time.current, :body => 'Message', :author_id => @item.owner_id}},
    {:user_id => @item.owner_id}

    assert_redirected_to item_path(@item)
    result = BacklogItem.find_by_id(@item.id)
    assert result.blocked, "Item should have been marked as blocked."
    assert_equal "Message", result.blocker_message.body, "The message was not properly saved."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end

  # Ensures that anonymous users cannot defer items.
  def test_defer_as_anonymous
    put :defer

    assert_redirected_to login_path
  end

  # Ensures that a valid item id is required.
  def test_defer_with_invalid_item_id
    put :defer, {}, {:user_id => @team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that deferring rules are applied.
  def test_defer_as_non_team_lead
    put :defer, {:id => @item.id}, {:user_id => @non_team_lead.id}

    assert_redirected_to error_path
  end

  # Ensure that deferring works as expected.
  def test_defer
    count = @item.sprint.product.rss_entries.size
    put :defer, {:id => @item.id}, {:user_id => @team_lead.id}

    assert_redirected_to items_path(:sprint => @item.sprint)
    result = BacklogItem.find_by_id(@item.id)
    assert result.deferred?, "Item should have been deferred."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end
end
