# Copyright (C) 2009, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class MessagesControllerTest < ActionController::TestCase
  def setup
    @user = users(:mcpierce)
    @message = messages(:blocker_message)
    @landing_page = dashboard_path

    @new_reply = {:body => 'This is a reply', :reference_id => @message.id}

    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []

    @emails = ActionMailer::Base.deliveries
    @emails.clear
  end

  # Ensures that viewing the list of messages works.
  def test_index
    get :index

    assert_response :success
    assert assigns['messages'], "Failed to load messages."
  end

  # Ensures that viewing all messages by a user works as expected.
  def test_index_with_author
    get :index, {:author => @user.id}

    assert_response :success
    assert assigns['messages'], "Failed toload any messages."
    assigns['messages'].each {|message| fail "Loaded messages for the wrong user." unless message.author_id == @user.id}
  end

  # Ensures that a valid id is require to view a message.
  def test_show_with_invalid_id
    get :show

    assert_redirected_to error_path
  end

  # Ensures that showing a message works as expected.
  def test_show
    get :show, {:id => @message.id}

    assert_response :success
    assert assigns['message'], "Failed to load a message."
    assert_equal @message.id, assigns['message'].id, "Failed to load the correct message."
  end

  # Ensures that entering a reply requires the user log in.
  def test_reply_as_anonymous
    get :reply

    assert_redirected_to login_path
  end

  # Ensures that entering a reply requires a valid id.
  def test_reply_with_invalid_id
    get :reply, { }, {:user_id => @user.id}

    assert_redirected_to error_path
  end

  # Ensures that editing a reply works as expected.
  def test_reply
    get :reply, {:id => @message.id}, {:user_id => @user.id}

    assert_response :success
    assert assigns['reply'], "Failed to create a reply object."
  end

  # Ensures that anonymous users cannot post replies.
  def test_post_reply_as_anonymous
    post :post_reply

    assert_redirected_to login_path
  end

  # Ensures that posting a reply requires a valid message id.
  def test_post_reply_with_invalid_message
    post :post_reply, { }, {:user_id => @user.id}

    assert_redirected_to error_path
  end

  # Ensures that, if the message is malformed, then the user is redirected to the src page.
  def test_post_reply_with_malformed_message
    post :post_reply, {:id => @message.id, :dest => @landing_page, :reply => {}}, {:user_id => @user.id}

    assert_template 'messages/reply'
  end

  # Ensures that a reply is posted as expected, and the user redirected to the landing page.
  def test_post_reply_with_landing_page
    post :post_reply, {:id => @message.id, :dest => @landing_page, :reply => @new_reply}, {:user_id => @user.id}

    assert_redirected_to @landing_page
    result = Message.find_by_id(@message.id).replies.last
    assert result, "Reply should have been saved."
    assert_equal @new_reply[:body], result.body, "Did not save the message."
    email = @emails.first
    assert email, "An email should have been sent."
    assert_equal @message.author.email, email.to[0], "Email was sent to the wrong user."
  end

  # Ensures that a landing page is provided.
  def test_post_reply
    post :post_reply, {:id => @message.id, :reply => @new_reply}, {:user_id => @user.id}

    assert_redirected_to message_path(@message)
    result = Message.find_by_id(@message.id).replies.last
    assert result, "Did not save the reply."
    assert_equal @new_reply[:body], result.body, "Did not save the message."
    # ensures that emails were sent
    email = @emails.first
    assert email, "An email should have been sent."
    assert_equal @message.author.email, email.to[0], "Email was sent to the wrong user."
  end

  # Ensures that a landing page is provided.
  def test_post_reply_with_emails_on_replies_disabled
    @message.author.notifications.email_on_replies = false
    @message.author.notifications.save!

    post :post_reply, {:id => @message.id, :reply => @new_reply}, {:user_id => @user.id}


    assert_redirected_to message_path(@message)
    result = Message.find_by_id(@message.id).replies.last
    assert result, "Did not save the reply."
    assert_equal @new_reply[:body], result.body, "Did not save the message."
    # ensures that emails were sent
    assert @emails.empty?, "No emails should have been sent."
  end
end
