# products_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

# +ProductsControllerTest+ performs unit tests on the +ProductController+ class.
#
class ProductsControllerTest < ActionController::TestCase
  fixtures :products
  fixtures :projects
  fixtures :roles
  fixtures :users

  def setup
    @project = projects(:projxp)
    @unapproved_project = projects(:unapproved_project)
    raise "Unapproved project cannot be approved!" if @unapproved_project.approved

    @project_owner = @project.owner
    @non_project_owner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same user!" if @project_owner.id == @non_project_owner.id

    @existing_product = products(:projxp_web)

    @product_owner = @existing_product.owner
    @non_product_owner = users(:jdonuts)
    raise "Owner and nonowner cannot be the same user!" if @product_owner.id == @non_product_owner.id
    raise "Non-product owner can't have a product role!" if @existing_product.is_member?(@non_product_owner)

    @member = users(:mcpierce)
    @nonmember = users(:jdonuts)
    @role = roles(:developer)

    @new_product = {
      :name        => 'product1',
      :owner_id    => @project_owner.id,
      :description => 'This is a new product.'
    }
    @product_update = {
      :id         => @existing_product.id.to_f,
      :name       => @existing_product.name + " Gold"
    }
  end

  # Ensures that only products for the given project are shown.
  def test_index_with_project
    all_products = Product.find_all_by_project_id(@project.id)

    get :index, {:project => @project.id}

    assert_response :success
    assert assigns['products'], "Failed to load any products."
    assigns['products'].each do |product|
      assert_equal @project.id, product.project_id,
        "Failed to load the right set of products."
    end
  end

  # Ensures that all products are shown.
  def test_index
    get :index

    assert_response :success
    assert assigns['products'], "Failed to load any products."
  end

  # Ensures that a product id is required,.
  def test_show_without_id
    get :show

    assert_redirected_to error_path
  end

  # Ensures that a valid product is id required.
  def test_show_with_invalid_id
    get :show, {:id => 9999}

    assert_redirected_to error_path
  end

  # Ensures that show works as expected.
  def test_show
    get :show, {:id => @existing_product.id}

    assert_response :success
    assert assigns['product'], "Failed to load the product."
    assert_equal @existing_product.id, assigns['product'].id,
      "Failed to load the right product."
    assert assigns['project'], "Failed to load the project."
    assert_equal @existing_product.project_id, assigns['project'].id,
      "Failed to load the right project."
    assert assigns['sprints'], "Failed to load the sprints for this product."
    assert assigns['user_stories'], "Failed to load the product backlog."
  end

  # Ensures an anonymous user cannot create a new product.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_url
  end

  # Ensures that a project id is required.
  def test_new_without_project_id
    get :new, {}, {:user_id => @project_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that a valid project is is required.
  def test_new_with_invalid_project_id
    get :new,
      {:project => 9999},
      {:user_id => @project_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that an unauthorized user cannot create a product.
  def test_new_as_unauthorized_user
    get :new,
      {:project => @project.id},
      {:user_id => @non_project_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that a product cannot be created for an unapproved project.
  def test_new_for_unapproved_project
    get :new,
    {:project => @unapproved_project.id },
    { :user_id => @unapproved_project.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that creating a project works as expected.
  def test_new
    get :new,
      {:project => @project.id},
      {:user_id => @project_owner.id}

    assert_response :success
    assert assigns['users'], "Failed to load the list of users."
  end

  # Ensures that anonymous users cannot edit a product.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_url
  end

  # Ensures that editing requires a product id.
  def test_edit_without_product_id
    get :edit, {}, {:user_id => @product_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that editing requires a valid product id.
  def test_edit_with_invalid_product_id
    get :edit,
      {:id => 9999},
      {:user_id => @product_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that nonowners can't edit a product.
  def test_edit_as_nonowner
    get :edit,
      {:id => @existing_product.id},
      {:user_id => @non_product_owner.id}

    assert_redirected_to product_path(@existing_product)
  end

  # Ensures that the project owner can edit a product.
  def test_edit_as_project_owner
    get :edit,
      {:id => @existing_product.id},
      {:user_id => @project_owner.id}

    assert_response :success
    assert assigns['product'], "Failed to load the product."
    assert_equal @existing_product.id, assigns['product'].id,
      "Failed to load the correct product."
    assert assigns['users'], "Failed to load the list of users."
  end

  # Ensures that editing works as expected.
  def test_edit
    get :edit,
      {:id => @existing_product.id},
      {:user_id => @product_owner.id}

    assert_response :success
    assert assigns['product'], "Failed to load the product."
    assert_equal @existing_product.id, assigns['product'].id,
      "Failed to load the correct product."
    assert assigns['users'], "Failed to load the list of users."
  end

  # Ensures that anonymous users cannot create a product.
  def test_create_as_anonymous
    put :create

    assert_redirected_to login_url
  end

  # Ensures that a product's details must be provided.
  def test_create_without_product_details
    put :create, {}, {:user_id => @project_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the project owner can call the create method.
  def test_create_as_non_project_owner
    put :create,
      {:product => @new_product, :project => @project.id},
      {:user_id => @non_project_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that creating a product work as expected.
  def test_create
    put :create,
      {:product => @new_product, :project => @project.id},
      {:user_id => @project_owner.id}

    assert_redirected_to product_path(assigns['product'])
    result = Product.find_by_name(@new_product[:name])
    assert result, "Failed to create the actual product."
    assert !result.rss_entries.empty?, "Product should have an RSS entry for its creation."
    assert result.is_member?(result.owner), "The product owner should have been made a product team member."
    role = ProductRole.find_by_product_id_and_user_id(result.id, result.owner_id)
    assert !role.pending, "The role should not be pending."
  end

  # Ensures that anonymous users can't update a product.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_url
  end

  # Ensures that an update without a product's details fails.
  def test_update_without_product
    put :update, {}, {:user_id => @product_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that a non-product owner can't update a product.
  def test_update_as_non_owner
    put :update,
      {:id      => @existing_product.id,
      :product => @product_update},
      {:user_id => @non_product_owner.id}

    assert_redirected_to product_path(@existing_product)
    result = Product.find_by_id(@existing_product.id)
    assert_equal @existing_product.name, result.name,
      "Name should not have changed."
  end

  # Ensures that updating a product works for the project owner.
  def test_update
    count = @existing_product.rss_entries.size
    put :update,{:id => @existing_product.id,:product => @product_update},{:user_id => @project_owner.id}

    assert_redirected_to product_path(@existing_product)
    result = Product.find_by_id(@existing_product.id)
    assert_equal @product_update[:name], result.name, "Name should have been updaed."
    assert_equal count + 1, result.rss_entries.size, "Failed to add a new rss entry."
    assert result.is_member?(result.owner), "The product owner should not have lost his role."
  end

  # Ensures that changing the product owner removes the old owner's product
  # role and creates a role for the new owner.
  def test_update_with_owner_change
    count = @existing_product.rss_entries.size
    old_owner = @existing_product.owner
    @product_update[:owner_id] = @non_product_owner.id
    put :update,{:id => @existing_product.id,:product => @product_update},{:user_id => @project_owner.id}

    assert_redirected_to product_path(@existing_product)
    result = Product.find_by_id(@existing_product.id)
    assert_equal @product_update[:name], result.name, "Name should have been updaed."
    assert_equal count + 1, result.rss_entries.size, "Failed to add a new rss entry."
    assert result.is_member?(result.owner), "The owner should have a product role."
    assert !result.is_member?(old_owner), "The old owner's role should have been removed."
  end

  # Ensures that anonymous users can't delete.
  def test_delete_as_anonymous
    delete :delete

    assert_redirected_to login_url
  end

  # Ensures that deletion is not supported.
  def test_delete
    delete :delete, {}, {:user_id => @product_owner.id}

    assert_redirected_to error_path
  end

  # Ensures that the RSS feed requires a valid product.
  def test_rss_with_invalid_product
    get :rss

    assert_redirected_to error_path
  end

  # Ensures that the RSS feed does not require authentication.
  def test_rss
    get :rss, {:id => @existing_product.id}

    assert_response :success
    assert assigns['entries'], "Failed to load any feed content."
  end
end
