# projects_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

# +ProjectsControllerTest+ performs tests on +ProjectController+ to ensure that
# it works as expected.
class ProjectsControllerTest < ActionController::TestCase
  fixtures :projects
  fixtures :users

  def setup
    @project            = projects(:projxp)
    @owner              = @project.owner
    @unapproved_project = projects(:unapproved_project)

    raise "Unapproved project cannot be approved!" if @unapproved_project.approved?

    @nonowner           = users(:mcpierce)
    raise "Owner and nonowner cannot be the same user!" if @owner.id == @nonowner.id

    @nonadmin = @nonowner
    raise "Nonadmin cannot have project admin rights!" if @nonadmin.can_create_projects?

    @admin    = users(:project_admin)
    raise "Nonadmin and admin cannot be the same user!" if @nonadmin.id == @admin.id
    raise "Admin must have project admin rights!" unless @admin.can_create_projects?

    # setup for testing mail
    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []

    @emails = ActionMailer::Base.deliveries
    @emails.clear
  end

  # Ensures that showing the project index works as expected.
  def test_index
    get :index

    assert_response :success
    assert assigns['projects'], "Didn't load any projects."
  end

  # Ensures that showing a single project requires an id.
  def test_show_without_id
    get :show

    assert_redirected_to error_url
  end

  # Ensures that showing a project fails on invalid ids.
  def test_show_with_invalid_id
    get :show, {:id => 9999}

    assert_redirected_to error_url
  end

  # Ensures tha show a project works as expected.
  def test_show
    get :show, {:id => @project.id}

    assert_response :success
    assert assigns['project'], "Failed to load the project."
    assert_equal @project.id, assigns['project'].id,
      "Failed to load the correct project."
  end

  # Ensures that an anonymous user cannot create a new project.
  def test_new_as_anonymous
    get :new

    assert_redirected_to :controller => :home, :action => :login
  end

  # Ensures that a user without project admin privileges can
  # create a project but that it is not approved.
  def test_new_as_nonadmin
    get :new, {}, {:user_id => @nonadmin.id}

    assert_response :success
    assert_equal 1, assigns['users'].size,
    "The only owner a non-admin can assign is himself!"
  end

  # Ensures that starting off a new project works as expected.
  def test_new
    get :new, {}, {:user_id => @admin.id}

    assert_response :success
  end

  # Ensures that the url is held.
  def test_new_with_url
    get :new, {"source" => "/farkle"}, {:user_id => @admin.id}

    assert_response :success
    assert assigns['project'], "Did not create a new project."
    assert_equal @admin.id, assigns['project'].owner_id,
      "Did not assign the default owner."
    assert assigns['users'], "Did not load the list of users."
    assert assigns['source'], "Didn't store the URL."
    assert_equal "/farkle", assigns['source'], "Failed to save the right url."
  end

  # Ensures an anonymous user cannot start editing a project.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to :controller => :home, :action => :login
  end

  # Ensures that edit fails on an invalid id.
  def test_edit_with_invalid_id
    get :edit,
      {:id => 9999},
      {:user_id => @owner.id}

    assert_redirected_to error_url
  end

  # Ensures that a nonowner cannot edit a project.
  def test_edit_as_nonowner
    get :edit,
      {:id => @project.id},
      {:user_id => @nonowner.id}

    assert_redirected_to project_path(@project)
  end

  # Ensures that the owner can edit a project.
  def test_edit
    get :edit,
      {:id => @project.id},
      {:user_id => @owner.id}

    assert_response :success
    assert assigns['project'], "Failed to load a project."
    assert_equal @project.id,assigns['project'].id,
      "Failed to load the correct project."
    assert !assigns['source'], "Should not have stored a source URL."
  end

  # Ensures that the source url is stored.
  def test_edit_with_url
    get :edit,
      {:id => @project.id, :source => "/farkle"},
      {:user_id => @owner.id}

    assert_response :success
    assert assigns['project'], "Failed to load a project."
    assert_equal @project.id,assigns['project'].id,
      "Failed to load the correct project."
    assert assigns['source'], "Failed to store the source url."
    assert_equal  "/farkle", assigns['source'], "Stored the wrong url."
  end

  # Ensures that an anonymous user cannot create a project.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_url
  end

  # Ensures that a non-admin can create an unapproved project.
  def test_create_as_nonadmin
    post :create,
      {:project => {:name => 'test', :owner_id => @admin.id}},
      {:user_id => @nonadmin.id}

    result = Project.find_by_name('test')
    assert_redirected_to project_url(result)
    assert result, "Project was not created!"
    assert !result.approved, "Project should not have been approved!"
  end

  # Ensures that an invalid project returns the user to the edit page.
  def test_create_with_invalid_project
    post :create,
      {:project => {}},
      {:user_id => @admin.id}

    assert_response :success
  end

  # Ensures that a project is created when one is submitted.
  def test_create
    post :create,
      {:project => {:name => 'test', :owner_id => @admin.id}},
      {:user_id => @admin.id}

    result = Project.find_by_name('test')
    assert_redirected_to project_url(result)
    assert result, "Project was not created!"
    assert result.approved, "Project should have been approved!"
  end

  # Ensures the user is returned to the origin page if a url was provided.
  def test_create_with_url
    post :create,
      {:project => {:name => 'test', :owner_id => @admin.id},
      :source => "/farkle"},
      {:user_id => @admin.id}

    assert_redirected_to "/farkle"
  end

  # Ensures that an anonymous user cannot update a project.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_url
  end

  # Ensures that a valid project id must be provided.
  def test_update_with_invalid_id
    put :update,
      {:id => 9999},
      {:user_id => @admin.id}

    assert_redirected_to error_url
  end

  # Ensures that a nonadmin cannot update a project.
  def test_update_by_nonowner
    put :update,
      {:id => @project.id,
      :project => {:name => 'newname'}},
      {:user_id => @nonowner.id}

    assert_redirected_to project_path(@project)
    result = Project.find_by_id(@project.id)
    assert_equal @project.name, result.name,
      "Project should not have been updated!"
  end

  # Ensures that the owner can edit a project.
  def test_update
    put :update,
      {:id => @project.id,
      :project => {:name => 'newname'}},
      {:user_id => @owner.id}

    assert_redirected_to project_path(@project)
    result = Project.find_by_id(@project.id)
    assert_equal 'newname', result.name,
      "Project should have been updated!"
  end

  # Ensures that the user is returned to the source url.
  def test_update_with_url
    put :update,
      {:id => @project.id,
      :project => {:name => 'newname'},
      :source => "/farkle"},
      {:user_id => @owner.id}

    assert_redirected_to "/farkle"
    result = Project.find_by_id(@project.id)
    assert_equal 'newname', result.name,
      "Project should have been updated!"
  end

  # Ensures that the destroy method just exists.
  def test_destroy
    delete :destroy, {:id => @project.id}, {:user_id => @admin.id}

    assert_redirected_to error_url
  end

  # Ensures that anonymous users can't approve projects.
  def test_approve_as_anonymous
    put :approve

    assert_redirected_to login_path
  end

  # Ensures that a project id is required to approve a project.
  def test_approve_requires_project_id
    put :approve, { }, {:user_id => @admin.id}

    assert_redirected_to error_path
  end

  # Ensures that a project cannot be approved by a non-admin.
  def test_approve_requires_admin_rights
    put :approve, {:id => @unapproved_project.id}, {:user_id => @nonadmin.id}

    assert_redirected_to error_path
    assert !Project.find_by_id(@unapproved_project.id).approved,
    "Project should not have been marked as approved."
  end

  # Ensures that an approved project raises an error.
  def test_approve_project_already_approved
    put :approve, {:id => @project.id}, {:user_id => @admin.id}

    assert_redirected_to project_path(@project)
    assert_equal @project.approved, Project.find_by_id(@project.id).approved,
    "Approval status should not have changed."
  end

  # Ensures that approvals work as expected.
  def test_approve
    put :approve, {:id => @unapproved_project.id},{:user_id => @admin.id}

    assert_redirected_to project_path(@unapproved_project)
    assert Project.find_by_id(@unapproved_project.id).approved,
    "Project should have been marked as approved."
    email = @emails.first
    assert email, "No email was generated."
    assert_equal @unapproved_project.owner.email, email.to[0], "Email sent to wrong user."
  end

  # Ensures that anonymous users can't reject a project.
  def test_reject_as_anonymous
    put :reject

    assert_redirected_to login_path
  end

  # Ensures that rejecting a project requires a valid id.
  def test_reject_requires_a_valid_id
    put :reject, {}, {:user_id => @admin.id}

    assert_redirected_to error_path
  end

  # Ensures that rejecting a project requires admin rights.
  def test_reject_requries_admin_rights
    put :reject, {:id => @unapproved_project.id}, {:user_id => @nonadmin.id}

    assert_redirected_to error_path
    assert Project.find_by_id(@unapproved_project.id),
    "Project should not have been rejected."
  end

  # Ensures that an approved project cannot be rejected.
  def test_reject_project_already_approved
    put :reject, {:id => @project.id}, {:user_id => @admin.id}

    assert_redirected_to project_path(@project)
    assert Project.find_by_id(@project.id),
    "Project should not have been deleted."
  end

  # Ensures that a rejected project is deleted.
  def test_reject
    put :reject, {:id => @unapproved_project.id}, {:user_id => @admin.id}

    assert_redirected_to projects_path
    assert_nil Project.find_by_id(@unapproved_project.id),
    "Rejected project should have been deleted."
    email = @emails.first
    assert email, "No email was generated."
    assert_equal @unapproved_project.owner.email, email.to[0], "Email sent to wrong user."
  end
end
