# roles_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class RolesControllerTest < ActionController::TestCase
  fixtures :product_roles
  fixtures :products
  fixtures :users

  def setup
    @product = products(:projxp_web)

    @owner = @product.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same person!" if @owner.id == @nonowner.id

    @active_role  = product_roles(:mcpierce_projxp)
    raise "Active roles should not be pending!" unless @active_role.active?
    @active_user = @active_role.user

    @pending_role = product_roles(:celliot_projxp)
    raise "Pending roles should not be active!" if @pending_role.active?
    @pending_user = @pending_role.user

    @user_with_no_role = users(:teatime_owner)
    raise "User should have no roles!" if ProductRole.find_by_product_id_and_user_id(@product.id, @user_with_no_role.id)

    @new_role = {
      :user_id    => @user_with_no_role.id,
      :product_id => @product.id,
      :role_id    => Role.find(:first).id
    }

    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []

    @emails = ActionMailer::Base.deliveries
    @emails.clear
  end

  # Ensures that anonymous users can only view approved roles.
  def test_index_as_anonymous
    get :index, {:product_id => @product.id}

    assert_response :success
    assert assigns['product_roles']
    assigns['product_roles'].each { |role| assert role.active?, "Only active roles are allowed." }
  end

  # Ensures that a valid product id must be provided.
  def test_index_with_invalid_product
    get :index, {:product_id => 9999}, {:user_id => @owner.id}

    assert_redirected_to error_url
  end

  # Ensures that the product owner sees all product roles.
  def test_index
    roles = ProductRole.find_all_by_product_id(@product.id)
    get :index, {:product_id => @product.id}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['product_roles']
    assert_equal roles.size, assigns['product_roles'].size,
      "Did not load the correct set of product roles."
  end

  # Ensures that a valid product id must be provided.
  def test_show_without_product_id
    get :show

    assert_redirected_to error_url
  end

  # Ensures that a valid role id must be provided.
  def test_show_without_role_id
    get :show, {:product_id => @product.id}

    assert_redirected_to product_path(@product)
  end

  # Ensures showing a role works as expected.
  def test_show
    get :show, {:product_id => @product.id, :id => @active_role.id}

    assert_response :success
  end

  # Ensures that an anonymous user cannot request a product role.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_url
  end

  # Ensures that creating a role requires a product id.
  def test_new_without_product_id
    get :new, {}, {:user_id => @user_with_no_role.id}

    assert_redirected_to error_url
  end

  # Ensures that create a role requires a valid id.
  def test_new_with_invalid_product_id
    get :new,
      {:product_id => 9999},
      {:user_id => @user_with_no_role.id}

    assert_redirected_to error_url
  end

  # Ensures that, if the user already has a role, they can't request a new one.
  def test_new_with_existing_role
    get :new,
      {:product_id => @product.id},
      {:user_id => @active_user.id}

    assert_redirected_to product_path(@product)
  end

  # Ensures that a user can request a new role.
  def test_new
    get :new,
      {:product_id => @product.id},
      {:user_id => @user_with_no_role.id}

    assert_response :success
    assert assigns['role'], "Failed to create a new role."
    assert_equal @user_with_no_role.id, assigns['role'].user_id,
      "Failed to associate the new role with the current user."
    assert assigns['roles'], "Failed to load the set of roles."
  end

  # Ensures that calling create fails for anonymous users.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_url
  end

  # Ensures that calling create fails without a product id.
  def test_create_without_valid_product_id
    post :create, {}, {:user_id => @user_with_no_role.id}

    assert_redirected_to error_url
  end

  # Ensures that a valid role has to be present.
  def test_create_without_an_invalid_role
    post :create,
      {:product_id => @product.id},
      {:user_id => @user_with_no_role.id}

    assert_redirected_to new_product_role_path(@product)
  end

  # Ensures that a user with a role can't request a new one.
  def test_create_for_current_member
    post :create,
      {:product_id => @product.id,
      :product_role => @new_role
    },
      {:user_id => @active_role.user_id}

    assert_redirected_to error_url
  end

  # Ensures that creating a role request works as expected.
  def test_create
    post :create,
      {:product_id => @product.id,
      :product_role => @new_role
    },
      {:user_id => @user_with_no_role.id}

    assert_redirected_to product_path(@product)
    result = ProductRole.find_by_product_id_and_user_id(@product.id, @user_with_no_role.id)
    assert result, "Failed to create a role."
    assert result.pending?, "User's role should be marked as pending."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @owner.email, email.to[0], "Email sent to wrong user."
  end

  # Ensures that anonymous users can't update a product role.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_url
  end

  # Ensures that a valid product id is required.
  def test_update_with_invalid_product
    put :update, {}, {:user_id => @owner.id}

    assert_redirected_to error_url
  end

  # Ensures that a valid role id is required.
  def test_update_with_invalid_role
    put :update,
      {:product_id => @product.id},
      {:user_id => @owner.id}

    assert_redirected_to product_path(@product)
  end

  # Ensures that only the product owner can update a product role.
  def test_update_as_nonowner
    put :update,
      {:product_id => @product.id,
      :id => @pending_role.id},
      {:user_id => @pending_user.id}

    assert_redirected_to product_path(@product)
  end

  # Ensures that updates work as expected when the role is not approved.
  def test_update_as_deny
    put :update,
      {:product_id => @product.id,
      :id => @pending_role.id,
      :product_role => {:approved => false}},
      {:user_id => @owner.id}

    assert_redirected_to product_roles_path(@product)
    assert !ProductRole.find_by_id(@pending_role.id),
      "Role should have been deleted."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @pending_role.user.email, email.to[0], "Email sent to wrong user."
  end

  # Ensures that updating a role works as expected.
  def test_update_as_deny
    put :update,
      {:product_id => @product.id,
      :id => @pending_role.id,
      :product_role => {:approved => true}},
      {:user_id => @owner.id}

    assert_redirected_to product_roles_path(@product)
    result = ProductRole.find_by_id(@pending_role.id)
    assert result, "Role should not have been deleted."
    assert result.approved, "Role should be marked as approved."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @pending_role.user.email, email.to[0], "Email sent to wrong user."
  end

  # Ensures that anonymous users can't delete roles.
  def test_delete_as_anonymous
    delete :destroy

    assert_redirected_to login_url
  end

  # Ensures that a valid product id is required.
  def test_delete_with_invalid_product_id
    delete :destroy, {}, {:user_id => @owner.id}

    assert_redirected_to error_url
  end

  # Ensures that a valid product role has to be provided.
  def test_delete_with_invalid_role_id
    delete :destroy,
      {:product_id => @product.id,
      :id => 9999},
      {:user_id => @owner.id}

    assert_redirected_to product_path(@product)
  end

  # Ensures that only the owner can delete a role.
  def test_delete_as_nonowner
    delete :destroy,
      {:product_id => @product.id,
      :id => @active_role.id},
      {:user_id => @nonowner.id}

    assert_redirected_to product_roles_path(@product)
    assert ProductRole.find_by_id(@active_role.id),
      "Role should not have been deleted."
  end

  # Ensures that deleting a role works as expected.
  def test_delete
    delete :destroy,
      {:product_id => @product.id,
      :id => @active_role.id},
      {:user_id => @owner.id}

    assert_redirected_to product_roles_path(@product)
    assert !ProductRole.find_by_id(@active_role.id),
      "Role should have been deleted."
  end
end
