# sprints_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class SprintsControllerTest < ActionController::TestCase
  fixtures :backlog_items
  fixtures :products
  fixtures :sprints
  fixtures :users

  def setup
    @member = users(:mcpierce)
    @nonmember = users(:celliot)

    @active_sprint = sprints(:active_sprint)
    raise "An active sprint must be active!" unless @active_sprint.active?
    raise "Product must have a planned sprint!" if Sprint.for_product(@active_sprint.product).planned.empty?
    raise "Member must be on this team!" unless @active_sprint.is_team_member?(@member)

    @plannable_sprint = sprints(:plannable_sprint)
    raise "Sprint must be in a non-planning state!" if @plannable_sprint.planning?

    @planned_sprint = sprints(:planned_sprint)
    raise "Sprint must be pending!" unless @planned_sprint.planning?
    raise "Products must have the same owner!" unless @planned_sprint.product.owner_id == @active_sprint.product.owner_id
    raise "Member must be a team member!" unless @planned_sprint.is_team_member?(@member)
    raise "Nonmember cannot be a team member!" if @planned_sprint.is_team_member?(@nonmember)

    @votes = {"#{@planned_sprint.backlog_items.first.id}" => "#{BacklogItemVote::ACK}"}
    @estimates = {"#{@planned_sprint.backlog_items.first.id}" => "5.0"}

    @startable_sprint = sprints(:startable_sprint)
    raise "Sprint must be in the planned start!" unless @startable_sprint.planning?
    raise "Product cannot have an active sprint!" unless Sprint.for_product(@startable_sprint).active.empty?

    @product = @active_sprint.product
    @other_product = products(:projxp_web_services)
    raise "The products cannot be the same!" if @product.id == @other_product.id

    @nonowner = users(:verified_user)
    raise "Nonowner cannot own a product!" unless Product.for_owner(@nonowner).empty?

    @product_nonmember = users(:jdonuts)
    raise "Nonmember cannot have product membership!" unless @product_nonmember.product_roles.empty?
    @other_member = users(:team_lead)
    raise "Other member must be a part of the same product team!" unless @product.is_member?(@other_member)

    @product_with_no_stories = products(:teatime_midp)
    raise "Product cannot have user stories!" unless @product_with_no_stories.user_stories.empty?

    @product_with_no_sprints = products(:teatime_brew)
    raise "Product cannot have anys sprints!" unless @product_with_no_sprints.sprints.empty?
    raise "Member must be on product team!" unless @product_with_no_sprints.is_member?(@member)

    raise "Nonowner is the product owner!" if @planned_sprint.product.owner_id == @nonowner.id
    raise "Nonwner is the team lead!" if @planned_sprint.team_lead_id == @nonowner.id

    @non_team_lead = users(:mcpierce)
    raise "Non-team lead cannot be a team lead!" if [@active_sprint.id, @plannable_sprint.id].include? @non_team_lead.id

    @new_sprint = {
      :title        => "New Sprint",
      :starts       => Date.today,
      :duration     => 28,
      :goals        => "Get lots of things done!",
      :team_lead_id => @product.owner_id
    }

    @user_story = user_stories(:create_user_story_web_service_api)

    @selected = [
      @user_story.id
    ]

    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []

    @emails = ActionMailer::Base.deliveries
    @emails.clear
  end

  # Ensures that a list of all sprints is viewable.
  def test_index
    get :index

    assert_response :success
  end

  # Ensures that a valid sprint id is required.
  def test_show_with_invalid_sprint_id
    get :show, { }

    assert_redirected_to error_path
  end

  # Ensures that showing a sprint works as expected.
  def test_show
    get :show, { :id => @active_sprint.id}

    assert_response :success
    assert assigns['sprint'], "Failed to load a sprint."
  end

  # Ensures that an anonymous user can't create a new sprint.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_path
  end

  # Ensures that a valid product id is required to create a new sprint.
  def test_new_with_invalid_product
    get :new, {}, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can create a new sprint.
  def test_new_as_nonowner
    get :new, {:product => @product_with_no_sprints.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that creating a new sprint works as expected.
  def test_new
    get :new, {:product => @product_with_no_sprints.id}, {:user_id => @product_with_no_sprints.owner_id}

    assert_response :success
    assert assigns['sprint'], "Failed to create a new sprint."
    assert assigns['selected'], "Failed to create the default selection list."
    assert_equal @product_with_no_sprints.id, assigns['sprint'].product_id,
      "Did not set the product correctly."
  end

  # Ensures that an anonymous user cannot edit a sprint.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_path
  end

  # Ensures that a valid user sprint id is required.
  def test_edit_with_invalid_sprint
    get :edit, { }, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that editing a sprint works as expected.
  def test_edit
    get :edit, { :id => @active_sprint.id}, {:user_id => @active_sprint.product.owner_id}

    assert_response :success
    assert assigns['sprint'], "Failed to load a sprint."
    assert assigns['selected'], "Failed to load the selection list."
  end

  # Ensures that anonymous users can't create sprints.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_path
  end

  # Ensures that only the owner can create a sprint.
  def test_create_as_nonowner
    post :create, {:product => @product.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that an invalid sprint redirects to the edit page.
  def test_create_with_invalid_sprint
    post :create, {:product => @product_with_no_sprints.id, :sprint => {:title => "Invalid Sprint"}},
    {:user_id => @product_with_no_sprints.owner_id}

    assert_response :success
    assert !Sprint.find_by_title("Invalid Sprint"), "Should not have saved the sprint."
  end

  # Ensures that, if no members were selected, the sprint is created with an empty membership.
  def test_create_with_no_members
    post :create, {:product => @product_with_no_sprints.id, :sprint => @new_sprint},
    {:user_id => @product_with_no_sprints.owner_id}

    assert_redirected_to plan_sprint_path(assigns['sprint'])
    assert assigns['sprint'].members.empty?, "No members should be defined"
  end

  # Ensures that non-product members cannot be added to the sprint team.
  def test_create_with_non_product_member
    post :create, {:product => @product.id, :sprint => @new_sprint, :selected => [@product_nonmember.id]},
    {:user_id => @product.owner_id}

    assert_redirected_to error_path
    assert !Sprint.find_by_title(@new_sprint[:title]), "Sprint should not have been saved!"
  end

  # Ensures that a sprint cannot be created on a product with no user stories.
  def test_create_for_product_with_no_stories
    post :create, {:product => @product_with_no_stories.id, :sprint => @new_sprint},
    {:user_id => @product_with_no_stories.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that creating a sprint works as expected.
  def test_create
    rss_count = @product_with_no_sprints.rss_entries.size
    post :create,{:product => @product_with_no_sprints.id, :sprint => @new_sprint, :selected => [@member.id]},
      {:user_id => @product_with_no_sprints.owner_id}

    assert_redirected_to plan_sprint_path(assigns['sprint'])
    result = Sprint.find_by_title(@new_sprint[:title])
    assert result, "Failed to save the sprint."
    assert_equal 1, result.members.size, "Did not populate the sprint team correctly."
    assert_equal @member.id, result.members.first.id, "Did not put the right person in the team."
    assert_equal rss_count + 1, result.product.rss_entries.size, "Failed to add RSS entry for sprint creation."
  end

  # Ensures that anonymous users cannot update a sprint.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint id must be provided.
  def test_update_with_invalid_sprint
    put :update, { }, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that only the product owner can update a sprint.
  def test_update_as_nonowner
    put :update, { :id => @active_sprint.id, :sprint => {:title => "Don't do it"}},
      {:user_id => @nonowner.id}

    assert_redirected_to error_path
    result = Sprint.find_by_id(@active_sprint.id)
    assert_equal @active_sprint.title, result.title,
      "Sprint should not have been updated."
  end

  # Ensures that non-product members cannot be added to the sprint team.
  def test_update_with_non_product_member
    put :update, { :id => @active_sprint.id, :sprint => {}, :selected => [@product_nonmember.id]},
    {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that updating a sprint works as expected.
  def test_update
    count = @active_sprint.product.rss_entries.size
    sprint = {:title => "Do it!"}
    put :update, {:id => @active_sprint.id,:sprint => sprint, :selected => [@other_member.id]},
      {:user_id => @product.owner_id}

    assert_redirected_to sprints_path(:product => @product)
    result = Sprint.find_by_id(@active_sprint.id)
    assert_equal sprint[:title], result.title,
      "Sprint should have been updated."
    assert_equal 1, result.members.size, "Failed to populate the sprint."
    assert_equal @other_member.id, result.members.first.id, "Failed to populate the sprint correctly."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to add an rss entry."
  end

  # Ensures that updating with a specified return URL sends the browser back
  # to that URL.
  def test_update_with_url
    count = @active_sprint.product.rss_entries.size
    sprint = {:title => "Do it!"}
    put :update, {:id => @active_sprint.id, :sprint => sprint, :url => sprint_path(@active_sprint)},
      {:user_id => @product.owner_id}

    assert_redirected_to sprint_path(@active_sprint)
    result = Sprint.find_by_id(@active_sprint.id)
    assert_equal sprint[:title], result.title,
      "Sprint should have been updated."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to insert an RSS entry."
  end

  # Ensures anonymous users can't delete sprins.
  def test_delete_as_anonymous
    delete :destroy

    assert_redirected_to login_url
  end

  # Ensures that a sprint is required.
  def test_delete_with_invalid_sprint
    delete :destroy, { }, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can delete a sprint.
  def test_delete_as_nonowner
    delete :destroy, {:id => @active_sprint.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
    assert Sprint.find_by_id(@active_sprint.id),
      "Sprint should not have been deleted."
  end

  # Ensures that deletion works as expected.
  def test_delete
    count = @active_sprint.product.rss_entries.size
    delete :destroy, {:id => @active_sprint.id}, {:user_id => @product.owner_id}

    assert_redirected_to sprints_path(:product => @product.id)
    assert !Sprint.find_by_id(@active_sprint.id),
      "Sprint should have been deleted."
    assert BacklogItem.find_all_by_sprint_id(@active_sprint.id).empty?,
      "All backlog items should have been deleted for this sprint."
    product = Product.find_by_id(@active_sprint.product_id)
    assert_equal count + 1, product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that an anonymous user cannot move sprints to the planning state.
  def test_planning_as_anonymous
    put :planning

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint is required.
  def test_planning_with_invalid_sprint
    put :planning, {}, {:user_id => @plannable_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that only team leads can move a sprint to the planning state.
  def test_planning_as_non_team_lead
    put :planning, {:id => @plannable_sprint.id}, {:user_id => @non_team_lead.id}
  end

  # Ensures that a sprint that cannot be planned fails.
  def test_planning_with_state_failure
    put :planning, {:id => @active_sprint.id}, {:user_id => @active_sprint.team_lead_id}

    assert_redirected_to error_path
    result = Sprint.find_by_id(@active_sprint.id)
    assert !result.planning?, "Sprint should not have been put into the planning state."
  end

  # Ensures that a sprint can be moved to the planned state.
  def test_planning
    count = @plannable_sprint.product.rss_entries.size
    put :planning, {:id => @plannable_sprint.id}, {:user_id => @plannable_sprint.team_lead_id}

    assert_redirected_to sprint_path(@plannable_sprint)
    result = Sprint.find_by_id(@plannable_sprint.id)
    assert result.planning?, "Sprint should have been put into the planning state."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that anonymous users can't start a sprint.
  def test_start_as_anonymous
    put :start

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint id is required.
  def test_start_with_invalid_id
    put :start, {}, {:user_id => @planned_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that only the team lead can start a sprint.
  def test_start_as_non_team_lead
    put :start, {:id => @planned_sprint.id}, {:user_id => @non_team_lead.id }

    assert_redirected_to error_path
  end

  # Ensures that a sprint that can't be started results in an error.
  def test_start_with_nonstarter_sprint
    put :start, {:id => @plannable_sprint.id },{:user_id => @plannable_sprint.team_lead_id}

    assert_redirected_to sprint_path(@plannable_sprint)
  end

  # Ensures that only sprints with accepted items can be started.
  def test_start_with_rejected_items
    vote = @startable_sprint.backlog_items.first.votes.first
    vote.vote = BacklogItemVote::NAK
    vote.save

    put :start, {:id => @startable_sprint.id}, {:user_id => @startable_sprint.team_lead_id}

    assert_redirected_to sprint_path(@startable_sprint)
    result = Sprint.find_by_id(@startable_sprint.id)
    assert !result.active?, "The sprint should not have been started."
  end

  # Ensures that starting a sprint work.s
  def test_start
    count = @startable_sprint.product.rss_entries.size
    put :start, {:id => @startable_sprint.id}, {:user_id => @startable_sprint.team_lead_id}

    assert_redirected_to sprint_path(@startable_sprint)
    result = Sprint.find_by_id(@startable_sprint.id)
    assert result.active?, "Sprint should have been started."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that an anonymous user cannot complete a sprint.
  def test_complete_as_anonymous
    put :complete

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint id is required.
  def test_complete_with_invalid_sprint
    put :complete, {}, {:user_id => @active_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that only the team lead can complete a sprint.
  def test_complete_as_non_team_lead
    put :complete, {:id => @active_sprint.id}, {:user_id => @non_team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that a sprint that cannot be completed results in an error.
  def test_complete_for_noncompletable_sprint
    put :complete, {:id => @planned_sprint.id}, {:user_id => @planned_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that a sprint can be completed.
  def test_complete
    count = @active_sprint.product.rss_entries.size
    put :complete, {:id => @active_sprint.id}, {:user_id => @active_sprint.team_lead_id}

    assert_redirected_to sprint_path(@active_sprint)
    result = Sprint.find_by_id(@active_sprint.id)
    assert result.completed?, "Sprint was not moved to the completed state."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that anonymous users can't cancel a sprint.
  def test_cancel_as_anonymous
    put :cancel

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint id is required.
  def test_cancel_with_invalid_sprint
    put :cancel, {}, {:user_id => @active_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that only the team lead can cancel a sprint.
  def test_cancel_as_non_team_lead
    put :cancel, {:id => @active_sprint.id}, {:user_id => @non_team_lead.id}

    assert_redirected_to error_path
  end

  # Ensures that anonymous users can't plan.
  def test_plan_as_anonymous
    get :plan

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint id is required.
  def test_plan_with_invalid_id
    get :plan, {}, {:user_id => @planned_sprint.team_lead_id}

    assert_redirected_to error_path
  end

  # Ensures that cancelling a sprint works.
  def test_cancel
    count = @active_sprint.product.rss_entries.size
    put :cancel, {:id => @active_sprint.id}, {:user_id => @active_sprint.team_lead_id}

    assert_redirected_to sprint_path(@active_sprint)
    result = Sprint.find_by_id(@active_sprint.id)
    assert result.cancelled?, "Sprint should have been marked cancelled."
    assert_equal count + 1, result.product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that a valid sprint is required.
  def test_plan_with_invalid_sprint
    get :plan, { }, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can plan a sprint.
  def test_plan_as_nonowner
    get :plan, {:id => @planned_sprint.id}, {:user_id => @nonowner.id}

    assert_redirected_to sprint_path(@planned_sprint)
  end

  # Ensures that planning works.
  def test_plan
    get :plan, { :id => @planned_sprint.id}, {:user_id => @planned_sprint.product.owner_id}

    assert_response :success
    assert assigns['user_stories'], "Failed to load any user stories."
    assert assigns['selected'], "Failed to setup a selection map."
  end

  # Ensures an anonymous user can't populate a sprint.
  def test_populate_as_anonymous
    post :populate

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint is required.
  def test_populate_with_invalid_sprint
    post :populate, {}, {:user_id => @product.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that you can only populate a pending sprint.
  def test_populate_requires_pending_sprint
    post :populate, { :id => @active_sprint.id}, {:user_id => @active_sprint.product.owner_id}

    assert_redirected_to sprint_path(@active_sprint)
  end

  # Ensures that only the owner can populate a sprint.
  def test_populate_as_nonowner
    post :populate, { :id => @planned_sprint.id, :estimates => @estimates, :selected => @selected},
      {:user_id => @nonowner.id}

    assert_redirected_to sprint_path(@planned_sprint)
    assert !BacklogItem.find_by_sprint_id_and_user_story_id(@planned_sprint.id,@user_story.id),
      "Backlog item should not have been created"
  end

  # Ensures that populating with no user stories is not an error. BUG#92
  def test_populate_with_no_user_stories
    post :populate, { :id => @planned_sprint.id},{:user_id => @planned_sprint.product.owner_id}

    assert_redirected_to sprint_path(@planned_sprint)
  end

  # Ensures that populating works.
  def test_populate
    count = @planned_sprint.product.rss_entries.size
    post :populate, {:id => @planned_sprint.id, :selected => @selected},{:user_id => @planned_sprint.product.owner_id}

    assert_redirected_to sprint_path(@planned_sprint)
    assert BacklogItem.find_by_sprint_id_and_user_story_id(@planned_sprint.id,@user_story.id),
      "Backlog item should have been created"
    product = Product.find_by_id(@planned_sprint.product_id)
    assert_equal count + 1, product.rss_entries.size, "Failed to add an RSS entry."
  end

  # Ensures that a valid product is required.
  def test_members_with_invalid_product
    get :members

    assert_redirected_to error_path
  end

  # Ensures that a valid sprint is required.
  def test_members_with_invalid_sprint
    get :members

    assert_redirected_to error_path
  end

  # Ensures that the sprint members are viewable.
  def test_members
    get :members, { :id => @active_sprint.id}

    assert_response :success
    assert assigns['sprint'], "Failed to load a sprint."
    assert_equal @active_sprint.id, assigns['sprint'].id, "Failed to load the correct sprint."
  end

  # Ensures that anonymous users cannot vote on sprint backlog items.
  def test_vote_as_anonymous
    get :vote

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint is required.
  def test_vote_with_invalid_sprint
    get :vote, {}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only a team member can vote on sprint items.
  def test_vote_with_nonmember
    get :vote, {:id => @planned_sprint.id}, {:user_id => @nonmember.id}

    assert_redirected_to sprint_path(@planned_sprint)
  end

  # Ensures that only a sprint in planning can get votes.
  def test_vote_with_nonplanning_sprint
    get :vote, {:id => @active_sprint.id}, {:user_id => @member.id}

    assert_redirected_to sprint_path(@active_sprint)
  end

  # Ensures that the vote action works as expected.
  def test_vote
    get :vote, {:id => @planned_sprint.id}, {:user_id => @member.id}

    assert_response :success
    assert assigns['votes'], "Failed to setup the votes."
    assert assigns['estimates'], "Failed to setup the estimates."
  end

  # Ensures that anonymous users cannot cast votes.
  def test_cast_votes_as_anonymous
    post :cast_votes

    assert_redirected_to login_path
  end

  # Ensures that a valid sprint is required.
  def test_cast_votes_with_invalid_sprint
    post :cast_votes, {}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only members can cast votes.
  def test_cast_votes_as_nonmember
    count = BacklogItemVote.for_sprint(@planned_sprint).size
    post :cast_votes, {:id => @planned_sprint.id}, {:user_id => @nonmember.id}

    assert_redirected_to sprint_path(@planned_sprint)
    assert_equal count, BacklogItemVote.for_sprint(@planned_sprint).size, "No new votes should have been cast."
  end

  # Ensures that only sprints in planning can have votes.
  def test_cast_votes_for_nonplanning_sprint
    count = BacklogItemVote.for_sprint(@active_sprint).size
    post :cast_votes, {:id => @active_sprint.id}, {:user_id => @member.id}

    assert_redirected_to sprint_path(@active_sprint)
    assert_equal count, BacklogItemVote.for_sprint(@active_sprint).size, "No new votes should have been cast."
  end

  # Ensures that a vote cast for a non-existent backlog item fails.
  def test_cast_votes_for_nonexistent_backlog_item
    count = BacklogItemVote.for_sprint(@planned_sprint).size
    post :cast_votes, {:id => @planned_sprint.id, :votes => {9999 => BacklogItemVote::ACK}}, {:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal count, BacklogItemVote.for_sprint(@planned_sprint).size, "Vote should not have been cast."
  end

  # Ensures that a vote cast for an item on a different sprint fails.
  def test_cast_votes_for_item_on_different_sprint
    count = BacklogItemVote.for_sprint(@planned_sprint).size
    post :cast_votes, {:id => @planned_sprint.id, :votes => {@active_sprint.backlog_items.first.id => BacklogItemVote::ACK}}, {:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal count, BacklogItemVote.for_sprint(@planned_sprint).size, "Vote should not have been cast."
  end

  # Ensures that, when casting a positive vote, an estimate must be included.
  def test_cast_votes_accept_with_no_estimate
    post :cast_votes, {:id => @planned_sprint.id, :votes => @votes}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that casting votes works as expected.
  def test_cast_votes
    count = BacklogItemVote.for_sprint(@planned_sprint).size
    post :cast_votes, {:id => @planned_sprint.id, :votes => @votes, :estimates => @estimates}, {:user_id => @member.id}

    assert_redirected_to sprint_path(@planned_sprint)
    assert_equal count + 1, BacklogItemVote.for_sprint(@planned_sprint).size, "A new vote should have been cast."
    email = @emails.first
    assert email, "An email should have been sent to the team lead."
    assert_equal @planned_sprint.team_lead.email, email.to[0], "Email was not sent to the correct person."
  end
end
