# stories_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class StoriesControllerTest < ActionController::TestCase
  fixtures :products
  fixtures :user_stories
  fixtures :users

  def setup
    @product       = products(:projxp_web)
    @other_product = products(:projxp_web_services)

    @owner    = @product.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner can't be the same person." if @owner.id == @nonowner.id

    @story = user_stories(:freerange_user_story)
    raise "Story must use the right product." unless @story.product_id == @product.id
    raise "Story must be deletable." unless @story.can_be_deleted?

    @member = ProductRole.for_product(@story.product).approved.first.user
    raise "No members were found!" unless @member

    @nonmember = users(:celliot)
    raise "Non-member cannot have a product role!" if @story.product.is_member?(@nonmember)

    @story_with_backlog_items = user_stories(:create_login)
    raise "Story must use the right product." unless @story_with_backlog_items.product_id == @product.id
    raise "Story must have backlog items." unless BacklogItem.find_by_user_story_id(@story_with_backlog_items.id)

    @new_story = {
      :title    => 'This is a new user story',
      :priority => 17
    }

    @new_comment = {
      :body => "This is a comment."
    }
  end

  # Ensures that all stories by product are viewable.
  def test_index_for_product
    get :index, { :product => @product.id }

    assert_response :success
    assert_template 'stories/index'
    assert assigns['user_stories'], "Failed to load stories."
    assigns['user_stories'].each { |story| assert_equal @product.id, story.product_id, "Loaded stories for the wrong product." }
  end

  # Ensures that viewing all user stories works.
  def test_index
    get :index

    assert_response :success
    assert_template 'stories/index'
  end

  # Ensures that without a user story id, the user is sent back to the user
  # story list.
  def test_show_without_user_story_id
    get :show, {:product_id => @product.id}

    assert_redirected_to error_path
  end

  # Ensures that showing a user story works.
  def test_show
    get :show, {:id => @story.id}

    assert_response :success
    assert assigns['user_story'], "Failed to load a user story."
    assert_equal @story.id, assigns['user_story'].id,
      "Failed to load the correct user story."
  end

  # Ensures that an anonymous user cannot create a new user story.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_path
  end

  # Ensures that a product id is required to create a new story.
  def test_new_with_invalid_product_id
    get :new, { }, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the product owner can create a new user story.
  def test_new_as_nonowner
    get :new, {:product => @product.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that creating a new user story works as expected.
  def test_new
    get :new, {:product => @product.id}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['user_story'], "Failed to create a new user story."
    assert assigns['epics'], "Failed to load any epic stories for this project."
  end

  # Ensures that anonymous users can't edit a user story.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_url
  end

  # Ensures that a user story id is required.
  def test_edit_with_invalid_user_story_id
    get :edit, { }, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the product owner can edit a user story.
  def test_edit_as_nonowner
    get :edit, {:id => @story.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that editing a user story works as expected.
  def test_edit
    get :edit, { :id => @story.id}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['user_story'], "Failed to load a user story to edit."
    assert assigns['epics'], "Failed to load the set of epics."
    assert_equal @story.id, assigns['user_story'].id,
      "Failed to load the correct user story."
  end

  # Ensures that anonymous users can't create a new user story.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_path
  end

  # Ensures that a valid product id is required to create a user story.
  def test_create_with_invalid_product_id
    post :create, { }, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the product owner can create a user story.
  def test_create_as_nonowner
    post :create, {:product_id => @product.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
  end

  # Ensures that an invalid user story gets sent back to the edit page.
  def test_create_with_invalid_user_story
    post :create, {:product => @product.id, :user_story => {}}, {:user_id => @owner.id}

    assert_response :success
    assert assigns['user_story'].new_record?, "Should not have saved the user story."
  end

  # Ensures that creating a user story story works as expected.
  def test_create
    post :create, {:product => @product.id, :user_story => @new_story}, {:user_id => @owner.id}

    assert_redirected_to stories_path(:product => @product)
    assert UserStory.find_by_title(@new_story[:title]),
      "User story was not saved."
  end

  # Ensures that creating a user story with a URL returns to the URL.
  def test_create_with_source
    post :create, {:product => @product.id, :user_story => @new_story, :source => "/farkle"}, {:user_id => @owner.id}

    assert_redirected_to "/farkle"
    assert UserStory.find_by_title(@new_story[:title]), "User story was not saved."
  end

  # Ensures that specifying to add another the user is returned to the
  # new action
  def test_create_and_add_another
    post :create, {:product => @product.id, :user_story => @new_story, 'add_another' => 'yup'}, {:user_id => @owner.id}

    assert_redirected_to new_story_path(:product => @product)
    assert UserStory.find_by_title(@new_story[:title]), "User story was not saved."
  end

  # Ensures that anonymous users can't update stories.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_path
  end

  # Ensures that a valid user story id is required.
  def test_update_with_invalid_user_story_id
    put :update, { }, {:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the product owner can update a user story.
  def test_update_as_nonowner
    put :update, {  :id => @story.id, :user_story => {:title => 'Updated'}}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
    result = UserStory.find_by_id(@story.id)
    assert_equal @story.title, result.title, "User story should not have been updated."
  end

  # Ensures that an invalid user story gets sent back for editing.
  def test_update_with_invalid_user_story
    put :update, { :id => @story.id, :user_story => {:title => ''}}, {:user_id => @owner.id}

    assert_response :success
    result = UserStory.find_by_id(@story.id)
    assert_equal @story.title, result.title, "User story should not have been updated."
  end

  # Ensures that updates work as expected.
  def test_update
    update = {:title => "This ia the new title for an existing story"}
    put :update, { :id => @story.id, :user_story => update}, {:user_id => @owner.id}

    assert_redirected_to stories_path(:product => @product)
    result = UserStory.find_by_id(@story.id)
    assert_equal update[:title], result.title, "User story should have been updated."
  end

  # Ensures that updates return to a url when one is supplied.
  def test_update_with_source
    update = {:title => "This ia the new title for an existing story"}
    put :update,{ :id => @story.id, :user_story => update, :source => "/farkle"}, {:user_id => @owner.id}

    assert_redirected_to "/farkle"
    result = UserStory.find_by_id(@story.id)
    assert_equal update[:title], result.title, "User story should have been updated."
  end

  # Ensures anonymous users can't delete user stories.
  def test_destroy_as_anonymous
    delete :destroy

    assert_redirected_to login_url
  end

  # Ensures that destroy requires a valid user story id.
  def test_destroy_with_invalid_user_story_id
    delete :destroy, { },{:user_id => @owner.id}

    assert_redirected_to error_path
  end

  # Ensures that only the owner can delete a user story.
  def test_destroy_as_nonowner
    delete :destroy, { :id => @story.id}, {:user_id => @nonowner.id}

    assert_redirected_to error_path
    assert UserStory.find_by_id(@story.id), "User story should not have been deleted."
  end

  # Ensures that user stories associated with backlog items can't be deleted.
  def test_destroy_for_stories_with_backlog_items
    delete :destroy, { :id => @story_with_backlog_items.id}, {:user_id => @owner.id}

    assert_redirected_to error_path
    assert UserStory.find_by_id(@story_with_backlog_items.id), "User story should not have been deleted."
  end

  # Ensures that deleting a user story works as expected.
  def test_destroy
    delete :destroy, {:id => @story.id}, {:user_id => @owner.id}

    assert_redirected_to stories_path(:product => @product)
    assert !UserStory.find_by_id(@story.id), "User story should have been deleted."
  end

  # Ensures that deleting returns to the url if one is supplied.
  def test_destroy_with_source
    delete :destroy,{ :id => @story.id, :source => "/farkle"}, {:user_id => @owner.id}

    assert_redirected_to "/farkle"
    assert !UserStory.find_by_id(@story.id), "User story should have been deleted."
  end

  # Ensures that an anonymous user cannot comment on a story.
  def test_comment_as_anonymous
    post :comment

    assert_redirected_to login_path
  end

  # Ensures that a valid story is is required.
  def test_comment_with_invalid_id
    post :comment, {}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only product members can comment on stories.
  def test_comment_as_nonmember
    count = @story.comments.size
    post :comment, {:id => @story.id}, {:user_id => @nonmember.id}

    assert_redirected_to story_path(@story)
    result = UserStory.find_by_id(@story.id)
    assert_equal count, result.comments.size, "Comment should not have been posted."
  end

  # Ensures that a comment body is required.
  def test_comment_without_text
    count = @story.comments.size
    post :comment, {:id => @story.id}, {:user_id => @member.id}

    assert_template 'stories/show'
    result = UserStory.find_by_id(@story.id)
    assert_equal count, result.comments.size, "Comment should not have been posted."
  end

  # Ensures that a comment is posted properly.
  def test_comment
    count = @story.comments.size
    post :comment, {:id => @story.id, :message => @new_comment}, {:user_id => @member.id}

    assert_redirected_to story_path(@story)
    result = UserStory.find_by_id(@story.id)
    assert_equal count + 1, result.comments.size, "Comment should have been saved!"
  end
end
