# tasks_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class TasksControllerTest < ActionController::TestCase
  fixtures :backlog_items
  fixtures :tasks
  fixtures :users

  def setup
    @user = users(:mcpierce)
    @other_user = users(:jdonuts)
    raise "Users cannot be the same!" if @user.id == @other_user.id

    @task = tasks(:created_login_view)
    raise "Task must be owned by the user!" unless @task.primary_id == @user.id

    @item = @task.backlog_item
    raise "Item cannot be closed!" if @item.completed?

    @sprint = @item.sprint
    @product = @sprint.product

    @closed_item = backlog_items(:closed_backlog_item)
    raise "Item must be closed!" unless @closed_item.completed?

    @owner = @item.owner
    @nonowner = users(:jdonuts)
    raise "Owner and nonowner cannot be the same user." if @owner.id == @nonowner.id

    @nonmember = users(:jdonuts)
    raise "Non-member cannot be a member!" if @product.is_member?(@nonmember)

    @shared_task = tasks(:created_login_controller)
    raise "The task must have a primary and backup." unless @shared_task.primary && @shared_task.backup

    @new_task = {
      :backlog_item_id => @item.id,
      :primary_id      => @owner.id,
      :description     => 'Got something done',
      :hours           => 2.5,
      :when_entered    => Date.today
    }
  end

  # Ensures that specifying a user shows only that user's tasks.
  def test_index_with_user
    get :index, { :user_id => @user.id }

    assert_response :success
    assert_template 'tasks/index'
    assigns['tasks'].each { |task| assert_equal @user.id, task.primary_id, "Should not load tasks for other users." }
  end

  # Ensures that a list of the most recent tasks is displayed.
  def test_index
    get :index

    assert_response :success
    assert_template 'tasks/index'
  end

  # Ensures that a task is required.
  def test_show_without_task
    get :show

    assert_redirected_to error_path
  end

  # Ensures that show works.
  def test_show
    get :show, { :id => @task.id }

    assert_response :success
    assert_template 'tasks/show'
  end

  # Ensures anonymous users can't create tasks.
  def test_new_as_anonymous
    get :new

    assert_redirected_to login_path
  end

  # Ensures that a user is required.
  def test_new_with_invalid_user
    get :new, { :user => 9999 }, { :user_id => @user.id }

    assert_redirected_to error_path
  end

  # Ensures that a valid backlog item was specified.
  def test_new_with_invalid_backlog_item
    get :new, { :item => 9999 }, { :user_id => @user.id }

    assert_redirected_to error_path
  end

  # Ensures that only the backlog owner can add a task.
  def test_new_as_nonowner
    get :new, { :item => @item.id }, { :user_id => @nonowner.id }

    assert_redirected_to error_path
  end

  # Ensures that only an open item can get a task.
  def test_new_on_completed_item
    get :new, { :item => @closed_item.id }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensures that new works.
  def test_new
    get :new, { :item => @item.id }, { :user_id => @owner.id }

    assert_response :success
    assert_template 'tasks/new'
    assert assigns['users'], "Failed to load a list of users."
  end

  # Ensures anonymous users can't edit a task.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_path
  end

  # Ensures that a user is required.
  def test_edit_with_invalid_user
    get :edit, { }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensures that a backlog item is required.
  def test_edit_with_invalid_task_item
    get :edit, { }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensures the owner is the only one who can edit tasks.
  def test_edit_as_nonowner
    get :edit, { :id => @task.id }, { :user_id => @nonowner.id }

    assert_redirected_to error_path
  end

  # Ensures that editing works as expected.
  def test_edit
    get :edit, { :id => @task.id }, { :user_id => @owner.id }

    assert_response :success
    assert_template 'tasks/edit'
    assert assigns['users'], 'Failed to load the list of members.'
  end

  # Ensures anonymous users can't create tasks.
  def test_create_as_anonymous
    post :create

    assert_redirected_to login_path
  end

  # Ensures that a valid backlog item was specified.
  def test_create_with_invalid_item
    post:create, { }, { :user_id => @user.id }

    assert_redirected_to error_path
  end

  # Ensures that only the backlog owner can add a task.
  def test_create_as_nonowner
    post :create, { :item => @item.id }, { :user_id => @nonowner.id }

    assert_redirected_to  error_path
  end

  # Ensures that only product members can add tasks.
  def test_create_for_non_member
    @new_task[:backup_id] = @nonmember.id
    post :create, { :item => @item.id, :task => @new_task }, { :user_id => @owner.id }

    assert_redirected_to error_path
    assert !Task.find_by_description(@new_task[:description]),
      "Task should not have been saved!"
  end

  # Ensures that only an open item can get a task.
  def test_create_on_completed_item
    post :create, { :item => @closed_item.id }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensure that an invalid task returns the user to the edit page.
  def test_create_with_invalid_task
    @new_task[:hours] = nil
    post :create, { :item => @item.id, :task => @new_task }, { :user_id => @owner.id }

    assert_response :success
    assert_template 'tasks/new'
    result = Task.find_by_description(@new_task[:description])
    assert !result, "Task should not have been saved."
  end

  # Ensures that creating a task works as expected.
  def test_create
    count = @item.sprint.product.rss_entries.size
    post :create, { :item => @item.id, :task => @new_task }, { :user_id => @owner.id }

    result = Task.find_by_description(@new_task[:description])
    assert result, "Task should have been created."
    assert_redirected_to task_path(result)
    assert !result.backlog_item.completed?, "Backlog item should not have been marked completed."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "A new RSS entry should have been added."
  end

  # Ensures that creating a task that marks the item completed works as expected.
  def test_create_and_complete
    count = @item.sprint.product.rss_entries.size
    post :create, {:item => @item.id, :task => @new_task, :completed => true}, {:user_id => @owner.id}

    result = Task.find_by_description(@new_task[:description])
    assert result, "Task should have been created."
    assert_redirected_to task_path(result)
    assert result.backlog_item.completed?, "Item should have been marked as completed."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "A new RSS entry should have been added."
  end

  # Ensures anonymous users can't update a task.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_path
  end

  # Ensures that a task id is required.
  def test_update_with_invalid_task_id
    put :update, { }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensures the owner is the only one who can update tasks.
  def test_update_as_nonowner
    put :update, { :id => @task.id }, { :user_id => @nonowner.id }

    assert_redirected_to error_path
  end

  # Ensures that an invalid update returns to the edit view.
  def test_update_with_invalid_task
    put :update, { :id => @task.id, :task => { :hours => nil }}, { :user_id => @owner.id }

    assert_response :success
    assert_template 'tasks/edit'
    assert_equal @task.hours, Task.find_by_id(@task.id).hours,
      "Hours should not have been updated."
  end

  # Ensures that an update works.
  def test_update
    count = @item.sprint.product.rss_entries.size
    put :update, { :id => @task.id, :task => { :hours => 12.5 }}, { :user_id => @owner.id }

    assert_redirected_to task_path(@task)
    assert_equal 12.5, Task.find_by_id(@task.id).hours,
      "Hours hould have been updated."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "A new RSS entry should have been added."
  end

  # Ensures that anonymous users cannot delete tasks.
  def test_delete_as_anonymous
    delete :destroy

    assert_redirected_to login_path
  end

  # Ensures that a valid task id is required.
  def test_delete_with_invalid_task_id
    delete :destroy, { }, { :user_id => @owner.id }

    assert_redirected_to error_path
  end

  # Ensures that only the owner can delete a task.
  def test_delete_as_nonowner
    delete :destory, { :id => @task.id }, { :user_id => @nonowner.id }

    assert_redirected_to error_path
  end

  # Ensures that delete a task works as expected.
  def test_delete
    count = @item.sprint.product.rss_entries.size
    delete :destroy, { :id => @task.id }, { :user_id => @owner.id }

    assert_redirected_to tasks_path(:user => @owner.id)
    assert !Task.find_by_id(@task.id), "Task should have been deleted."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "A new RSS entry should have been added."
  end
end
