# user_items_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

# +UserItemsTest+ performs specific unit tests relating to user workflow with
# backlog items.
class UserItemsTest < ActionController::TestCase
  tests ItemsController

  fixtures :backlog_items
  fixtures :products
  fixtures :sprints
  fixtures :user_stories
  fixtures :users

  def setup
    @product = products(:projxp_web)
    @other_product = products(:projxp_web_services)
    raise "Product and other cannot be the same!" if @product.id == @other_product.id

    @owner = @product.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same person." if @owner.id == @nonowner.id

    @sprint  = sprints(:active_sprint)
    raise "Product and sprint are mismatched!" unless @product.id == @sprint.product_id

    @pending_sprint = sprints(:planned_sprint)
    raise "Sprint should be pending!" unless @pending_sprint.planning?

    @pending_item = backlog_items(:planned_sprint_backlog_item)
    raise "Pending item should be on a pending sprint." unless @pending_item.sprint_id == @pending_sprint.id

    @other_sprint = sprints(:planned_sprint)
    raise "Product and sprint are mismatched!" unless @product.id == @sprint.product_id

    @item = backlog_items(:owned_backlog_item)
    raise "Item and sprint are mismatched!" unless @item.sprint_id == @sprint.id

    @owned_item = @item
    raise "Item must be owned!" unless @owned_item.owner_id
    raise "Item does not belong to sprint!" unless @owned_item.sprint_id == @sprint.id

    @unowned_item = backlog_items(:unowned_backlog_item)
    raise "Item must not be owned!" if @unowned_item.owner_id
    raise "Item does not belong to sprint!" unless @unowned_item.sprint_id == @sprint.id

    @closed_item = backlog_items(:closed_backlog_item)
    raise "item must not be active!" unless @closed_item.completed?
    raise "Item does not belong to sprint!" unless @closed_item.sprint_id == @sprint.id

    @member = users(:mcpierce)
    raise "User is not a member!" unless @product.is_member?(@member)
    raise "User is not a member!" unless @other_product.is_member?(@member)

    @nonmember = users(:jdonuts)
    raise "User is a member!" if @product.is_member?(@nonmember)
    raise "user is a member!" if @other_product.is_member?(@nonmember)
  end

  # Ensures that anonymous users can't update estimaetes.
  def test_estimate_as_anonymous
    post :estimate

    assert_redirected_to login_path
  end

  # Ensures that a valid item id is required.
  def test_estimate_with_invalid_item
    post :estimate, {}, {:user_id => @owned_item.owner_id}

    assert_redirected_to error_path
  end

  # Ensures that an hours estimate is required.
  def test_estimate_without_hours
    post :estimate, {:id => @owned_item.id}, {:user_id => @owned_item.owner_id}

   assert_redirected_to error_path
   assert_equal @owned_item.remaining_hours, BacklogItem.find_by_id(@owned_item.id).remaining_hours,
      "Hours should not have been updated."
  end

  # Ensures that the sprint must be active for an estimate to be updated.
  def test_estimate_on_planned_sprint
    post :estimate, {:id => @pending_item.id}, {:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal @pending_item.remaining_hours, BacklogItem.find_by_id(@pending_item.id).remaining_hours,
      "Remaining hours should not have been updated!"
  end

  # Ensures that only the item owner can update the estimate hours.
  def test_estimate_as_nonowner
    post :estimate, {:id => @owned_item.id, :hours => @owned_item.remaining_hours + 1}, {:user_id => @nonmember.id}

    assert_redirected_to error_path
    assert_equal @owned_item.remaining_hours, BacklogItem.find_by_id(@owned_item.id).remaining_hours,
      "Remaining hours should not have been updated."
  end

  # Ensures that updating an estimate works as expected.
  def test_estimate
    count = @owned_item.sprint.product.rss_entries.size
    post :estimate, {:id => @owned_item.id, :hours => 1.7}, {:user_id => @owned_item.owner_id}

    assert_redirected_to item_path(@owned_item)
    assert_equal 1.7, BacklogItem.find_by_id(@owned_item.id).remaining_hours.to_f,
      "Hours should have been updated."
    newcount = Product.find_by_id(@owned_item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "A new RSS entry should have been added."
  end

  # Ensures that you have to be logged in.
  def test_accept_as_anonymous
    get :accept

    assert_redirected_to login_path
  end

  # Ensures that a valid product is present.
  def test_accept_with_invalid_product
    get :accept, {}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that a backlog item is present.
  def test_accept_with_invalid_backlog_item
    get :accept, {}, {:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only members can accept items.
  def test_accept_as_nonmember
    get :accept, {:id => @unowned_item.id},{:user_id => @nonmember.id}

    assert_redirected_to error_path
    assert !BacklogItem.find_by_id(@unowned_item.id).owner_id,
      "Owner should not have been assigned."
  end

  # Ensures that a user can accept an owned item.
  def test_accept_of_owned_item
    get :accept, {:id => @owned_item.id}, {:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal @owned_item.owner_id, BacklogItem.find_by_id(@owned_item.id).owner_id,
      "Owner should not have changed."
  end

  # Ensures that only active sprint items can be accepted.
  def test_accept_from_planned
    get :accept, {:id => @pending_item.id}, {:user_id => @member.id}

    assert_redirected_to error_path
    assert !BacklogItem.find_by_id(@pending_item.id).owner_id,
      "Owner should not have been assigned."
  end

  # Ensures that accepting works as expected.
  def test_accept
    count = @unowned_item.sprint.product.rss_entries.size
    get :accept,{:id => @unowned_item.id},{:user_id => @member.id}

    assert_redirected_to item_path(@unowned_item)
    result = BacklogItem.find_by_id(@unowned_item.id)
    assert_equal @member.id, result.owner_id, "Owner should have been assigned."
    assert result.assigned?, "Item should be marked as assigned."
    newcount = Product.find_by_id(@unowned_item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end

  # Ensures an anonymous user can't drop an item.
  def test_drop_as_anonymous
    get :drop

    assert_redirected_to login_path
  end

  # Ensures that a backlog item is present.
  def test_drop_with_invalid_backlog_item
    get :drop,{},{:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only active sprint items can be dropped.
  def test_drop_from_planned
    get :accept, {:id => @pending_item.id},{:user_id => @member.id}

    assert_redirected_to error_path
    assert !BacklogItem.find_by_id(@pending_item.id).owner_id,
      "Owner should not have been changed."
  end

  # Ensures that dropping works as expected.
  def test_drop
    count = @owned_item.sprint.product.rss_entries.size
    get :drop, {:id => @owned_item.id},{:user_id => @member.id}

    assert_redirected_to items_path(:sprint => @sprint)

    result = BacklogItem.find_by_id(@owned_item.id)
    assert !result.owner_id,"Owner should have been removed."
    assert result.pending?, "Items hould be marked as pending."
    newcount = Product.find_by_id(@owned_item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end

  # Ensures an anonymous user can't complete an item.
  def test_complete_as_anonymous
    get :complete

    assert_redirected_to login_path
  end

  # Ensures that a backlog item is present.
  def test_complete_with_invalid_backlog_item
    get :complete,{},{:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that only active sprint items can be completed.
  def test_complete_from_planned
    get :complete, {:id => @pending_item.id},{:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal BacklogItem::STATE_PENDING, BacklogItem.find_by_id(@pending_item.id).state,
      "State should not have changed."
  end

  # Ensures that completing works as expected.
  def test_complete
    count = @owned_item.sprint.product.rss_entries.size
    get :complete,{:id => @owned_item.id}, {:user_id => @owned_item.owner_id}

    assert_redirected_to items_path(:sprint => @sprint)
    result = BacklogItem.find_by_id(@owned_item.id)
    assert result.completed?, "Item should be marked as completed."
    newcount = Product.find_by_id(@owned_item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end

  # Ensures an anonymous user can't reopen an item.
  def test_reopen_as_anonymous
    get :reopen

    assert_redirected_to login_path
  end

  # Ensures that the sprint must belong to the product.
  def test_reopen_with_product_sprint_mismatch
    get :reopen,{:sprint_id => @sprint.id},{:user_id => @member.id}

    assert_redirected_to error_path
  end

  # Ensures that an item must be closed to be reopened.
  def test_reopen_with_open_item
    get :reopen,{:id => @owned_item.id},{:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal @owned_item.state, BacklogItem.find_by_id(@owned_item.id).state,
      "Item state should not have changed."
  end

  # Ensures that only active sprint items can be reopened.
  def test_reopen_from_planned
    get :reopen,{:id => @pending_item.id},{:user_id => @member.id}

    assert_redirected_to error_path
    assert_equal @pending_item.state, BacklogItem.find_by_id(@pending_item.id).state,
      "State should not have changed."
  end

  # Ensures that reopening works as expected.
  def test_reopen
    count = @closed_item.sprint.product.rss_entries.size
    get :reopen,{:id => @closed_item.id},{:user_id => @member.id}

    assert_redirected_to item_path(@closed_item)
    result = BacklogItem.find_by_id(@closed_item.id)
    assert_nil result.owner_id, "Should not have been reassigned."
    assert result.pending?, "Item should be marked as pending."
    newcount = Product.find_by_id(@item.sprint.product_id).rss_entries.size
    assert_equal count + 1, newcount, "An RSS entry should have been created for this event."
  end
end
