# users_controller_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class UsersControllerTest < ActionController::TestCase
  fixtures :notifications
  fixtures :user_verifications
  fixtures :users

  def setup
    @user = users(:mcpierce)
    @other_user = users(:jdonuts)
    raise "User and other can't be the same!" if @user.id == @other_user.id

    @new_user = {
      :email                 => 'test@projxp.org',
      :display_name          => 'New User',
      :password              => 'test',
      :password_confirmation => 'test',
      :introduction          => 'Yo, wassup?'
    }

    @nonadmin = users(:mcpierce)
    fail "User cannot have admin rights!" if @nonadmin.create_users?

    @admin = users(:admin)
    fail "Admin must have admin rights!" unless @admin.create_users?

    @verified_user = users(:mcpierce)
    raise "User must be verified!" unless @verified_user.verified?

    @unverified_user = users(:unverified_user)
    raise "User must not be verified!" if @unverified_user.verified?

    ActionMailer::Base.delivery_method = :test
    ActionMailer::Base.perform_deliveries = true
    ActionMailer::Base.deliveries = []

    @emails = ActionMailer::Base.deliveries
    @emails.clear
  end

  # Ensures that anybody can view a list of users.
  def test_index
    get :index

    assert_response :success
    assert assigns['users'], "Failed to load a list of users."
  end

  # Ensures that a valid user must be supplied.
  def test_show_with_invalid_user
    get :show

    assert_redirected_to users_path
  end

  # Ensures that showing a user works.
  def test_show
    get :show, {:id => @user.id}

    assert_response :success
    assert assigns['backlog_items'], "Failed to load the user's backlog."
  end

  # Ensures that anonymous users can't edit accounts.
  def test_edit_as_anonymous
    get :edit

    assert_redirected_to login_path
  end

  # Ensures that a valid user is sent.
  def test_edit_with_invalid_user
    get :edit, {}, {:user_id => @user.id}

    assert_redirected_to users_path
  end

  # Ensures that an admin can edit a user.
  def test_edit_as_admin
    get :edit, {:id => @user.id}, {:user_id => @admin.id}

    assert_response :success
    assert assigns['this_user'], "Failed to load user."
    assert_equal @user.id, assigns['this_user'].id,
      "Failed to load the right user."
  end

  # Ensures that only the user can edit his account.
  def test_edit_as_other_user
    get :edit, {:id => @user.id}, {:user_id => @other_user.id}

    assert_redirected_to users_path
  end

  # Ensures that only an anonymous user can create a new account.
  def test_new_as_nonanonymous
    get :new, {}, {:user_id => @user.id}

    assert_redirected_to users_path
  end

  # Ensures that an anonymous user starts the registration process.
  def test_new
    get :new

    assert_response :success
    assert assigns['this_user'], "Failed to create a new user object."
  end

  # Ensures that anoymous users can create a new account.
  def test_create_as_nonanonymous
    post :update, {}, {:user_id => @user.id}

    assert_redirected_to users_path
    email = @emails.first
    assert !email, "No email should have been generated."
  end

  # Ensures that an admin can create a new user.
  def test_create_as_admin
    post :create, {:user => @new_user}, {:user_id => @admin.id}

    assert_redirected_to user_path(assigns['this_user'])
    assert User.find_by_email(@new_user[:email]),
      "A new user should have been created."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @new_user[:email], email.to[0], "Email sent to wrong user."
  end

  # Ensures that a bad account gets sent back for editing.
  def test_create_with_bad_info
    post :create, {:user => {:email => 'test@here.now'}}, {:user_id => @admin.id}

    assert_response :success
    assert_template 'edit'
    assert !User.find_by_email('test@here.now'),
      "A new user should not have been created."
    email = @emails.first
    assert !email, "No email should have been generated."
  end

  # Ensures that a newly created user gets a validation email.
  def test_create
    post :create, {:user => @new_user}, {:user_id => @admin.id}

    assert_redirected_to user_path(assigns['this_user'])
    result = User.find_by_email(@new_user[:email])

    assert result, "A new user should have been created."
    assert result.privileges, "No privileges were created."
    assert result.notifications, "No notifications were created."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @new_user[:email], email.to[0], "Email sent to wrong user."
  end

  # Ensures that an anonymous user can't update an account.
  def test_update_as_anonymous
    put :update

    assert_redirected_to login_path
  end

  # Ensures that a user can't update someone else's account.
  def test_update_as_other_user
    put :update,
      {:id => @user.id, :user => {:email => "test@fodder.com"}},
      {:user_id => @other_user.id}

    assert_redirected_to users_path
    assert_not_equal "test@fodder.com", User.find_by_id(@user.id).email,
      "User should not have been updated."
  end

  # Ensures admins can update a user's account.
  def test_update_as_admin
    put :update,
      {:id => @user.id, :user => {:display_name => "Don Farkle"}},
      {:user_id => @admin.id}

    assert_redirected_to user_path(@user)
    assert_equal "Don Farkle", User.find_by_id(@user.id).display_name,
      "User should have been updated."
  end

  # Ensures users can update their accounts.
  def test_update
    put :update,
      {:id => @user.id, :user => {:display_name => "Joe Farkles"}},
      {:user_id => @user.id}

    assert_redirected_to user_path(@user)
    assert_equal "Joe Farkles", User.find_by_id(@user.id).display_name,
      "User should have been updated."
  end

  # Ensures an email is sent to the user if they change their email.
  def test_update_with_email_change
    put :update,
      {:id => @user.id, :user => {:email => "test@fodder.com"}},
      {:user_id => @user.id}

    assert_redirected_to user_path(@user)
    assert_equal "test@fodder.com", User.find_by_id(@user.id).email,
      "User should have been updated."
    email = @emails.first
    assert email, "No email was generated."
    assert_equal @user.email, email.to[0], "Email sent to wrong user."
  end

  # Ensures that a valid user must be supplied for a backlog.
  def test_backlog_with_invalid_user
    get :backlog

    assert_redirected_to users_path
  end

  # Ensures that the backlog's loaded for the given user.
  def test_backlog
    get :backlog, {:id => @user.id}

    assert_response :success
    assert assigns['backlog_items'], "Failed to load the backlog."
  end

  # Ensures that a valid id must be present.
  def test_verify_as_anonymous
    get :verify

    assert_redirected_to login_path
  end

  # Ensures that a user must be specified.
  def test_verify_with_invalid_user
    get :verify, {}, {:user_id => @unverified_user.id}
    assert_redirected_to users_path
  end

  # Ensures that authenticated user and the user to be verified are the same.
  def test_verify_with_user_mismatch
    get :verify, {:id => @unverified_user.id}, {:user_id => @verified_user.id}

    assert_redirected_to user_path(@verified_user)
  end

  # Ensures that the user must be unverified.
  def test_verify_with_verified_user
    get :verify, {:id => @verified_user.id}, {:user_id => @verified_user.id}

    assert_redirected_to user_path(@verified_user)
  end

  # Ensures that a token must be present.
  def test_verify_without_token
    get :verify, {:id => @unverified_user.id}, {:user_id => @unverified_user.id}

    assert_response :success
    assert !User.find_by_id(@unverified_user.id).verified?,
      "User should not be verified."
  end

  # Ensures that a valid token is present.
  def test_verify_with_invalid_token
    get :verify,
      {:id => @unverified_user.id, :token => @unverified_user.verification.token.reverse},
      {:user_id => @unverified_user.id}

    assert_response :success
    assert !User.find_by_id(@unverified_user.id).verified?,
      "User should not be verified."
  end

  # Ensures that a user can properly resend a verification token.
  def test_verify_with_resend_request
    post :verify,
      {:id => @unverified_user.id, :resend => true},
      {:user_id => @unverified_user.id}

    assert_response :success
    assert !User.find_by_id(@unverified_user.id).verified?,
      "User should not be verified."

    email = @emails.first
    assert email, "No email was generated."
    assert_equal @unverified_user[:email], email.to[0], "Email sent to wrong user."
  end

  # Ensures that verification works as expected.
  def test_verify
    post :verify,
      {:id => @unverified_user.id, :token => @unverified_user.verification.token},
      {:user_id => @unverified_user.id}

    assert_redirected_to user_path(@unverified_user)
    assert User.find_by_id(@unverified_user.id).verified?,
      "User should be verified."
  end

  # Ensures that viewing roles requires a valid user id.
  def test_roles_with_invalid_user_id
    get :roles

    assert_redirected_to users_path
  end

  # Ensures that the set of roles are loaded.
  def test_roles
    get :roles, {:id => @user.id}

    assert_response :success
    assert assigns['product_roles'], "Failed to load the user's roles."
  end

  # Ensures that notifications updates require the user be logged in.
  def test_notifications_as_anonymous
    post :notifications

    assert_redirected_to login_path
  end

  # Ensures anonymous users can't change passwords.
  def test_change_password_as_anonymous
    post :change_password

    assert_redirected_to login_path
  end

  # Ensures that a valid user's required.
  def test_notifications_with_invalid_user
    post :notifications, {}, {:user_id => @user.id}

    assert_redirected_to users_path
  end

  # Ensures that a valid user must be supplied.
  def test_change_password_with_invalid_user
    post :change_password, {}, {:user_id => @user.id}

    assert_redirected_to users_path
  end

  # Ensures that a user cannot edit someone else's notifications.
  def test_notifications_for_other_user
    post :notifications, {:id => @other_user.id}, {:user_id => @user.id}

    assert_redirected_to user_path(@other_user)
  end

  # Ensures that only the user can change his password.
  def test_change_password_for_other_user
    post :change_password, {:id => @other_user.id}, {:user_id => @user.id}

    assert_redirected_to user_path(@other_user)
  end

  # Ensures that notifications are updated.
  def test_notifications
    post :notifications,
      {:id => @user.id, :notifications => {:task_reminders => true}},
      {:user_id => @user.id}

    assert_redirected_to user_path(@user)
    assert Notifications.find_by_user_id(@user.id).task_reminders,
      "Notification update should've been saved."
  end

  # Ensures that the current password has to match.
  def test_change_password_with_wrong_current_password
    post :change_password,
      {:id => @user.id, :old_password => "garbledina"},
      {:user_id => @user.id}

    assert_redirected_to password_user_path(@user)
  end

  # Ensures that the new passwords have to valid.
  def test_change_password_with_incorrect_passwords
    post :change_password,
      {:id => @user.id, :old_password => "farkle",
      :user => {:password => "a", :password_confirmation => "b" }},
      {:user_id => @user.id}

    assert_response :success
    assert !User.authenticate(@user.email, "a"), "Password should not have been updated."
  end

  # Ensures that a valid password update works as expected.
  def test_change_password
    post :change_password,
      {:id => @user.id, :old_password => "farkle",
      :user => {:password => "visible", :password_confirmation => "visible"}},
      {:user_id => @user.id}

    assert_redirected_to user_path(@user)
    assert User.authenticate(@user.email, "visible"),
      "Password was not properly updated."
  end
end
