# backlog_item_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class BacklogItemTest < ActiveSupport::TestCase
  fixtures :backlog_items
  fixtures :sprints
  fixtures :user_stories
  fixtures :users

  def setup
    @sprint = sprints(:active_sprint)
    @team_lead = @sprint.team_lead
    @product = @sprint.product

    @non_team_lead = users(:mcpierce)
    raise "Non-team lead cannot be the team lead!" if @non_team_lead.id == @team_lead.id

    @new_item = BacklogItem.new(
      :sprint_id     => @sprint.id,
      :user_story_id => user_stories(:create_login).id)

    @owned_item = backlog_items(:owned_backlog_item)
    raise "Owned item must have an owner!" unless @owned_item.owner
    @owner = @owned_item.owner

    @unowned_item = backlog_items(:unowned_backlog_item)
    raise "Unowned items must not have an owner!" if @unowned_item.owner
    raise "Unowned item must have estimates!" if @unowned_item.votes.empty?

    @closed_item = backlog_items(:closed_backlog_item)
    raise "Closed items must be closed!" unless @closed_item.completed?

    @member = users(:mcpierce)
    raise "Member is not a sprint team member!" unless @sprint.is_team_member?(@member)

    @nonmember = users(:verified_user)
    raise "Nonmember cannot be on the sprint team!" if @sprint.is_team_member?(@nonmember)
    raise "Nonmember must be on the product team!" unless @product.is_member?(@nonmember)
    raise "Nonmember cannot be the team lead!" if @sprint.team_lead_id == @nonmember.id

    @item_on_planned_sprint = backlog_items(:planned_sprint_backlog_item)
    raise "Item should not have an owner!" if @item_on_planned_sprint.owner
    raise "That sprint must no be active!" if @item_on_planned_sprint.sprint.active?

    @item_on_closed_sprint = backlog_items(:closed_sprint_backlog_item)
    raise "Item must have an owner!" unless @item_on_closed_sprint.owner
    raise "That sprint must be closed!" unless @item_on_closed_sprint.sprint.completed?

    @deferrable_item = backlog_items(:owned_backlog_item)
  end

  # Ensures that a sprint is required.
  def test_valid_fails_without_sprint
    @new_item.sprint_id = nil

    flunk "A sprint is required." if @new_item.valid?
  end

  # Ensures that a user story is required.
  def test_valid_fails_without_user_story
    @new_item.user_story_id = nil

    flunk "A user story is required." if @new_item.valid?
  end

  # Ensures a well-formed story passes validation.
  def test_valid
    flunk "Something's fundamentally wrong." unless @new_item.valid?
  end

  # Ensures that the default value for remaining hours is the original estimated
  # hours.
  def test_remaining_when_undefined
    assert_equal @unowned_item.votes.first.estimated_hours, @unowned_item.remaining_hours,
    'Remaining hours should equal estimated hours.'
  end

  # Ensures that adding a new remaining hours estimation affects the hours
  # reported.
  def test_remaining_hours_when_updated
    @new_item.remaining_hours_estimates << RemainingHoursEstimate.new(:hours => 2.5, :estimated_on => Date.today.next)

    assert_equal 2.5, @new_item.remaining_hours, 'Remaining hours is incorrectly reported.'
  end

  # Ensure a non-member cannot accept a backlog item.
  def test_account_as_nonmember
    assert !@unowned_item.user_can_accept?(@nonmember),
    "Non-members cannot accept backlog items."
  end

  # Ensures that a user can accept an unowned backlog item.
  def test_accept_an_unowned_item
    assert @unowned_item.user_can_accept?(@member),
      "Users must be able to accept backlog items."
  end

  # Ensures that a user cannot accept an item on a non-active sprint.
  def test_accept_on_item_on_planned_sprint
    assert !@item_on_planned_sprint.user_can_accept?(@owner),
      "Users must not be allowed to accept items on planned sprints."
  end

  # Ensures that a user cannot accept an already accepted backlog item.
  def test_accept_on_owned_backlog_item
    assert !@owned_item.user_can_accept?(@owner),
      "Users cannot accept an owned backlog item."
  end

  # Ensures that a user can drop a backlog item on an active sprint.
  def test_drop_on_backlog_item_on_closed_sprint
    assert !@item_on_closed_sprint.user_can_drop?(@owner),
      "Users cannot drop an item on a closed sprint."
  end

  # Ensures that a non-owner can't drop an owned sprint.
  def test_drop_for_nonowner
    assert !@owned_item.user_can_drop?(@nonowner),
      "Someone who doens't own a backlog item can't drop it!"
  end

  # Ensures that nobody can drop an unowned backlog item.
  def test_drop_for_unowned_item
    assert !@unowned_item.user_can_drop?(@owner),
      "You can't drop a backlog item that's not owned!"
  end

  # Ensures that the owner can drop a backlog item.
  def test_drop
    assert @owned_item.user_can_drop?(@owner),
      "The owern has to be able to drop a backlog item."
  end

  # Ensures that not just anybody can reopen a backlog item
  def test_reopen_by_anybody
    assert !@closed_item.user_can_reopen?(@nonmember)
  end

  # Ensures that the item owner can reopen a backlog item.
  def test_reopen_by_owner
    assert @closed_item.user_can_reopen?(@closed_item.owner)
  end

  # Ensures that a team lead can reopen a backlog item.
  def test_team_lead_user_can_reopen
    team_lead = @closed_item.sprint.team_lead

    assert @closed_item.user_can_reopen?(team_lead),
    "Team leads must be able to reopen completed tasks."
  end

  # Ensures that an item can only be reopened on an active sprint.
  def test_user_can_reopen_on_planned_sprint
    fail "You cannot reopen an item on a closed sprint." if @item_on_closed_sprint.user_can_reopen?(@item_on_closed_sprint.owner)
  end

  # Ensures that only the owner can mark an item as blocked.
  def test_mark_blocked_as_nonowner
    fail "Only the owner can mark an item as blocked." if @owned_item.can_mark_blocked?(@nonowner)
  end

  # Ensures that a closed item cannot be marked as blocked.
  def test_mark_blocked_for_closed_item
    fail "Only open items can be marked as blocked." if @closed_item.can_mark_blocked?(@closed_item.owner)
  end

  # Ensures that an item must be on an active sprint to be marked as blocked.
  def test_mark_blocked_for_planned_sprint
    fail "Only items on active sprints can be marked as blocked." if @item_on_closed_sprint.can_mark_blocked?(@item_on_closed_sprint.owner)
  end

  # Ensures marking an item as blocked is allowed.
  def test_mark_blocked
    fail "An item must allow the owner to mark it as blocked." unless @owned_item.can_mark_blocked?(@owner)
  end

  # Ensures that only the team lead can defer an item.
  def test_defer_as_non_team_lead
    fail "Only the team lead can defer an item." if @deferrable_item.user_can_defer?(@non_team_lead)
  end

  # Ensures that an item not on an active sprint cannot be deferred.
  def test_defer_for_planned_sprint_item
    fail "Defer is only for active sprints." if @item_on_planned_sprint.user_can_defer?(@item_on_planned_sprint.sprint.team_lead)
  end

  # Ensures that an owned item can be deferred
  def test_defer_for_owned_items
    fail "Items should be deferrable by the team lead." unless @owned_item.user_can_defer?(@owned_item.sprint.team_lead)

    @owned_item.defer
    assert @owned_item.deferred?, "Item should have been marked as deferred."
    assert !@owned_item.owner, "Deferred items cannot have owners."
  end

  # Ensures that completed items cannot be deferred.
  def test_defer_for_completed_items
    fail "Closed items cannot be deferred." if @closed_item.user_can_defer?(@closed_item.sprint.team_lead)
  end

  # Ensures that items can be deferred.
  def test_defer
    fail "Items should be deferrable by the team lead." unless @unowned_item.user_can_defer?(@unowned_item.sprint.team_lead)

    @unowned_item.defer
    assert @unowned_item.deferred?, "Item should have been deferred."
  end

  # Ensures that an item on a proposed sprint with a NAK is not considered accepted.
  def test_accepted_fails_with_nak
    vote = @item_on_planned_sprint.votes.first
    vote.vote = BacklogItemVote::NAK
    vote.save

    assert !@item_on_planned_sprint.accepted?, "An item with a NAK cannot be accepted."
  end

  # Ensures that an item that has no rejections but only abstains is not considered accepted.
  def test_accepted_fails_when_abstained_only
    @item_on_planned_sprint.votes.each {|vote| vote.vote = BacklogItemVote::ABSTAIN; vote.save}

    assert !@item_on_planned_sprint.accepted?, "An item with only abstains cannot be accepted."
  end

  # Ensures that a single abstain does not cause an item to be rejected.
  def test_accepted_passes_with_abstain_and_ack
    vote = @item_on_planned_sprint.votes.first
    vote.vote = BacklogItemVote::ABSTAIN
    vote.save

    assert @item_on_planned_sprint.accepted?, "An item should be accepted when it has ACKs and abstains."
  end

  # Ensures that an accepted backlog item is reported as such.
  def test_accepted
    assert @item_on_planned_sprint.accepted?, "An accepted item should report itself as such."
  end
end
