# config_property_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class ConfigPropertyTest < ActiveSupport::TestCase
  def setup
    @newproperty = ConfigProperty.new(:name => 'name', :value => 'value')
    @oldproperty = config_properties(:system_name)
  end

  # Ensures that the name is required.
  def test_name_required
    @newproperty.name = nil

    fail "A name is required." if @newproperty.valid?
  end

  # Ensures that the name is required to be unique.
  def test_name_unique
    @newproperty.name = @oldproperty.name

    fail "The name must be unique." if @newproperty.valid?
  end

  # Ensures that a value is not required.
  def test_value_not_required
    @newproperty.value = nil

    fails "The value is not required." unless @newproperty.valid?
  end

  # Ensures that a well-formed property is valid.
  def test_valid
    fail "There is a problem with validation." unless @newproperty.valid?
  end

  # Ensures that a name must be provided when fetching a property.
  def test_fetch_without_name
    assert_raises(Exception) { ConfigProperty.fetch(nil) }
  end

  # Ensures that retrieving an existing value works.
  def test_fetch_for_existing
    result = ConfigProperty.fetch(@oldproperty.name, nil)

    assert result
    assert_equal @oldproperty.value, result, "returned the wrong value"
  end

  # Ensures that the default value's returned when no such property is found.
  def test_fetch_with_default_value
    result = ConfigProperty.fetch(@oldproperty.name.reverse, "default value")

    assert result
    assert_equal "default value", result, "Did not return the default value."
  end

  # Ensures that a name is required when saving a property.
  def test_store_without_name
    assert_raises(Exception) { ConfigProperty.store(nil) }
  end

  # Ensures that a new configuration property is saved property.
  def test_store_new_property
    assert_nil ConfigProperty.fetch(@oldproperty.name.reverse), "Whoa! That shouldn't exist!"

    ConfigProperty.store(@oldproperty.name.reverse, "farkle")
    result = ConfigProperty.fetch(@oldproperty.name.reverse)

    assert_equal "farkle", result, "Property was not property saved."
  end

  # Ensures that an existing property is overwritten.
  def test_store
    ConfigProperty.store(@oldproperty.name, "farkle")

    result = ConfigProperty.fetch(@oldproperty.name)
    assert_equal "farkle", result, "Property was not overwritten."
  end
end
