# user_story_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class EpicTest < ActiveSupport::TestCase
  fixtures :epics
  fixtures :projects
  fixtures :user_stories
  fixtures :users

  def setup
    @project = projects(:projxp)
    @owner = @project.owner
    @nonowner = users(:mcpierce)
    raise "Nonowner cannot be the owner!" if @nonowner.id == @owner.id

    @new_epic = Epic.new(:project => @project, :priority => 1, :title => "This is an epic")
    @epic = epics(:login_epic)
    raise "Epic must be part of the project!" unless @epic.project_id == @project.id
    raise "Epic must be open!" if @epic.closed

    @member = users(:mcpierce)
    raise "Member must have a product member!" unless @epic.project.is_member?(@member)

    @nonmember = users(:celliot)
    raise "Non-member cannot be a part of any product team!" if @epic.project.is_member?(@nonmember)

    @closed_epic = epics(:closed_epic)
    raise "Epic must be closed!" unless @closed_epic.closed

    @epic_with_open_stories = epics(:user_stories_epic)
    raise "Epic must have open stories!" if UserStory.for_epic(@epic_with_open_stories).open.empty?

    @epic_with_no_stories = epics(:unused_epic)
    raise "This epic should not have user stories!" unless @epic_with_no_stories.user_stories.empty?
  end

  # Ensures the project_id field is required.
  def test_valid_fails_without_project_id
    @new_epic.project = nil
    fail "Project must be required for an epic." if @new_epic.valid?
  end

  # Ensures that a priority is required.
  def test_valid_fails_without_priority
    @new_epic.priority = nil
    fail "Priority must be required for an epic." if @new_epic.valid?
  end

  # Ensures that priority is a number.
  def test_valid_fails_with_nonnumeric_priority
    @new_epic.priority = 'a'
    fail "Priority must be a number." if @new_epic.valid?
  end

  # Ensures that a priority must be a positive integer.
  def test_valid_fails_with_negative_integer_priority
    @new_epic.priority = -1
    fail "Priority cannot be negative." if @new_epic.valid?
  end

  # Ensures that a priority cannot be 0.
  def test_valid_fails_with_zero_priority
    @new_epic.priority = 0
    fail "Priority cannot be zero." if @new_epic.valid?
  end

  # Ensures that a title is required.
  def test_valid_fails_without_title
    @new_epic.title = nil
    fail "A title must be provided." if @new_epic.valid?
  end

  # Ensures that a title cannot be an empty string.
  def test_valid_fails_with_empty_title
    @new_epic.title = ""
    fail "A title cannot be 0 bytes long." if @new_epic.valid?
  end

  # Ensures that a well-formed epic is valid.
  def test_valid
    fail "There is a general problect with validation." unless @new_epic.valid?
  end

  # Ensures that non-admins cannot edit epics.
  def test_can_edit_as_nonadmin
    flunk "Non-admins cannot edit epics." if @epic.can_edit?(@nonowner)
  end

  # Ensures that admins can edit an epic.
  def test_can_edit
    flunk "Admins must be able to edit epics." unless @epic.can_edit?(@owner)
  end

  # Ensures that non-admins cannot close epics.
  def test_can_close_as_nonadmin
    flunk "Non-admins cannot close epics." if @epic.can_close?(@nonadmin)
  end

  # Ensures that a closed epic cannot be closed again.
  def test_can_close_when_already_closed
    flunk "Closed epics cannot be closed again." if @closed_epic.can_close?(@owner)
  end

  # Ensures that an epic with open user stories cannot be closed.
  def test_can_close_with_open_user_stories
    flunk "Cannot close epics with open user stories." if @epic_with_open_stories.can_close?(@owner)
  end

  # Ensures that an epic with no user stories cannot be closed.
  def test_can_close_fails_with_no_user_stories
    flunk "Cannot close an epic with no user stories." if @epic_with_no_stories.can_close?(@owner)
  end

  # Ensures that an epic can be closed by the admin.
  def test_can_close
    flunk "Admins must be able to close epics." unless @epic.can_close?(@owner)
  end

  # Ensures that an epic with no user stories can be deleted.
  def test_can_delete_with_no_user_stories
    flunk "Admins must be able to delete epics." unless @epic_with_no_stories.can_delete?(@owner)
  end

  # Ensures that closed epics cannot be reopened.
  def test_can_reopen_an_open_epic
    flunk "Users cannot reopen an open epic." if @epic.can_reopen?(@owner)
  end

  # Ensures non-admins cannot reopen epics.
  def test_can_reopen_as_nonadmin
    flunk "Non-admins cannot reopen epics." if @closed_epic.can_reopen?(@nonadmin)
  end

  # Ensures reopening an epic works as expected..
  def test_can_reopen
    flunk "Closed tasks should be able to reopen." unless @closed_epic.can_reopen?(@owner)
  end

  # Ensures that an anonymous user cannot comment on epics.
  def test_can_comment_fails_for_anonymous
    flunk "Anonymous users cannot comment on epics." if @epic.can_comment?(nil)
  end

  # Ensures that users who aren't members of a project's products teams cannot comment.
  def test_can_comment_as_nonmember
    flunk "Non-product members cannot comment on epics." if @epic.can_comment?(@nonmember)
  end

  # Ensures that members can comment on epics.
  def test_can_comment
    flunk "Members must be able to comment on epics." unless @epic.can_comment?(@member)
  end
end
