# project_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class ProductTest < ActiveSupport::TestCase
  fixtures :products
  fixtures :projects
  fixtures :users

  def setup
    @project = projects(:projxp)

    @new_product = Product.new(
      :project_id   => @project.id,
      :owner_id     => @project.owner.id,
      :name         => "Test Product",
      :mailing_list => 'test@projxp.org')

    @existing_product  = products(:projxp_web)
    @owner    = @existing_product.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same user!" if @owner.id == @nonowner.id

    @product_with_logo = products(:projxp_web)
    raise "Product must have a logo!" unless @product_with_logo.logo_name
    @product_without_logo = products(:projxp_web_services)
    raise "Product must not have a logo!" if @product_without_logo.logo_name

    @product_with_no_stories = products(:teatime_midp)
    raise "Product must not have stories!" unless @product_with_no_stories.user_stories.empty?

    @product_with_planned_sprint = products(:projxp_web_services)
    raise "Product must have at least one planned sprint!" if Sprint.for_product(@product_with_planned_sprint).planned.empty?
  end

  # Ensures that a project is required.
  def test_valid_fails_without_project
    @new_product.project = nil

    flunk "Products must have a project." if @new_product.valid?
  end

  # Ensures that an owner has to be set.
  def test_valid_fails_without_owner
    @new_product.owner = nil

    flunk "Products must have an owner." if @new_product.valid?
  end

  # Ensures that a mailing list must be well formed if present.
  def test_valid_fails_with_bad_mailing_list
    @new_product.mailing_list = 'farkle'

    flunk "Products must have a well-formatted mailing list." if @new_product.valid?
  end

  # Ensures that a mailing list is not required.
  def test_valid_without_mailing_list
    @new_product.mailing_list = ""

    flunk "A mailing list is not required." unless @new_product.valid?
  end

  # Ensures that a well-formed product is valid.
  def test_valid
    flunk "There is a general validation problem." unless @new_product.valid?
  end

  # Ensures that a non-owner cannot modify a product.
  def test_user_can_modify_with_nonowner
    flunk "Non-owners cannot modify products!" if @existing_product.can_edit?(@nonowner)
  end

  # Ensures that the product owner can modify a product.
  def test_user_can_modify
    flunk "Owners can modify products!" unless @existing_product.can_edit?(@owner)
  end

  # Ensures that a product with no logo uses its parent project's logo.
  def test_logo_url_for_product_with_no_logo
    assert_equal @product_without_logo.project.logo_url, @product_without_logo.logo_url,
    "Product should return the project's logo."
  end

  # Ensures that the logo url is returned.
  def test_logo_url
    assert_equal "#{ConfigProperty.fetch('image.product_logo_path')}/#{@product_with_logo.logo_name}",
    @product_with_logo.logo_url, "Did not return the correct logo url."
  end

  # Ensures that a product with no user stories won't allow sprints.
  def test_can_create_with_no_user_stories
    flunk "The owner should not be allowed to create sprints when there are no stories." if
      @product_with_no_stories.can_create_sprints?(@product_with_no_stories.owner)
  end

  # Ensures that a new sprint cannot be created if an existing one exists in the planned state.
  def test_can_create_sprint_with_existing_planned_sprint
    flunk "You cannot have two planned sprints." if
      @product_with_planned_sprint.can_create_sprints?(@product_with_planned_sprint.owner)
  end
end
