# project_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class ProjectTest < ActiveSupport::TestCase
  fixtures :projects
  fixtures :users

  def setup
    @project   = projects(:projxp)
    @unapproved_project = projects(:unapproved_project)
    raise "Unapproved project cannot be approved!" if @unapproved_project.approved

    @owner = @project.owner
    @nonowner = users(:mcpierce)
    raise "Owner and nonowner cannot be the same person!" if @owner.id == @nonowner.id

    @admin = users(:project_admin)
    raise "Admin must have project admin rights!" unless @admin.privileges.admin_projects
    @nonadmin = users(:mcpierce)
    raise "Nonadmin must not have project admin rights!" if @nonadmin.privileges.admin_projects

    @member = ProductRole.for_product(@project.products.first).approved.first.user
    raise "No product member found!" unless @member

    @nonmember = users(:celliot)
    Product.for_project(@project).each {|product| raise "Nonmember cannot have product role!" if product.is_member?(@nonmember)}

    @project_with_logo = projects(:projxp)
    raise "Project must have a logo!" unless @project_with_logo.logo_name
    @project_without_logo = projects(:teatime)
    raise "Project must not have a logo!" if @project_without_logo.logo_name

    @new_project = Project.new(
      :name     => "New Projects",
      :owner_id => @nonowner.id,
      :url      => 'http://localhost/')
  end

  # Ensures that a project must have a name.
  def test_valid_fails_without_name
    @new_project.name = ''

    flunk 'A project must have a name.' if @new_project.valid?
  end

  # Ensures that a project must have a unique name.
  def test_valid_fails_without_unique_name
    @new_project.name = @project.name

    flunk 'A project must have a unique name.' if @new_project.valid?
  end

  # Ensures that a project requires an owner.
  def test_valid_fails_without_owner
    @new_project.owner = nil

    flunk 'A project must have an owner.' if @new_project.valid?
  end

  # Ensures that a well-formed project is considered valid.
  def test_valid
    flunk "There is a general problem with validation." unless @new_project.valid?
  end

  # Ensures that a non-approved project cannot have producgts.
  def test_create_products_for_unapproved_projects
    flunk "Unapproved projects cannot have products." if @unapproved_project.can_create_products?(@owner)
  end

  # Ensures non-owners cannot create products.
  def test_create_products_for_nonowners
    flunk "Non-owners should not be able to create products!" if @project.can_create_products?(@nonowner)
  end

  # Ensures that a project owner can create products.
  def test_create_products_for_project_owner
    flunk "A project owner should be able to create products." unless @project.can_create_products?(@owner)
  end

  # Ensures that non admins cannot approve projects.
  def test_nonadmin_cannot_approve_projects
    flunk "Non-admins must not be able to approve projects." if @unapproved_project.can_approve?(@nonadmin)
  end

  # Ensures that a user with project admin rights can approve projects.
  def test_admin_can_approve_projects
    flunk "Admins must be able to approve projects." unless @unapproved_project.can_approve?(@admin)
  end

  # Ensures that an unapproved project cannot have epics.
  def test_create_epics_for_unapproved_project
    flunk "Unapproved projects cannot have epics." if @unapproved_project.can_create_epics?(@owner)
  end

  # Ensures that non-admins cannot create epics.
  def test_create_epics_for_nonadmins
    flunk "Non-admins cannot create epics." if @project.can_create_epics?(@nonowner)
  end

  # Ensures that creating epics checks is fine.
  def test_create_epics
    flunk "A project owner can create epics." unless @project.can_create_epics?(@owner)
  end

  # Ensures that a project with no logo returns the default logo.
  def test_logo_url_for_project_with_no_logo
    expected = ConfigProperty.fetch("image.default_project_logo")
    assert_equal expected, @project_without_logo.logo_url, "The default URL is not correct."
  end

  # Ensures that anonymous users aren't listed as members.
  def test_is_member_for_anonymous
    flunk "Anonymous users are not members." if @project.is_member?(nil)
  end

  # Ensures that users who are not members of a project's products are not listed as members.
  def test_is_member_for_nonmember
    flunk "This user is not a member!" if @project.is_member?(@nonmember)
  end

  # Ensures that membership is determined correctly.
  def test_is_member
    flunk "This user is a member!" unless @project.is_member?(@member)
  end
end
