# sprint_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class SprintTest < ActiveSupport::TestCase
  def setup
    @new_sprint = Sprint.new(:product   => products(:projxp_web),
                             :title     => "Inaugural sprint",
                             :starts    => Date.today,
                             :duration  => 14,
                             :goals     => "To get stuff done!",
                             :team_lead => users(:mcpierce)
                             )
    @planned_sprint = sprints(:planned_sprint)
    raise "Sprint must be in planning!" unless @planned_sprint.planning?
    raise "Product must have an active sprint!" if Sprint.for_product(@planned_sprint.product).active.empty?

    @active_sprint = sprints(:active_sprint)
    raise "Sprint must be active!" unless @active_sprint.active?
    raise "Product must have a planned sprint!" if Sprint.for_product(@active_sprint.product).planned.empty?

    @completed_sprint = sprints(:completed_sprint)
    raise "Sprint must be completed!" unless @completed_sprint.completed?
    raise "Product must have an active sprint!" if Sprint.for_product(@completed_sprint.product).active.empty?

    @cancelled_sprint = sprints(:cancelled_sprint)
    raise "Sprint must be cancelled!" unless @cancelled_sprint.cancelled?

    @startable_sprint = sprints(:startable_sprint)
    raise "Sprint must be planned." unless @startable_sprint.planning?
    raise "Product must not have an active sprint." unless Sprint.for_product(@startable_sprint).active.empty?

    @startable_completed_sprint = sprints(:startable_completed_sprint)
    raise "Sprint must be completed!" unless @startable_completed_sprint.completed?
    raise "Product must not have an active sprint." unless Sprint.for_product(@startable_completed_sprint.product).active.empty?

    @startable_cancelled_sprint = sprints(:startable_cancelled_sprint)
    raise "Sprint must be cancelled!" unless @startable_cancelled_sprint.cancelled?
    raise "Product must not have an active sprint." unless Sprint.for_product(@startable_cancelled_sprint.product).active.empty?
  end

  # Ensures a product id is required.
  def test_valid_fails_without_product_id
    @new_sprint.product_id = nil
    flunk "A sprint must require a product." if @new_sprint.valid?
  end

  # Ensures a title is required.
  def test_valid_fails_without_title
    @new_sprint.title = nil
    flunk "A sprint must require a title." if @new_sprint.valid?
  end

  # Ensures a title cannot be an empty string.
  def test_valid_fails_for_empty_title
    @new_sprint.title = ''
    flunk "A sprint title cannot be empty." if @new_sprint.valid?
  end

  # Ensures a start date is required.
  def test_valid_fails_without_start_date
    @new_sprint.starts = nil
    flunk "A sprint must have a start date." if @new_sprint.valid?
  end

  # Ensures a duration is required.
  def test_valid_fails_without_duration
    @new_sprint.duration = nil
    flunk "A sprint must have a duration." if @new_sprint.valid?
  end

  # Ensures a duration must be a positive integer.
  def test_valid_fails_with_negative_duration
    @new_sprint.duration = -1
    flunk "A sprint must have a positive duration." if @new_sprint.valid?
  end

  # Ensures a sprint requires set of goals.
  def test_valid_fails_without_goals
    @new_sprint.goals = nil
    flunk "A sprint must have goals." if @new_sprint.valid?
  end

  # Ensures that goals must have content.
  def test_valid_fails_with_empty_goals
    @new_sprint.goals = ''
    flunk "A sprint must define goals." if @new_sprint.valid?
  end

  # Ensures that a team lead is defined.
  def test_valid_fails_without_team_lead_id
    @new_sprint.team_lead_id = nil
    flunk "A sprint must have a team lead." if @new_sprint.valid?
  end

  # Ensures that a well-formed sprint is valid.
  def test_valid
    unless @new_sprint.valid?
      puts @new_sprint.errors.full_messages
      flunk "There is a general validation error."
    end
  end

  # Ensures that nobody can edit a completed sprint.
  def test_can_edit_with_completed_sprint
    flunk "Completed sprints must not be edited by the team lead." if @completed_sprint.can_edit?(@completed_sprint.team_lead)
    flunk "Completed sprints must not be edited by the product owner." if @completed_sprint.can_edit?(@completed_sprint.product.owner)
  end

  # Ensures that nobody can edit a cancelled sprint.
  def test_can_edit_with_cancelled_sprint
    flunk "Cancelled sprints must not be edited by the team lead." if @cancelled_sprint.can_edit?(@cancelled_sprint.team_lead)
    flunk "Cancelled sprints must not be edited by the product owner." if @cancelled_sprint.can_edit?(@cancelled_sprint.product.owner)
  end

  # Ensures the team lead can edit an active sprint.
  def test_can_edit_as_team_lead
    flunk "Team lead must be able to edit an active sprint." unless @active_sprint.can_edit?(@active_sprint.team_lead)
  end

  # Ensures that only one planned sprint is allowed.
  def test_plan_fails_if_there_is_a_planned_sprint
    flunk "There's already a planning sprint." if @active_sprint.can_plan?
  end

  # Ensures that only one sprint may be active.
  def test_start_fails_if_there_is_an_active_sprint
    flunk "There's already an active sprint." if @planned_sprint.can_start?
    flunk "There's already an active sprint." if @completed_sprint.can_start?
    flunk "There's already an active sprint." if @cancelled_sprint.can_start?
  end

  # Ensures that a planned sprint can be started.
  def test_start
    flunk "A sprint must be startable." unless @startable_sprint.can_start?
  end

  # Ensures that an active sprint can be cancelled.
  def test_cancel
    flunk "A sprint must be cancellable." unless @active_sprint.can_cancel?
  end

  # Ensures that an active sprint can be completed.
  def test_complete
    flunk "A sprint must be completable." unless @active_sprint.can_complete?
  end

  # Ensures that a completed sprint can be started.
  def test_start_for_completed
    flunk "A completed sprint must be startable." unless @startable_completed_sprint.can_start?
  end

  # Ensures that a cancelled sprint can be started.
  def test_start_for_cancelled
    flunk "A cancelled sprint must be startable." unless @startable_cancelled_sprint.can_start?
  end

  # Ensures that a sprint is not accepted if it has a rejected backlog item.
  def test_accept_fails_with_rejected_item
    vote = @planned_sprint.backlog_items.first.votes.first
    vote.vote = BacklogItemVote::NAK
    vote.save

    fail "A sprint with a NAK cannot be accepted." if @planned_sprint.accepted?
  end

  # Ensures that a sprint is accepted if all of its backlog items are accepted.
  def test_accept
    fail "This sprint should report itself as accepted." unless @planned_sprint.accepted?
  end
end
