# task_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'

class TaskTest < ActiveSupport::TestCase
  fixtures :backlog_items
  fixtures :sprints
  fixtures :users

  def setup
    @backlog_item = backlog_items(:owned_backlog_item)
    @new_task = Task.new(
      :primary_id      => @backlog_item.owner_id,
      :description     => 'This is a task',
      :hours           => 5,
      :when_entered => @backlog_item.sprint.starts + (@backlog_item.sprint.duration / 2))
    @new_task.backlog_item = @backlog_item
    @backup = users(:jdonuts)

    raise "Backup cannot be the owner!" if @backup.id == @backlog_item.owner.id
  end

  # Ensures that at least a primary must be defined.
  def test_valid_fails_without_primary
    @new_task.primary = nil

    flunk "A primary developer must be set." if @new_task.valid?
  end

  # Ensures that the primary and backup developer cannot be the same person.
  def test_valid_fails_if_primary_is_backup
    @new_task.backup = @new_task.primary

    flunk "The primary developer cannot be the backup." if @new_task.valid?
  end

  # Ensures that a backlog item is required.
  def test_valid_fails_without_backlog_item
    @new_task.backlog_item = nil

    flunk "A backlog item must be defined." if @new_task.valid?
  end

  # Ensures that the description must be present.
  def test_valid_fails_without_description
    @new_task.description = ""

    flunk "A description must be provided." if @new_task.valid?
  end

  # Ensures that hours must be provided.
  def test_valid_fails_without_hours
    @new_task.hours = nil

    flunk "Hours must be specified." if @new_task.valid?
  end

  # Ensures that hours cannot be negative.
  def test_valid_fails_with_negative_hours
    @new_task.hours = -3

    flunk "Hours must be a positive value." if @new_task.valid?
  end

  # Ensures that hours cannot be zero.
  def test_valid_fails_with_zero_for_hours
    @new_task.hours = 0

    flunk "Hours cannot be zero." if @new_task.valid?
  end

  # Ensures that a date must be present.
  def test_valid_fails_without_date
    @new_task.when_entered = nil

    flunk "Date must be set." if @new_task.valid?
  end

  # Ensures that the date for a task can't be before the sprint starts.
  def test_valid_fails_when_date_preceeds_sprint
    @new_task.when_entered = @backlog_item.sprint.starts - 1

    flunk "Date cannot be before the sprint starts." if @new_task.valid?
  end

  # Ensures that the date for a task can't be after the sprint ends.
  def test_valid_fail_when_date_exceeds_sprint
    @new_task.when_entered = @backlog_item.sprint.starts + @backlog_item.sprint.duration

    flunk "Date cannot be after the sprint ends." if @new_task.valid?
  end

  # Ensures that a well-formed task if valid.
  def test_valid
    flunk "Something is wrong with validation." unless @new_task.valid?
  end
end
