# user_test.rb
# Copyright (C) 2008, Darryl L. Pierce <mcpierce@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

require File.dirname(__FILE__) + '/../test_helper'
require 'avatar'

class UserTest < ActiveSupport::TestCase
  fixtures :users
  def setup
    @mcpierce = users(:mcpierce)

    @verified_user = @mcpierce
    raise "User should be verified!" unless @verified_user.verified?
    @unverified_user = users(:unverified_user)

    @user_with_no_avatar = users(:admin)
    raise "User should not have an avatar!" unless !@user_with_no_avatar.use_gravatar && @user_with_no_avatar.url_for_avatar.nil?
    @user_with_avatar = users(:celliot)
    raise "User should have an avatar!" if @user_with_avatar.url_for_avatar.nil?
    @user_with_gravatar = @mcpierce
    raise "User does not use gravatar!" unless @user_with_gravatar.use_gravatar
    @user_with_special_gravatar = users(:jdonuts)
    raise "User is not using gravatar!" unless @user_with_special_gravatar.use_gravatar
    raise "User does not have a special avatar URL!" if @user_with_special_gravatar.avatar_url.nil?
  end

  # Ensures that authenticating a user using the wrong password fails.
  def test_authenticate_with_wrong_password
    result = User.authenticate(@mcpierce.email, 'klefar')

    assert_nil result
  end

  # Ensures that trying to verify a token for a user with no pending verification
  # raises an exception.
  def test_verify_for_verified_user
    assert_raise(Exception) { @verified_user.verify(UserVerification.create_token) }
  end

  # Ensures that giving the wrong token does not verified the user.
  def test_verify_with_wrong_token
    flunk "User should not have verified the token." if @unverified_user.verify(UserVerification.create_token)
  end

  # Ensures that an unverified user is marke as unverified.
  def test_verified_with_unverified_user
    flunk "User is not verified." if @unverified_user.verified?
  end

  # Ensures that a user with no avatar returns the right value when queries.
  def test_avatar_reports_no
    flunk "User should not say he has an avatar." if @user_with_no_avatar.avatar?
  end

  # Ensures that a user with an avatar URL returns the right URL.
  def test_avatar_url
    expected = @user_with_avatar.url_for_avatar

    assert_equal expected, @user_with_avatar.avatar_url,
    "Wrong avatar URL returned."
  end

  # Ensures that a user with a gravatar image returns the right url.
  def test_gravatar_url
    expected = gravatar_url(@user_with_gravatar.email)

    assert_equal expected, @user_with_gravatar.avatar_url,
    "Wrong avatar URL returned."
  end

  # Ensures that a user with a special gravatar image gets the right URL.
  def test_gravatar_special_url
    expected = gravatar_url(@user_with_special_gravatar.url_for_avatar)

    assert_equal expected, @user_with_special_gravatar.avatar_url,
    "Wrong avatar URL returned."
  end

  # Ensures that a user with no avatar or gravatar icon gets the default.
  def test_avatar_with_no_avatar
    expected = ConfigProperty.fetch("image.default_user_avatar")

    assert_equal expected, @user_with_no_avatar.avatar_url,
    "Wrong default avatar URL returned."
  end
end
